/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.car.impl;

import static java.lang.annotation.ElementType.PACKAGE;

import java.lang.annotation.ElementType;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.ow2.util.archive.api.IArchive;
import org.ow2.util.ee.metadata.car.api.ICarMetadata;
import org.ow2.util.ee.metadata.common.api.struct.IEnvEntry;
import org.ow2.util.scan.api.metadata.CommonMetadata;
import org.ow2.util.scan.api.metadata.IClassMetadata;
import org.ow2.util.scan.api.metadata.structures.IMember;

/**
 * Metadata of a client archive.
 * @author Gael Lalire
 */
public class CarMetadata extends CommonMetadata implements ICarMetadata {

    /**
     * List of class annotations metadata.
     */
    private Map<String, IClassMetadata> carClassMetadataMap = null;

    /**
     * The car archive.
     */
    private final IArchive archive;

    /**
     * envEntryCollection.
     */
    private final Collection<? extends IEnvEntry> envEntryCollection;

    /**
     * Metadata-complete element has been set to true or false.
     */
    private boolean isMetadacomplete = false;

    /**
     * Constructor.
     * @param carDeployable the client archive deployable
     */
    public CarMetadata(final IArchive archive) {
        this.archive = archive;
        carClassMetadataMap = new HashMap<>();
        envEntryCollection = new ArrayList<>();
    }

    /**
     * Add annotation metadata for a given class.
     * @param carClassMetadata metadata of a class.
     */
    @Override
    public void addClassMetadata(final IClassMetadata carClassMetadata) {
        String key = carClassMetadata.getJClass().getName();
        // already exists ?
        if (carClassMetadataMap.containsKey(key)) {
            throw new IllegalStateException("addCarClassMetadata : key already exists");
        }
        carClassMetadataMap.put(key, carClassMetadata);
    }

    /**
     * Get class annotation metadata.
     * @param className key of the map of annotations bean.
     * @return Bean annotation metadata of a given name.
     */
    @Override
    public IClassMetadata getClassMetadata(final String className) {
        return carClassMetadataMap.get(className);
    }

    /**
     * Get collections of bean annotation metadata.
     * @return collections of bean annotation metadata.
     */
    @Override
    public Collection<IClassMetadata> getClassMetadataCollection() {
        return carClassMetadataMap.values();
    }

    /**
     * @return the web archive
     */
    public IArchive getArchive() {
        return archive;
    }

    /**
     * @return true if metadata-complete flag is enabled on the deployable.
     */
    public boolean isMetadataComplete() {
        return isMetadacomplete;
    }

    /**
     * Allow to define if this metadata is annotation completed or not.
     */
    public void setMetadataComplete(final boolean complete) {
        this.isMetadacomplete = complete;
    }

    @Override
    public IMember getMember() {
        return null;
    }

    @Override
    protected ElementType getElementType() {
        return PACKAGE;
    }

}
