/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.car.impl;

import java.net.URL;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.ow2.util.archive.api.ArchiveException;
import org.ow2.util.archive.api.IArchive;
import org.ow2.util.ee.metadata.car.api.ICarMetadata;
import org.ow2.util.ee.metadata.car.api.ICarMetadataFactory;
import org.ow2.util.ee.metadata.car.api.ICarView;
import org.ow2.util.ee.metadata.car.api.exceptions.CARMetadataException;
import org.ow2.util.ee.metadata.car.impl.configurator.CarSessionConfigurator;
import org.ow2.util.ee.metadata.car.impl.xml.merge.CarMetadataMerge;
import org.ow2.util.ee.metadata.car.impl.xml.parsing.CARDeploymentDescLoader;
import org.ow2.util.ee.metadata.car.impl.xml.struct.CAR;
import org.ow2.util.ee.metadata.common.impl.xml.parsing.ParsingException;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.scan.api.IClassesLocator;
import org.ow2.util.scan.api.IScanner;
import org.ow2.util.scan.api.ScanException;
import org.ow2.util.scan.api.classlocator.ArchiveClassesLocator;
import org.ow2.util.scan.api.classlocator.ClassLoaderClassesLocator;

/**
 * Factory to contruct car deployable metadata.
 * @author Gael Lalire
 */
public class CarMetadataFactory implements ICarMetadataFactory {

    /**
     * Logger.
     */
    private static final Log LOGGER = LogFactory.getLog(CarMetadataFactory.class);

    /**
     * Application-client DD entry name.
     */
    private static final String APPLICATION_CLIENT = "META-INF/application-client.xml";

    private static final String CLASS_SUFFIX = ".class";

    private final IScanner scanner;

    /**
     * Constructor with an archive scanner factory.
     * @param scanner a class scanner
     */
    public CarMetadataFactory(final IScanner scanner) {
        this.scanner = scanner;
    }

    /**
     * Create war metadata.
     * @param archive the war archive
     * @return the metadata
     * @throws CARMetadataException if error occurs
     */
    @Override
    public ICarMetadata createArchiveMetadata(final IArchive archive) throws CARMetadataException {
        return createArchiveMetadata(archive, (ClassLoader) null);
    }

    /**
     * Create war metadata.
     * @param archive the war archive
     * @param classLoader a classloader to use if a class is not in archive
     * @return the metadata
     * @throws WARMetadataException if error occurs
     */
    @Override
    public ICarMetadata createArchiveMetadata(final IArchive archive, final ClassLoader classLoader)
            throws CARMetadataException {
        List<? extends IClassesLocator> classesLocatorList = null;
        if (classLoader != null) {
            classesLocatorList = Arrays.asList(new ClassLoaderClassesLocator(classLoader));
        } else {
            classesLocatorList = Arrays.asList(new ClassLoaderClassesLocator(ClassLoader.getSystemClassLoader()));
        }
        return createArchiveMetadata(archive, classesLocatorList);
    }

    /**
     * Extract a class list from xml.
     * @param car representation of application-client.xml
     * @param classNameSet a set to fill with classes name referenced in application-client.xml
     */
    private void completeClassList(final CAR car, final Set<String> classNameSet) {
        if (car != null) {
            addClassToSetIfNotNull(car.getCallbackHandler(), classNameSet);
        }

        //TODO: complete with all elements like all ejb-ref, resource-ref, etc
    }

    /**
     * Create car deployable metadata.
     * @param deployable the car deployable
     * @param carExternLibraries extern libraries
     * @param specificDeployableFactoryList a specific deployable factory list
     * @return the metadata
     * @throws DeployableMetadataException if error occurs
     */
    @Override
    public ICarMetadata createArchiveMetadata(final IArchive archive,
            final List<? extends IClassesLocator> carExternLibraries)
            throws CARMetadataException {

        // for time debugging
        long tAnalyzeStart = System.currentTimeMillis();

        CAR car = null;
        URL applicationClientURL;
        try {
            applicationClientURL = archive.getResource(APPLICATION_CLIENT);
        } catch (ArchiveException e) {
            throw new CARMetadataException("Unable to get DD URL", e);
        }

        if (applicationClientURL != null) {
            try {
                car = CARDeploymentDescLoader.loadDeploymentDescriptor(applicationClientURL);
            } catch (ParsingException e) {
                throw new CARMetadataException("Unable to parse DD URL", e);
            }
        }

        CarSessionConfigurator carSessionConfigurator = new CarSessionConfigurator(archive);


        Set<String> classNameSet = new HashSet<String>();

        // if metadata is complete then filter class to scan
        if (car != null && car.isMetadataComplete()) {
            // disable annotation parsing
            carSessionConfigurator.setAnnotationParsingDesactived(true);

        } else {
            // enable annotation parsing because metadata descriptor is not
            // complete
            carSessionConfigurator.setAnnotationParsingDesactived(false);
        }

        // FIXME: this should be done only if metadata is completed
        Iterator<String> entryIterator;
        try {
            entryIterator = archive.getEntries();
        } catch (ArchiveException e) {
            throw new CARMetadataException(e);
        }
        while (entryIterator.hasNext()) {
            String entry = entryIterator.next();
            if (entry.endsWith(CLASS_SUFFIX)) {
                classNameSet.add(entry.substring(0, entry.length() - CLASS_SUFFIX.length()));
            }
        }
        // END FIXME


        // get class name to scan from deployment descriptor
        completeClassList(car, classNameSet);

        // do annotation scan
        try {
            this.scanner.scan(classNameSet, new ArchiveClassesLocator(archive), carExternLibraries, carSessionConfigurator);
        } catch (ScanException e) {
            throw new CARMetadataException("Unable to scan", e);
        }

        // get result
        ICarMetadata carMetadata = carSessionConfigurator.getCarMetadata();

        // Default is "not completed"
        if (car != null) {
            ICarView view = carMetadata.as(ICarView.class);
            //view.setMetadataComplete(car.isMetadataComplete());
        }

        // do merge with DD
        if (car != null) {
            new CarMetadataMerge(carMetadata).merge(car);
        }

        // time if debugging
        if (LOGGER.isDebugEnabled()) {
            long tAnalyzeStartEnd = System.currentTimeMillis();
            LOGGER.debug("Analyze of file {0} took {1} ms.", archive.getName(), (tAnalyzeStartEnd - tAnalyzeStart));
        }

        return carMetadata;
    }


    protected void addClassToSetIfNotNull(final String className, final Set<String> stringSet) {
        if (className != null) {
            stringSet.add(className.replace('.', '/'));
        }
    }

}
