/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.car.impl.enc;

import java.util.Collection;
import java.util.List;

import org.ow2.util.ee.metadata.car.api.ICarMetadata;
import org.ow2.util.ee.metadata.common.api.struct.IJAnnotationResource;
import org.ow2.util.ee.metadata.common.api.struct.IJavaxPersistenceContext;
import org.ow2.util.ee.metadata.common.api.struct.IJavaxPersistenceUnit;
import org.ow2.util.ee.metadata.common.api.struct.IJaxwsWebServiceRef;
import org.ow2.util.ee.metadata.common.impl.enc.AbsENCBindingBuilder;
import org.ow2.util.ee.metadata.common.impl.enc.ENCBindingException;
import org.ow2.util.ee.metadata.common.impl.enc.ENCBindingHolder;
import org.ow2.util.ee.metadata.common.impl.view.CommonView;
import org.ow2.util.scan.api.metadata.IClassMetadata;

/**
 * Utility class that helps to build the ENC Binding by analyzing metadata.
 * @author Florent BENOIT
 */
public class CarENCBindingBuilder extends AbsENCBindingBuilder {

    /**
     * All class metadata are kept by this object.
     */
    private ICarMetadata carMetadata = null;


    /**
     * Constructor used only by this class/subclasses.
     * @param carDeployableMetadata the given metadata to analyze.
     */
    protected CarENCBindingBuilder(final ICarMetadata carMetadata) {
        super();
        this.carMetadata = carMetadata;
    }

    /**
     * @return list of metadata to analyze.
     */
    @Override
    protected Collection<IClassMetadata> getClassMetadataCollection() {
        return carMetadata.getClassMetadataCollection();
    }


    /**
     * Found all ENC bindings from the given set of metadata.
     * @param carDeployableMetadata the given metadata to analyze.
     * @return object that contains all bindings found for the given set of
     *         classes.
     * @throws ENCBindingException if the analyze of the set of the metadata has
     *         failed.
     */
    public static ENCBindingHolder analyze(final ICarMetadata carMetadata) throws ENCBindingException {
        CarENCBindingBuilder bindingBuilder = new CarENCBindingBuilder(carMetadata);
        bindingBuilder.analyze();
        return bindingBuilder.getHolder();
    }

    /**
     * Analyze the metadata and populate the binding holder.
     * @throws ENCBindingException if the analyze is failing.
     */
    @Override
    protected void analyze() throws ENCBindingException {

        CommonView view = carMetadata.as(CommonView.class);

        // Merge Resources
        List<IJAnnotationResource> annotationResources = view.getJAnnotationResources();
        if (annotationResources != null) {
            for (IJAnnotationResource jAnnotationResource : annotationResources) {
                addResource(jAnnotationResource, null);
            }
        }

        // Merge Persistence Units
        List<IJavaxPersistenceUnit> persistenceUnits = view.getJavaxPersistencePersistenceUnits();
        if (persistenceUnits != null) {
            for (IJavaxPersistenceUnit javaxPersistenceUnit : persistenceUnits) {
                addPersistenceUnit(javaxPersistenceUnit, null);
            }
        }

        // Merge Persistence Contexts
        List<IJavaxPersistenceContext> javaxPersistenceContexts = view.getJavaxPersistencePersistenceContexts();
        if (javaxPersistenceContexts != null) {
            for (IJavaxPersistenceContext javaxPersistenceContext : javaxPersistenceContexts) {
                addPersistenceContext(javaxPersistenceContext, null);
            }
        }

        // Merge super stuff
        super.analyze();


        // Merge WebService Ref
        List<IJaxwsWebServiceRef> jaxWSWebServiceRefs = view.getJaxwsWebServiceRefs();
        if (jaxWSWebServiceRefs != null) {
            for (IJaxwsWebServiceRef jaxwsWebServiceRef : jaxWSWebServiceRefs) {
                addWebServiceRef(jaxwsWebServiceRef, null);
            }
        }




    }

}
