/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.car.impl.xml.merge;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.jar.Attributes;

import org.ow2.util.archive.api.IArchiveMetadata;
import org.ow2.util.ee.metadata.car.api.ICarMetadata;
import org.ow2.util.ee.metadata.car.api.ICarView;
import org.ow2.util.ee.metadata.car.impl.xml.struct.CAR;
import org.ow2.util.ee.metadata.common.api.interfaces.IEnvironmentRefAccessor;
import org.ow2.util.ee.metadata.common.api.struct.IJaxwsWebServiceRef;
import org.ow2.util.ee.metadata.common.impl.view.CommonView;
import org.ow2.util.ee.metadata.common.impl.xml.merge.MetadataMerge;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.scan.api.metadata.IClassMetadata;
import org.ow2.util.scan.api.metadata.IFieldMetadata;
import org.ow2.util.scan.api.metadata.IMethodMetadata;

/**
 * CarMetadataMerge.
 * @author Gael Lalire
 */
public class CarMetadataMerge extends MetadataMerge {

    /**
     * Logger.
     */
    private static final Log LOGGER = LogFactory.getLog(CarMetadataMerge.class);

    /**
     * The metadata where merge will be done.
     */
    private final ICarMetadata carMetadata;

    /**
     * Constructor.
     * @param carDeployableMetadata the metadata where merge will be done
     */
    public CarMetadataMerge(final ICarMetadata carMetadata) {
        this.carMetadata = carMetadata;
    }

    /**
     * Merge current metadata with car.
     * @param car object representing the xml
     */
    public void merge(final CAR car) {

        CommonView view = carMetadata.as(CommonView.class);

        applyJndiEnvironmentRefsGroup(car, view);

        // only in CAR
        String callbackHandler = car.getCallbackHandler();
        if (callbackHandler != null) {
            IClassMetadata carClassMetadata = getClassMetadata(callbackHandler.replace('.', '/'));
            if (carClassMetadata == null) {
                LOGGER.error("The callback handler class '" + callbackHandler + "' defined in xml was not found.");
            } else {
                ICarView classView = carClassMetadata.as(ICarView.class);
                classView.setCallbackHandler(true);
            }
        }
    }

    /**
     * @param className the class name
     * @return class metadata of class name
     */
    @Override
    public IClassMetadata getClassMetadata(final String className) {
        return carMetadata.getClassMetadata(className);
    }

    /**
     * Provide the main-class of the client.
     * @param env the car environment
     * @return the main-class of the client archive
     */
    @Override
    public String findDefaultClassForEnvironment(final IEnvironmentRefAccessor env) {
        IArchiveMetadata metadata = carMetadata.getArchive().getMetadata();
        String mainClassName = metadata.get(Attributes.Name.MAIN_CLASS.toString());
        if (mainClassName == null) {
            // try with the attribute name in lowercase (not spec compliant...)
            mainClassName = metadata.get("main-class");
            if (mainClassName != null) {
                LOGGER.warn("The '" + Attributes.Name.MAIN_CLASS.toString()
                        + "' attribute is not written correctly in the archive '"
                        + carMetadata.getArchive() + "'");
            }

        }
        if (mainClassName == null) {
            throw new IllegalStateException("No '" + Attributes.Name.MAIN_CLASS.toString()
                    + "' attribute found in the archive '" + carMetadata.getArchive() + "'");
        }

        return encode(mainClassName);
    }


    /**
     * Needs to search the WebServiceRef in all fields/methods of the CAR.
     * @param carMetadata the car metadata
     * @param name the xml name of the service ref
     * @return a list of matching web service ref
     */
    @Override
    protected List<IJaxwsWebServiceRef> findJaxwsWebServiceRefMetadata(final IEnvironmentRefAccessor carMetadata,
            final String name) {
        return Collections.emptyList();
//
//        List<IJaxwsWebServiceRef> elements = new ArrayList<IJaxwsWebServiceRef>();
//
//        Collection<IClassMetadata> clientClassesMetadata = carMetadata.get();
//        if (clientClassesMetadata != null) {
//            for (IClassMetadata classMetadata : clientClassesMetadata) {
//                Collection<IFieldMetadata> fieldMetadatas = classMetadata.();
//                if (fieldMetadatas != null) {
//                    for (IFieldMetadata fieldMetadata : fieldMetadatas) {
//                        addJaxwsWebServiceRefIfFound(fieldMetadata, name, elements);
//                    }
//                }
//                Collection<IMethodMetadata> methodMetadatas = classMetadata.getMethodMetadataCollection();
//                if (methodMetadatas != null) {
//                    for (IMethodMetadata methodMetadata : methodMetadatas) {
//                        addJaxwsWebServiceRefIfFound(methodMetadata, name, elements);
//                    }
//                }
//            }
//        }
//        return elements;
    }

    /**
     * @return A collection of all ClassMetadata of the merged object.
     */
    @Override
    protected Collection<IClassMetadata> getAllClassMetadatas() {
        return carMetadata.getClassMetadataCollection();
    }

    /**
     * @return A collection of all MethodMetadata of the merged object.
     */
    @Override
    protected Collection<IMethodMetadata> getAllMethodMetadatas() {
        // Init the returned List
        Collection<IMethodMetadata> methodMetadatas = new ArrayList<>();
        Collection<IClassMetadata> classMetadatas = carMetadata.getClassMetadataCollection();
        for (IClassMetadata classMetadata : classMetadatas) {
            methodMetadatas.addAll(classMetadata.getMethodMetadataCollection());
        }
        return methodMetadatas;
    }

    /**
     * @return A collection of all FieldMetadata of the merged object.
     */
    @Override
    protected Collection<IFieldMetadata> getAllFieldMetadatas() {
        // Init the returned List
        Collection<IFieldMetadata> fieldMetadatas = new ArrayList<>();

        Collection<IClassMetadata> classMetadatas = carMetadata.getClassMetadataCollection();
        for (IClassMetadata classMetadata : classMetadatas) {
            fieldMetadatas.addAll(classMetadata.getFieldMetadataCollection());
        }
        return fieldMetadatas;
    }
}
