/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.car.impl.xml.parsing;

import java.net.URL;

import org.ow2.util.ee.metadata.car.impl.xml.struct.CAR;
import org.ow2.util.ee.metadata.common.impl.xml.parsing.CommonDeploymentDescLoader;
import org.ow2.util.ee.metadata.common.impl.xml.parsing.ParsingException;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.xml.DocumentParser;
import org.ow2.util.xml.DocumentParserException;
import org.ow2.util.xml.XMLUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * This class analyzes the CAR deployment descriptor and fill a struct with
 * getters.
 * @author Gael Lalire
 */
public final class CARDeploymentDescLoader extends CommonDeploymentDescLoader {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(CARDeploymentDescLoader.class);

    /**
     * EJB3 instance object that is the result of the parsing.
     */
    private CAR car = null;

    /**
     * Utility class, no public constructor.
     * @param url the url that is pointing to the EJB Xml file.
     */
    private CARDeploymentDescLoader(final URL url) {
        super(url);
        this.car = new CAR();
    }

    /**
     * Load the ejb-jar.xml file.
     * @param url the URL of the XML file.
     * @throws ParsingException if parsing of XML file fails.
     * @return an application object.
     */
    public static CAR loadDeploymentDescriptor(final URL url) throws ParsingException {
        logger.debug("Analyzing url {0}", url);

        CARDeploymentDescLoader carDeploymentDescLoader = new CARDeploymentDescLoader(url);
        carDeploymentDescLoader.parse();
        return carDeploymentDescLoader.getCAR();
    }

    /**
     * Analyze the URL and build an EJB3 object which contains data of the XML.
     * @throws ParsingException if the analyze of the file fails.
     */
    public void parse() throws ParsingException {
        // Get document
        Document document = null;
        try {
            document = DocumentParser.getDocument(getURL(), isValidating(), new CAREntityResolver());
        } catch (DocumentParserException e) {
            throw new ParsingException("Cannot parse the url", e);
        }

        // Root element = <application-client>
        Element applicationClientRootElement = document.getDocumentElement();

        if (applicationClientRootElement.getNamespaceURI() != null) {
            // Define NS
            defineXMLNS(applicationClientRootElement);
        }

        analyzeEnvironment(applicationClientRootElement, car);

        car.setCallbackHandler(XMLUtils.getStringValueElement(getXMLNS(), applicationClientRootElement, "callback-handler"));

        if ("true".equals(applicationClientRootElement.getAttribute("metadata-complete"))) {
            car.setMetadataComplete(true);
        } else {
            car.setMetadataComplete(false);
        }

    }

    /**
     * @return the object resulting of the XML parsing.
     */
    public CAR getCAR() {
        return car;
    }

}
