/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.common.api.xml.struct;

import java.io.Serializable;
import java.util.List;


/**
 * Environment used by beans.
 * @author Florent Benoit
 */
public interface IEnvironment extends Serializable {

    /**
     * Add a new env-entry element to this object.
     * @param envEntry the ejb-ref object
     */
    void addEnvEntry(final IEnvEntry envEntry);

    /**
     * @return the list of all env-entry elements
     */
    List<IEnvEntry> getEnvEntryList();

    /**
     * Gets the list of &lt;around-invoke&gt; elements.
     * @return list of &lt;around-invoke&gt; elements.
     */
    List<IAroundInvoke> getAroundInvokeList();

    /**
     * Adds a new &lt;around-invoke&gt; element to assembly descriptor.
     * @param aroundInvoke the &lt;around-invoke&gt; element
     */
    void addAroundInvoke(final IAroundInvoke aroundInvoke);

    /**
     * Gets the list of &lt;post-construct&gt; elements.
     * @return list of &lt;post-construct&gt; elements.
     */
    List<ILifeCycleCallback> getPostConstructCallbackList();

    /**
     * Adds a new &lt;post-construct&gt; element to assembly descriptor.
     * @param postConstructCallback the &lt;post-construct&gt; element
     */
    void addPostConstructCallback(final ILifeCycleCallback postConstructCallback);

    /**
     * Gets the list of &lt;post-activate&gt; elements.
     * @return list of &lt;post-activate&gt; elements.
     */
    List<ILifeCycleCallback> getPostActivateCallbackList();

    /**
     * Adds a new &lt;post-activate&gt; element to assembly descriptor.
     * @param postActivateCallback the &lt;post-activate&gt; element
     */
    void addPostActivateCallback(final ILifeCycleCallback postActivateCallback);

    /**
     * Gets the list of &lt;pre-destroy&gt; elements.
     * @return list of &lt;pre-destroy&gt; elements.
     */
    List<ILifeCycleCallback> getPreDestroyCallbackList();

    /**
     * Adds a new &lt;pre-destroy&gt; element to assembly descriptor.
     * @param preDestroyCallback the &lt;pre-destroy&gt; element
     */
    void addPreDestroyCallback(final ILifeCycleCallback preDestroyCallback);

    /**
     * Gets the list of &lt;pre-passivate&gt; elements.
     * @return list of &lt;pre-passivate&gt; elements.
     */
    List<ILifeCycleCallback> getPrePassivateCallbackList();

    /**
     * Adds a new &lt;pre-passivate&gt; element to assembly descriptor.
     * @param prePassivateCallback the &lt;pre-passivate&gt; element
     */
    void addPrePassivateCallback(final ILifeCycleCallback prePassivateCallback);

    /**
     * Gets the list of &lt;ejb-ref&gt; elements.
     * @return list of &lt;ejb-ref&gt; elements.
     */
    List<IEJBRef> getEJBRefList();

    /**
     * Adds a new &lt;ejb-ref&gt; element.
     * @param ejbRef the &lt;ejb-ref&gt; element
     */
    void addEJBRef(final IEJBRef ejbRef);

    /**
     * Gets the list of &lt;resource-ref&gt; elements.
     * @return list of &lt;resource-ref&gt; elements.
     */
    List<IResourceRef> getResourceRefList();

    /**
     * Adds a new &lt;resource-ref&gt; element.
     * @param resourceRef the &lt;resource-ref&gt; element
     */
    void addResourceRef(final IResourceRef resourceRef);

    /**
     * Gets the list of &lt;message-destination-ref&gt; elements.
     * @return list of &lt;message-destination-ref&gt; elements.
     */
    List<IMessageDestinationRef> getMessageDestinationRefList();

    /**
     * Adds a new &lt;message-destination-ref&gt; element.
     * @param messageDestinationRef the &lt;message-destination-ref&gt; element
     */
    void addMessageDestinationRef(final IMessageDestinationRef messageDestinationRef);

    /**
     * Gets the list of &lt;service-ref&gt; elements.
     * @return list of &lt;service-ref&gt; elements.
     */
    List<IServiceRef> getServiceRefList();

    /**
     * Adds a new &lt;service-ref&gt; element.
     * @param serviceRef the &lt;service-ref&gt; element
     */
    void addServiceRef(final IServiceRef serviceRef);

    /**
     * Gets the list of &lt;resource-env-ref&gt; elements.
     * @return list of &lt;resource-env-ref&gt; elements.
     */
    List<IResourceEnvRef> getResourceEnvRefList();

    /**
     * Adds a new &lt;resource-ref&gt; element.
     * @param resourceEnvRef the &lt;resource-ref&gt; element
     */
    void addResourceEnvRef(final IResourceEnvRef resourceEnvRef);

    /**
     * Gets the list of &lt;ejb-local-ref&gt; elements.
     * @return list of &lt;ejb-local-ref&gt; elements.
     */
    List<IEJBLocalRef> getEJBLocalRefList();

    /**
     * Adds a new &lt;ejb-local-ref&gt; element.
     * @param ejLocalRef the &lt;ejb-local-ref&gt; element
     */
    void addEJBLocalRef(final IEJBLocalRef ejLocalRef);

    /**
     * Gets the list of &lt;persistence-unit-ref&gt; elements.
     * @return list of &lt;persistence-unit-ref&gt; elements.
     */
    List<IPersistenceUnitRef> getPersistenceUnitRefList();

    /**
     * Adds a new &lt;persistence-unit-ref&gt; element.
     * @param persistenceUnitRef the &lt;persistence-unit-ref&gt; element
     */
    void addPersistenceUnitRef(final IPersistenceUnitRef persistenceUnitRef);

    /**
     * Gets the list of &lt;persistence-context-ref&gt; elements.
     * @return list of &lt;persistence-context-ref&gt; elements.
     */
    List<IPersistenceContextRef> getPersistenceContextRefList();

    /**
     * Adds a new &lt;persistence-context-ref&gt; element.
     * @param persistenceContextRef the &lt;persistence-context-ref&gt; element
     */
    void addPersistenceContextRef(final IPersistenceContextRef persistenceContextRef);

    /**
     * Gets the list of &lt;security-role-ref&gt; elements.
     * @return list of &lt;security-role-ref&gt; elements.
     */
    List<ISecurityRoleRef> getSecurityRoleRefList();

    /**
     * Adds a new &lt;security-role-ref&gt; element.
     * @param ejLocalRef the &lt;security-role-ref&gt; element
     */
    void addSecurityRoleRef(final ISecurityRoleRef securityRoleRef);

    List<IDataSource> getDataSourceList();

    void addDataSource(final IDataSource dataSource);
}
