/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.common.impl;

import java.util.Arrays;

import org.ow2.util.ee.metadata.common.api.interfaces.ISharedMetadataView;
import org.ow2.util.ee.metadata.common.api.struct.IJAnnotationResource;
import org.ow2.util.ee.metadata.common.api.struct.IJEjbEJB;
import org.ow2.util.ee.metadata.common.api.struct.IJavaxPersistenceContext;
import org.ow2.util.ee.metadata.common.api.struct.IJavaxPersistenceUnit;
import org.ow2.util.ee.metadata.common.api.struct.IJaxwsWebServiceRef;
import org.ow2.util.ee.metadata.common.api.struct.IJwsHandlerChain;

/**
 * This class provides method to keep env state.
 * @author Gael Lalire
 */
public class AbsSharedMetadata implements ISharedMetadataView {

    /**
     * This field is used as a PersistenceContext.
     */
    private IJavaxPersistenceContext javaxPersistenceContext = null;

    /**
     * This field is used as a PersistenceUnit.
     */
    private IJavaxPersistenceUnit javaxPersistenceUnit = null;

    /**
     * Object representing &#64;{@link javax.ejb.EJB} annotation.
     */
    private IJEjbEJB jEjbEJB = null;

    /**
     * Object representing &#64;{@link javax.annotation.Resource} annotation.
     */
    private IJAnnotationResource jAnnotationResource = null;

    /**
     * Object representing &#64;{@link javax.xml.ws.WebServiceRef} annotation.
     */
    private IJaxwsWebServiceRef jaxwsWebServiceRef = null;

    /**
     * Object representing &#64;{@link javax.jws.HandlerChain} annotation.
     */
    private IJwsHandlerChain jwsHandlerChain = null;

    /**
     * Constructor.
     */
    public AbsSharedMetadata() {
    }

    /**
     * @return JEjbEJB object representing javax.ejb.EJB annotation.
     */
    public IJEjbEJB getJEjbEJB() {
        return jEjbEJB;
    }

    /**
     * Set JAnnotationEJB object.
     * @param jEjbEJB object representing javax.annotation.EJB annotation.
     */
    public void setJEjbEJB(final IJEjbEJB jEjbEJB) {
        this.jEjbEJB = jEjbEJB;
    }

    /**
     * @return JAnnotationResource object representing javax.annotation.Resource
     *         annotation.
     */
    public IJAnnotationResource getJAnnotationResource() {
        return jAnnotationResource;
    }

    /**
     * Set JAnnotationResource object.
     * @param jAnnotationResource object representing javax.annotation.Resource
     *        annotation.
     */
    public void setJAnnotationResource(final IJAnnotationResource jAnnotationResource) {
        this.jAnnotationResource = jAnnotationResource;
    }

    /**
     * @return true if this field is used as a persistence context.
     */
    public boolean isPersistenceContext() {
        return javaxPersistenceContext != null;
    }

    /**
     * @return the persistence context infos.
     */
    public IJavaxPersistenceContext getJavaxPersistenceContext() {
        return javaxPersistenceContext;
    }

    /**
     * Sets the persistence context info on this field.
     * @param javaxPersistenceContext information on persistence context.
     */
    public void setJavaxPersistenceContext(final IJavaxPersistenceContext javaxPersistenceContext) {
        this.javaxPersistenceContext = javaxPersistenceContext;
    }

    /**
     * @return true if this field is used as a persistence unit.
     */
    public boolean isPersistenceUnit() {
        return javaxPersistenceUnit != null;
    }

    /**
     * @return the persistence unit infos.
     */
    public IJavaxPersistenceUnit getJavaxPersistenceUnit() {
        return javaxPersistenceUnit;
    }

    /**
     * Sets the persistence unit info on this field.
     * @param javaxPersistenceUnit information on persistence unit.
     */
    public void setJavaxPersistenceUnit(final IJavaxPersistenceUnit javaxPersistenceUnit) {
        this.javaxPersistenceUnit = javaxPersistenceUnit;
    }

    /**
     * @return the WebServiceRef infos
     */
    public IJaxwsWebServiceRef getJaxwsWebServiceRef() {
        return jaxwsWebServiceRef;
    }

    /**
     * Sets the WebServiceRef infos.
     * @param jaxwsWebServiceRef information on WebServiceRef
     */
    public void setJaxwsWebServiceRef(final IJaxwsWebServiceRef jaxwsWebServiceRef) {
        this.jaxwsWebServiceRef = jaxwsWebServiceRef;
    }

    /**
     * @return the HandlerChain infos
     */
    public IJwsHandlerChain getJwsHandlerChain() {
        return jwsHandlerChain;
    }

    /**
     * Sets the HandlerChain infos.
     * @param jwsHandlerChain information on HandlerChain
     */
    public void setJwsHandlerChain(final IJwsHandlerChain jwsHandlerChain) {
        this.jwsHandlerChain = jwsHandlerChain;
    }

    /**
     * @return string representation
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        String titleIndent = " ";
        // classname
        sb.append(titleIndent);
        sb.append(this.getClass().getName().substring(this.getClass().getPackage().getName().length() + 1));
        sb.append("[\n");

        // jEjbEJB
        concatStringBuilder("jEjbEJB", jEjbEJB, sb);

        // jAnnotationResource
        concatStringBuilder("jAnnotationResource", jAnnotationResource, sb);

        // javaxPersistenceContext
        concatStringBuilder("javaxPersistenceContext", javaxPersistenceContext, sb);

        // javaxPersistenceUnit
        concatStringBuilder("javaxPersistenceUnit", javaxPersistenceUnit, sb);

        // jaxwsWebServiceRef
        concatStringBuilder("jaxwsWebServiceRef", jaxwsWebServiceRef, sb);

        // jwsHandlerChain
        concatStringBuilder("jwsHandlerChain", jwsHandlerChain, sb);

        sb.append(titleIndent);
        sb.append("]\n");
        return sb.toString();
    }

    /**
     * Adds an entry to the given StringBuilder.
     * @param name the name of the entry.
     * @param object object to add.
     * @param sb the string builder object on which add the given element.
     * @param indent the indent to add at each line.
     */
    public static void concatStringBuilder(final String name, final Object object, final StringBuilder sb, final String indent) {
        if (object instanceof Boolean) {
            // do not print if false.
            if (!((Boolean) object).booleanValue()) {
                return;
            }
        }
        if (object != null) {
            sb.append(indent);
            sb.append(name);
            sb.append("=");
            // For arrays, use asList
            if (object instanceof Object[]) {
                sb.append(Arrays.asList((Object[]) object));
            } else {
                sb.append(object);
            }
            sb.append("\n");
        }
    }

    /**
     * Adds an entry to the given StringBuilder.
     * @param name the name of the entry.
     * @param object object to add.
     * @param sb the string builder object on which add the given element.
     */
    public static void concatStringBuilder(final String name, final Object object, final StringBuilder sb) {
        concatStringBuilder(name, object, sb, "    ");
    }

}
