/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.common.impl.configurator.visitor;

import static java.lang.annotation.ElementType.FIELD;
import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.ElementType.TYPE;

import org.ow2.util.ee.metadata.common.api.struct.IJEjbEJB;
import org.ow2.util.ee.metadata.common.impl.struct.JEjbEJB;
import org.ow2.util.scan.api.IAnnotationVisitorContext;
import org.ow2.util.scan.api.IType;
import org.ow2.util.scan.api.annotation.VisitorTarget;
import org.ow2.util.scan.api.annotation.VisitorType;
import org.ow2.util.scan.api.visitor.DefaultAnnotationVisitor;

/**
 * This class manages the handling of &#64;{@link javax.ejb.EJB} annotation.
 * @author Florent Benoit
 */
@VisitorTarget({TYPE, METHOD, FIELD})
@VisitorType("javax.ejb.EJB")
public class JavaxEjbEJBVisitor extends DefaultAnnotationVisitor<IJEjbEJB> {

    /**
     * Name attribute of the annotation.
     */
    private static final String NAME = "name";

    /**
     * Bean interface attribute of the annotation.
     */
    private static final String BEAN_INTERFACE = "beanInterface";

    /**
     * Bean name attribute of the annotation.
     */
    private static final String BEAN_NAME = "beanName";

    /**
     * Mapped name attribute of the annotation.
     */
    private static final String MAPPED_NAME = "mappedName";

    /**
     * Lookup attribute of the annotation.
     */
    private static final String LOOKUP = "lookup";

    /**
     * Visits a primitive value of the annotation.<br>
     * @param name the value name.
     * @param value the actual value, whose type must be {@link Byte},
     *        {@link Boolean}, {@link Character}, {@link Short},
     *        {@link Integer}, {@link Long}, {@link Float}, {@link Double},
     *        {@link String} or {@link IType}.
     */
    @Override
    public void visit(final String name, final Object value, IAnnotationVisitorContext<IJEjbEJB> visitorContext) {
        IJEjbEJB jEjbEJB = visitorContext.getLocal();

        if (name.equals(NAME)) {
            jEjbEJB.setName((String) value);
        } else if (BEAN_INTERFACE.equals(name)) {
            IType type = (IType) value;
            jEjbEJB.setBeanInterface(type.getClassName());
        } else if (BEAN_NAME.equals(name)) {
            jEjbEJB.setBeanName((String) value);
        } else if (MAPPED_NAME.equals(name)) {
            jEjbEJB.setMappedName((String) value);
        } else if (LOOKUP.equals(name)) {
            jEjbEJB.setLookup((String) value);
        }
    }

    /**
     * Visits the end of the annotation. <br>
     * Creates the object and store it.
     */
    @Override
    public void visitEnd(IAnnotationVisitorContext<IJEjbEJB> visitorContext) {
        // add object
        visitorContext.set(visitorContext.getLocal());
    }

    @Override
    public IJEjbEJB buildInstance() {
        return new JEjbEJB();
    }


}
