/**
 * OW2 Utilities
 * Copyright (C) 2009-2012 Bull S.A.S.
 * Contact: utilities@ow2.org
 *
 * Licensed  under the  Apache License,  Version 2.0  (the "License");
 * you may not use  this file  except in  compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed  under the  License is distributed on an "AS IS" BASIS,
 * WITHOUT  WARRANTIES OR CONDITIONS  OF ANY KIND, either  express  or
 * implied.
 *
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.common.impl.configurator.visitor.clazz;

import static java.lang.annotation.ElementType.TYPE;

import java.util.ArrayList;
import java.util.List;

import org.ow2.util.ee.metadata.common.api.struct.IJAnnotationSqlDataSourceDefinition;
import org.ow2.util.ee.metadata.common.api.struct.IJAnnotationSqlDataSourceDefinitions;
import org.ow2.util.ee.metadata.common.impl.enc.ENCBinding;
import org.ow2.util.ee.metadata.common.impl.struct.JAnnotationSqlDataSourceDefinition;
import org.ow2.util.ee.metadata.common.impl.struct.JAnnotationSqlDataSourceDefinitions;
import org.ow2.util.scan.api.IAnnotationVisitor;
import org.ow2.util.scan.api.IAnnotationVisitorContext;
import org.ow2.util.scan.api.annotation.VisitorTarget;
import org.ow2.util.scan.api.annotation.VisitorType;
import org.ow2.util.scan.api.visitor.DefaultAnnotationVisitor;

/**
 * This class manages the handling of &#64;{@link javax.annotation.sql.DataSourceDefinition} annotation.
 *
 * @param <T> An implementation of IJAnnotationSqlDataSourceDefinition interface.
 *
 * @author Loic Albertin
 */
@VisitorTarget(TYPE)
@VisitorType("javax.annotation.sql.DataSourceDefinition")
public class JavaxAnnotationSqlDataSourceDefinitionVisitor extends DefaultAnnotationVisitor<IJAnnotationSqlDataSourceDefinition> {

    /**
     * Name attribute of the annotation.
     */
    private static final String NAME = "name";

    /**
     * ClassName attribute of the annotation.
     */
    private static final String CLASS_NAME = "className";

    /**
     * Description attribute of the annotation.
     */
    private static final String DESCRIPTION = "description";


    /**
     * Url attribute of the annotation.
     */
    private static final String URL = "url";


    /**
     * User attribute of the annotation.
     */
    private static final String USER = "user";


    /**
     * password attribute of the annotation.
     */
    private static final String PASSWORD = "password";


    /**
     * databaseName attribute of the annotation.
     */
    private static final String DATABASE_NAME = "databaseName";


    /**
     * portNumber attribute of the annotation.
     */
    private static final String PORT_NUMBER = "portNumber";


    /**
     * serverName attribute of the annotation.
     */
    private static final String SERVER_NAME = "serverName";


    /**
     * isolationLevel attribute of the annotation.
     */
    private static final String ISOLATION_LEVEL = "isolationLevel";

    /**
     * transactional attribute of the annotation.
     */
    private static final String TRANSACTIONAL = "transactional";

    /**
     * initialPoolSize attribute of the annotation.
     */
    private static final String INITIAL_POOL_SIZE = "initialPoolSize";


    /**
     * maxPoolSize attribute of the annotation.
     */
    private static final String MAX_POOL_SIZE = "maxPoolSize";


    /**
     * minPoolSize attribute of the annotation.
     */
    private static final String MIN_POOL_SIZE = "minPoolSize";


    /**
     * maxIdleTime attribute of the annotation.
     */
    private static final String MAX_IDLE_TIME = "maxIdleTime";


    /**
     * maxStatements attribute of the annotation.
     */
    private static final String MAX_STATEMENTS = "maxStatements";


    /**
     * properties attribute of the annotation.
     */
    private static final String PROPERTIES = "properties";


    /**
     * loginTimeout attribute of the annotation.
     */
    private static final String LOGIN_TIMEOUT = "loginTimeout";


    private final StringArrayVisitor propertiesArrayVisitor = new StringArrayVisitor();



    /**
     * Visits a primitive value of the annotation.<br>
     *
     * @param name  the value name.
     * @param value the actual value, whose type must be {@link Byte},
     *              {@link Boolean}, {@link Character}, {@link Short},
     *              {@link Integer}, {@link Long}, {@link Float}, {@link Double},
     *              {@link String} or {@link org.ow2.util.scan.api.IType}.
     */
    @Override
    public void visit(final String name, final Object value, IAnnotationVisitorContext<IJAnnotationSqlDataSourceDefinition> visitorContext) {
        IJAnnotationSqlDataSourceDefinition jAnnotationSqlDataSourceDefinition = visitorContext.getLocal();

        if (NAME.equals(name)) {
            //In order to ease XML desc and annotations merging adding default ns prefix on names without explicit
            // ns declaration
            String jndiName = (String) value;
            if (!jndiName.startsWith(ENCBinding.NS_JAVA_APP_PREFIX) &&
                !jndiName.startsWith(ENCBinding.NS_JAVA_COMP_PREFIX) &&
                !jndiName.startsWith(ENCBinding.NS_JAVA_GLOBAL_PREFIX) &&
                !jndiName.startsWith(ENCBinding.NS_JAVA_MODULE_PREFIX)) {
                jndiName = ENCBinding.DEFAULT_NS_PREFIX + jndiName;
            }
            jAnnotationSqlDataSourceDefinition.setName(jndiName);
        } else if (CLASS_NAME.equals(name)) {
            jAnnotationSqlDataSourceDefinition.setClassName((String) value);
        } else if (DESCRIPTION.equals(name)) {
            jAnnotationSqlDataSourceDefinition.setDescription((String) value);
        } else if (URL.equals(name)) {
            jAnnotationSqlDataSourceDefinition.setUrl((String) value);
        } else if (USER.equals(name)) {
            jAnnotationSqlDataSourceDefinition.setUser((String) value);
        } else if (PASSWORD.equals(name)) {
            jAnnotationSqlDataSourceDefinition.setPassword((String) value);
        } else if (DATABASE_NAME.equals(name)) {
            jAnnotationSqlDataSourceDefinition.setDatabaseName((String) value);
        } else if (PORT_NUMBER.equals(name)) {
            jAnnotationSqlDataSourceDefinition.setPortNumber((Integer) value);
        } else if (SERVER_NAME.equals(name)) {
            jAnnotationSqlDataSourceDefinition.setServerName((String) value);
        } else if (ISOLATION_LEVEL.equals(name)) {
            jAnnotationSqlDataSourceDefinition.setIsolationLevel((Integer) value);
        } else if (TRANSACTIONAL.equals(name)) {
            jAnnotationSqlDataSourceDefinition.setTransactional((Boolean) value);
        } else if (INITIAL_POOL_SIZE.equals(name)) {
            jAnnotationSqlDataSourceDefinition.setInitialPoolSize((Integer) value);
        } else if (MAX_POOL_SIZE.equals(name)) {
            jAnnotationSqlDataSourceDefinition.setMaxPoolSize((Integer) value);
        } else if (MIN_POOL_SIZE.equals(name)) {
            jAnnotationSqlDataSourceDefinition.setMinPoolSize((Integer) value);
        } else if (MAX_IDLE_TIME.equals(name)) {
            jAnnotationSqlDataSourceDefinition.setMaxIdleTime((Integer) value);
        } else if (MAX_STATEMENTS.equals(name)) {
            jAnnotationSqlDataSourceDefinition.setMaxStatements((Integer) value);
        } else if (LOGIN_TIMEOUT.equals(name)) {
            jAnnotationSqlDataSourceDefinition.setLoginTimeout((Integer) value);
        }
    }

    @Override
    public IAnnotationVisitor<IJAnnotationSqlDataSourceDefinition> visitArray(String name, IAnnotationVisitorContext<IJAnnotationSqlDataSourceDefinition> visitorContext) {
        if (PROPERTIES.equals(name)) {
            return propertiesArrayVisitor;
        }
        return this;
    }

    /**
     * Visits the end of the annotation. <br>
     * Creates the object and store it.
     */
    @Override
    public void visitEnd(IAnnotationVisitorContext<IJAnnotationSqlDataSourceDefinition> visitorContext) {
        IJAnnotationSqlDataSourceDefinition jAnnotationSqlDataSourceDefinition = visitorContext.getLocal();
        //Add properties
        jAnnotationSqlDataSourceDefinition.setProperties(propertiesArrayVisitor.getValues().toArray(new String[propertiesArrayVisitor.getValues().size()]));
        // add object in the global list
        IJAnnotationSqlDataSourceDefinitions lst = visitorContext.get(IJAnnotationSqlDataSourceDefinitions.class);
        if (lst == null) {
            lst = new JAnnotationSqlDataSourceDefinitions();
            visitorContext.set(IJAnnotationSqlDataSourceDefinitions.class, lst);
        }
        if (!lst.getDatasourceDefinitions().contains(jAnnotationSqlDataSourceDefinition)) {
            lst.getDatasourceDefinitions().add(jAnnotationSqlDataSourceDefinition);
        }

        propertiesArrayVisitor.values.clear();
    }


    class StringArrayVisitor extends
        DefaultAnnotationVisitor {

        private final List<String> values = new ArrayList<String>();

        public List<String> getValues() {
            return values;
        }

        /**
         * Constructor.
         */
        public StringArrayVisitor() {
            super();
        }

        @Override
        public void visit(String name, Object value, IAnnotationVisitorContext visitorContext) {
            if (value instanceof String) {
                values.add((String) value);
            }
        }

        @Override
        public Object buildInstance() {
            // TODO Auto-generated method stub
            return null;
        }
    }


    @Override
    public IJAnnotationSqlDataSourceDefinition buildInstance() {
        return new JAnnotationSqlDataSourceDefinition();
    }

}
