/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.common.impl.enc;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import org.ow2.util.asm.Type;
import org.ow2.util.ee.metadata.common.api.interfaces.ISharedMetadataView;
import org.ow2.util.ee.metadata.common.api.struct.IJAnnotationResource;
import org.ow2.util.ee.metadata.common.api.struct.IJAnnotationSqlDataSourceDefinition;
import org.ow2.util.ee.metadata.common.api.struct.IJEjbEJB;
import org.ow2.util.ee.metadata.common.api.struct.IJavaxPersistenceContext;
import org.ow2.util.ee.metadata.common.api.struct.IJavaxPersistenceUnit;
import org.ow2.util.ee.metadata.common.api.struct.IJaxwsWebServiceRef;
import org.ow2.util.ee.metadata.common.impl.view.CommonView;
import org.ow2.util.scan.api.metadata.IClassMetadata;
import org.ow2.util.scan.api.metadata.IFieldMetadata;
import org.ow2.util.scan.api.metadata.IMetadata;
import org.ow2.util.scan.api.metadata.IMethodMetadata;
import org.ow2.util.scan.api.metadata.structures.IMethod;

/**
 * Class that helps to build the ENC Binding by analyzing metadata.
 * @author Florent BENOIT
 */
public abstract class AbsENCBindingBuilder {

    /**
     * Binding holder object.
     */
    private ENCBindingHolder bindingHolder = null;

    /**
     * Replace length to create default JNDI names.
     */
    private static final int GETTER_LENGTH = 3;

    /**
     * @return list of metadata to analyze.
     */
    protected abstract Collection<IClassMetadata> getClassMetadataCollection();

    /**
     * Constructor used only by this class/subclasses.
     */
    protected AbsENCBindingBuilder() {
        this.bindingHolder = new ENCBindingHolder();
    }

    /**
     * Analyze the metadata and populate the binding holder.
     * @throws ENCBindingException if the analyze is failing.
     */
    protected void analyze() throws ENCBindingException {

        // Analyze each class
        for (IClassMetadata classMetadata : getClassMetadataCollection()) {
            analyzeClassMetadata(classMetadata);
        }
    }

    /**
     * Analyze the given class metadata.
     * @param classMetadata the given metadata to analyze.
     * @throws ENCBindingException if the class has not been analyzed
     */
    protected void analyzeClassMetadata(final IClassMetadata classMetadata) throws ENCBindingException {

        CommonView classView = classMetadata.as(CommonView.class);

        // Are there persistence contexts (only on Class)
        List<IJavaxPersistenceContext> javaxPersistenceContexts = classView.getJavaxPersistencePersistenceContexts();
        if (javaxPersistenceContexts != null) {
            for (IJavaxPersistenceContext javaxPersistenceContext : javaxPersistenceContexts) {
                addPersistenceContext(javaxPersistenceContext, null);
            }
        }

        // Are there persistence units (only on Class)
        List<IJavaxPersistenceUnit> javaxPersistenceUnits = classView.getJavaxPersistencePersistenceUnits();
        if (javaxPersistenceUnits != null) {
            for (IJavaxPersistenceUnit javaxPersistenceUnit : javaxPersistenceUnits) {
                addPersistenceUnit(javaxPersistenceUnit, null);
            }
        }

        // Are there resources (only on Class)
        List<IJAnnotationResource> jAnnotationResources = classView.getJAnnotationResources();
        if (jAnnotationResources != null) {
            for (IJAnnotationResource jAnnotationResource : jAnnotationResources) {
                addResource(jAnnotationResource, null);
            }
        }

        // Are there EJBs (only on Class)
        List<IJEjbEJB> jEjbEJBs = classView.getJEjbEJBs();
        if (jEjbEJBs != null) {
            for (IJEjbEJB jEjbEJB : jEjbEJBs) {
                addEJB(jEjbEJB, null);
            }
        }

        List<IJAnnotationSqlDataSourceDefinition> dataSourceDefinitionList =
                classView.getJAnnotationSqlDataSourceDefinitions();
        if (dataSourceDefinitionList != null) {
            for (IJAnnotationSqlDataSourceDefinition dataSourceDefinition : dataSourceDefinitionList) {
               addDataSource(dataSourceDefinition);
            }
        }

        // Are there WebServiceRefs (only on Class)
        List<IJaxwsWebServiceRef> refs = classView.getJaxwsWebServiceRefs();
        if (refs != null) {
            for (IJaxwsWebServiceRef ref : refs) {
                if (isNullOrEmpty(ref.getName())) {
                    throw new ENCBindingException("@WebServiceRef.name (or service-ref/service-ref-name)"
                                                  + " required in the annotation/XML defined for the class '"
                                                  + classMetadata.getJClass() + "'.");
                }
                if (isNullOrEmpty(ref.getType())) {
                    throw new ENCBindingException("@WebServiceRef.type (or service-ref/service-ref-type)"
                                                  + " required in the annotation/XML defined for the class '"
                                                  + classMetadata.getJClass() + "'.");
                }
                if (isNullOrEmpty(ref.getValue())) {
                    // type is set, simply use the type (that denotes a service-interface) as the value
                    ref.setValue(ref.getType());
                }
                addWebServiceRef(ref, null);
            }
        }

        // Analyze common stuff on the class
        analyzeCommonMetadata(classMetadata);

        // Then, analyze each field of the class
        for (IFieldMetadata fieldAnnotationMetadata : classMetadata.getFieldMetadataCollection()) {
            analyzeFieldMetadata(fieldAnnotationMetadata);
        }

        // And each method of the class
        for (IMethodMetadata methodAnnotationMetadata : classMetadata.getMethodMetadataCollection()) {
            analyzeMethodMetadata(methodAnnotationMetadata);
        }

    }

    /**
     * Analyze the given field metadata.
     * @param fieldMetadata the given metadata to analyze.
     * @throws ENCBindingException if the field has not been analyzed
     */
    protected void analyzeFieldMetadata(final IFieldMetadata fieldMetadata) throws ENCBindingException {
        // Analyze common stuff on the field
        analyzeCommonMetadata(fieldMetadata);
    }

    /**
     * Analyze the given method metadata.
     * @param methodMetadata the given metadata to analyze.
     * @throws ENCBindingException if the method has not been analyzed
     */
    protected void analyzeMethodMetadata(final IMethodMetadata methodMetadata) throws ENCBindingException {
        // Analyze common stuff on the method
        analyzeCommonMetadata(methodMetadata);
    }

    /**
     * Analyze the given common metadata.
     * @param commonMetadata the given metadata to analyze.
     * @throws ENCBindingException if the common metadata has not been analyzed
     */
    protected void analyzeCommonMetadata(final IMetadata commonMetadata) throws ENCBindingException {

        CommonView commonView = commonMetadata.as(CommonView.class);

        // Handle persistence context
        IJavaxPersistenceContext javaxPersistenceContext = commonView.getJavaxPersistenceContext();
        if (javaxPersistenceContext != null) {
            // Set name if null
            if (isNullOrEmpty(javaxPersistenceContext.getName())) {
                javaxPersistenceContext.setName(getName(commonMetadata, javaxPersistenceContext));
            }
            addPersistenceContext(javaxPersistenceContext, commonView);
        }

        // Handle persistence unit
        IJavaxPersistenceUnit javaxPersistenceUnit = commonView.getJavaxPersistenceUnit();
        if (javaxPersistenceUnit != null) {
            // Set name if null
            if (isNullOrEmpty(javaxPersistenceUnit.getName())) {
                javaxPersistenceUnit.setName(getName(commonMetadata, javaxPersistenceUnit));
            }
            addPersistenceUnit(javaxPersistenceUnit, commonView);
        }

        // Handle resource
        IJAnnotationResource jAnnotationResource = commonView.getJAnnotationResource();
        if (jAnnotationResource != null) {
            // Set name if null
            if (isNullOrEmpty(jAnnotationResource.getName())) {
                jAnnotationResource.setName(getName(commonMetadata, jAnnotationResource));
            }
            jAnnotationResource.setType(getType(commonMetadata, jAnnotationResource.getType()));
            addResource(jAnnotationResource, commonView);
        }

        // Handle EJB
        IJEjbEJB jEjbEJB = commonView.getJEjbEJB();
        if (jEjbEJB != null) {
            // Set name if null
            if (isNullOrEmpty(jEjbEJB.getName())) {
                jEjbEJB.setName(getName(commonMetadata, jEjbEJB));
            }
            jEjbEJB.setBeanInterface(getType(commonMetadata, jEjbEJB.getBeanInterface()));
            addEJB(jEjbEJB, commonView);
        }

        // Handle WebServiceRef
        IJaxwsWebServiceRef ref = commonView.getJaxwsWebServiceRef();
        if (ref != null) {
            // Set name if null
            if (isNullOrEmpty(ref.getName())) {
                ref.setName(getName(commonMetadata, ref));
            }

            // Infer the 'WSR.value' value if null, empty or default value
            if (isNullOrEmpty(ref.getValue()) || "java.lang.Object".equals(ref.getValue())) {
                // value field may not be set in attribute case:
                // @WebServiceRef private SOAPService service;
                // In that case the value is infered from the
                // attribute's type

                ref.setValue(getType(commonMetadata, null));
            }
            String type = ref.getType();
            if ("javax.xml.ws.Service".equals(type)) {
                type = null;
            }
            ref.setType(getType(commonMetadata, type));

            addWebServiceRef(ref, commonView);
        }


    }

    /**
     * Checks if the given string is empty or null.
     * @param name the given string to test
     * @return true if the string is null or empty
     */
    private boolean isNullOrEmpty(final String name) {
        return (name == null) || "".equals(name);
    }

    /**
     * Gets the ENC name for the given metadata.
     * @param commonMetadata the metadata to analyze.
     * @param o the given object associated to the metadata (used for logging).
     * @return the given ENC name
     * @throws ENCBindingException if no name has been defined on the given name
     *         and the metadata is invalid.
     */
    private String getName(final IMetadata commonMetadata, final Object o) throws ENCBindingException {
        if (commonMetadata instanceof IFieldMetadata) {
            IFieldMetadata fieldAnnotationMetadata = (IFieldMetadata) commonMetadata;
            return fieldAnnotationMetadata.getClassMetadata().getJClass().getName().replace("/", ".") + "/"
                    + fieldAnnotationMetadata.getJField().getName();
        }

        if (commonMetadata instanceof IMethodMetadata) {
            IMethodMetadata methodAnnotationMetadata = (IMethodMetadata) commonMetadata;

            StringBuilder propertyBuilder = new StringBuilder(methodAnnotationMetadata.getJMethod().getName());
            propertyBuilder.delete(0, GETTER_LENGTH);
            propertyBuilder.setCharAt(0, Character.toLowerCase(propertyBuilder.charAt(0)));
            propertyBuilder.insert(0, methodAnnotationMetadata.getClassMetadata().getJClass().getName().replace("/", ".")
                    + "/");
            return propertyBuilder.toString();
        }

        // Last case is the Class case.
        throw new ENCBindingException("There is no defined name in the object '" + o + "' that is defined on the class '"
                + commonMetadata + "'.");

    }

    /**
     * Gets the type for the given metadata.
     * @param commonMetadata the metadata to analyze.
     * @param currentType the actual type of the selected annotation
     * @return the given type value (interface)
     * @throws ENCBindingException if no name has been defined on the given name
     *         and the metadata is invalid.
     */
    private String getType(final IMetadata commonMetadata,
                           final String currentType)
            throws ENCBindingException {

        // return given type if specified
        if (currentType != null && !"".equals(currentType) && !"java.lang.Object".equals(currentType)) {
            return currentType;
        }

        if (commonMetadata instanceof IFieldMetadata) {
            IFieldMetadata fieldAnnotationMetadata = (IFieldMetadata) commonMetadata;
            Type typeInterface = Type.getType(fieldAnnotationMetadata.getJField().getDescriptor());
            return typeInterface.getClassName();
        } else if (commonMetadata instanceof IMethodMetadata) {
            IMethodMetadata methodAnnotationMetadata = (IMethodMetadata) commonMetadata;

            IMethod jMethod = methodAnnotationMetadata.getJMethod();
            Type[] args = Type.getArgumentTypes(jMethod.getDescriptor());
            if (args.length != 1) {
                throw new ENCBindingException("Method args '" + Arrays.asList(args) + "' for method '" + jMethod
                        + "' are invalid. Length should be of 1.");
            }
            Type typeInterface = args[0];
            return typeInterface.getClassName();
        }

        return currentType;
    }

    /**
     * Adds the given persistence context to the list of bindings.
     * @param javaxPersistenceContext the given persistence context.
     */
    protected void addPersistenceContext(final IJavaxPersistenceContext javaxPersistenceContext, final ISharedMetadataView commonMetadata) {
        this.bindingHolder.addPersistenceContextBinding(new ENCBinding<IJavaxPersistenceContext>(javaxPersistenceContext
                .getName(), javaxPersistenceContext));
        addFieldOrMethodENC(javaxPersistenceContext.getName(), commonMetadata);
    }

    /**
     * Adds the given persistence unit to the list of bindings.
     * @param javaxPersistenceUnit the given persistence unit.
     */
    protected void addPersistenceUnit(final IJavaxPersistenceUnit javaxPersistenceUnit, final ISharedMetadataView commonMetadata) {
        this.bindingHolder.addPersistenceUnitBinding(new ENCBinding<IJavaxPersistenceUnit>(javaxPersistenceUnit.getName(),
                javaxPersistenceUnit));
        addFieldOrMethodENC(javaxPersistenceUnit.getName(), commonMetadata);
    }

    /**
     * Adds the given resource to the list of bindings.
     * @param jAnnotationResource the given resource.
     */
    protected void addResource(final IJAnnotationResource jAnnotationResource, final ISharedMetadataView commonMetadata) {
        // Check Managed Beans
        if (jAnnotationResource.getLookup() == null) {
            for (IClassMetadata iCommonClassMetadata : this.getClassMetadataCollection()) {
                if (jAnnotationResource.getType() != null &&
                        iCommonClassMetadata.getJClass().getName().equals(jAnnotationResource.getType().replace(".", "/"))) {
                    String managedBeanName = iCommonClassMetadata.as(CommonView.class).getManagedBeanName();
                    if (managedBeanName != null) {
                        // Add a lookup to the ManagedBean
                        jAnnotationResource.setLookup("java:module/".concat(managedBeanName));
                    }
                }
            }
        }
        this.bindingHolder.addResourceBinding(new ENCBinding<IJAnnotationResource>(jAnnotationResource.getName(),
                jAnnotationResource));
        addFieldOrMethodENC(jAnnotationResource.getName(), commonMetadata);
    }

    /**
     * Adds the given EJB to the list of bindings.
     * @param jEjbEJB the given EJB.
     */
    protected void addEJB(final IJEjbEJB jEjbEJB, final ISharedMetadataView commonMetadata) {
        this.bindingHolder.addEJBBinding(new ENCBinding<IJEjbEJB>(jEjbEJB.getName(), jEjbEJB));
        addFieldOrMethodENC(jEjbEJB.getName(), commonMetadata);
    }

    /**
     * Adds the given WebServiceRef to the list of bindings.
     * @param ref the given WebServiceRef.
     */
    protected void addWebServiceRef(final IJaxwsWebServiceRef ref, final ISharedMetadataView commonMetadata) {
        this.bindingHolder.addWebServicesBinding(new ENCBinding<IJaxwsWebServiceRef>(ref.getName(), ref));
        addFieldOrMethodENC(ref.getName(), commonMetadata);
    }

    protected void addDataSource(final IJAnnotationSqlDataSourceDefinition dataSourceDefinition) {
        this.bindingHolder.addDataSourceBinding(
            new ENCBinding<IJAnnotationSqlDataSourceDefinition>(dataSourceDefinition.getName(), dataSourceDefinition));
    }

    /**
     * Add the given ENC name for the given field or method.
     * @param encName the name of the ENC value
     * @param commonMetadata the field or method or null object
     */
    protected void addFieldOrMethodENC(final String encName, final ISharedMetadataView commonMetadata) {

        // Don't do anything if there is no field/method
        if (commonMetadata == null) {
            return;
        }

        // Add ENC injection target
        if (commonMetadata instanceof IFieldMetadata) {
            IFieldMetadata fieldAnnotationMetadata = (IFieldMetadata) commonMetadata;
            String fieldName = fieldAnnotationMetadata.getJField().getName();
            this.bindingHolder.addFieldInjectionTarget(fieldAnnotationMetadata.getClassMetadata().getJClass().getName().replace('/', '.'), new InjectionTargetEncImpl(fieldName, encName));

        } else if (commonMetadata instanceof IMethodMetadata) {
            IMethodMetadata methodAnnotationMetadata = (IMethodMetadata) commonMetadata;
            String methodName = methodAnnotationMetadata.getJMethod().getName();
            this.bindingHolder.addMethodInjectionTarget(methodAnnotationMetadata.getClassMetadata().getJClass().getName().replace('/', '.'), new InjectionTargetEncImpl(methodName, encName));

        }
    }

    /**
     * @return the Holder of all ENC bindings.
     */
    protected ENCBindingHolder getHolder() {
        return this.bindingHolder;
    }

}
