/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.common.impl.enc;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.ow2.util.ee.metadata.common.api.enc.IENCBinding;
import org.ow2.util.ee.metadata.common.api.enc.IENCBindingHolder;
import org.ow2.util.ee.metadata.common.api.enc.IInjectionTargetEnc;
import org.ow2.util.ee.metadata.common.api.struct.IJAnnotationManagedBean;
import org.ow2.util.ee.metadata.common.api.struct.IJAnnotationResource;
import org.ow2.util.ee.metadata.common.api.struct.IJAnnotationSqlDataSourceDefinition;
import org.ow2.util.ee.metadata.common.api.struct.IJEjbEJB;
import org.ow2.util.ee.metadata.common.api.struct.IJavaxPersistenceContext;
import org.ow2.util.ee.metadata.common.api.struct.IJavaxPersistenceUnit;
import org.ow2.util.ee.metadata.common.api.struct.IJaxwsWebServiceRef;

/**
 * Class that can manage different kind of binding used in ENC.
 * @author Florent BENOIT
 */
public class ENCBindingHolder implements IENCBindingHolder {

    /**
     * List of field injection targets for each class.
     */
    private Map<String, List<IInjectionTargetEnc>> fieldInjections = null;

    /**
     * List of method injection targets for each class.
     */
    private Map<String, List<IInjectionTargetEnc>> methodInjections = null;

    /**
     * List of binding for Persistence context (Entity Manager).
     */
    private List<IENCBinding<IJavaxPersistenceContext>> persistenceContextBindings = null;

    /**
     * List of binding for Persistence unit (Entity Manager Factory).
     */
    private List<IENCBinding<IJavaxPersistenceUnit>> persistenceUnitBindings = null;

    /**
     * List of binding for javax.Annotation.Resources.
     */
    private Map<String, IENCBinding<IJAnnotationResource>> resourceBindings = null;

    /**
     * List of binding for javax.ejb.EJB(s).
     */
    private List<IENCBinding<IJEjbEJB>> ejbBindings = null;

    /**
     * List of binding for javax.ws.jaxws.WebServiceRef(s).
     */
    private List<IENCBinding<IJaxwsWebServiceRef>> wsBindings = null;

    private List<IENCBinding<IJAnnotationSqlDataSourceDefinition>> dataSourceBindings = null;

    private List<IENCBinding<IJAnnotationManagedBean>> managedBeanBindings = null;

    /**
     * Default constructor.
     */
    public ENCBindingHolder() {
        this.persistenceContextBindings = new ArrayList<IENCBinding<IJavaxPersistenceContext>>();
        this.persistenceUnitBindings = new ArrayList<IENCBinding<IJavaxPersistenceUnit>>();
        this.resourceBindings = new HashMap<String, IENCBinding<IJAnnotationResource>>();
        this.ejbBindings = new ArrayList<IENCBinding<IJEjbEJB>>();
        this.wsBindings = new ArrayList<IENCBinding<IJaxwsWebServiceRef>>();
        this.fieldInjections = new HashMap<String, List<IInjectionTargetEnc>>();
        this.methodInjections = new HashMap<String, List<IInjectionTargetEnc>>();
        this.dataSourceBindings = new ArrayList<IENCBinding<IJAnnotationSqlDataSourceDefinition>>();
        this.managedBeanBindings = new ArrayList<IENCBinding<IJAnnotationManagedBean>>();
    }

    /**
     * Add a given persistence context binding.
     * @param binding the given binding.
     */
    public void addPersistenceContextBinding(final IENCBinding<IJavaxPersistenceContext> binding) {
        this.persistenceContextBindings.add(binding);
    }

    /**
     * @return the list of persistence context bindings.
     */
    public List<IENCBinding<IJavaxPersistenceContext>> getPersistenceContextBindings() {
        return persistenceContextBindings;
    }

    /**
     * Add a given persistence unit binding.
     * @param binding the given binding.
     */
    public void addPersistenceUnitBinding(final IENCBinding<IJavaxPersistenceUnit> binding) {
        this.persistenceUnitBindings.add(binding);
    }

    /**
     * @return the list of persistence unit bindings.
     */
    public List<IENCBinding<IJavaxPersistenceUnit>> getPersistenceUnitBindings() {
        return persistenceUnitBindings;
    }

    /**
     * Add a given resource binding.
     * @param binding the given binding.
     */
    public void addResourceBinding(final IENCBinding<IJAnnotationResource> binding) {
        // Check already exists ?
        String bindingName = binding.getName();

        IENCBinding<IJAnnotationResource> existingResource = resourceBindings.get(bindingName);
        // Already defined, ignore it
        if (existingResource != null) {
            return;
        }

        this.resourceBindings.put(bindingName, binding);
    }
    /**
     * @return the list of resource bindings.
     */
    public List<IENCBinding<IJAnnotationResource>> getResourceBindings() {
        Collection<IENCBinding<IJAnnotationResource>> collection = resourceBindings.values();
        List<IENCBinding<IJAnnotationResource>> list = new ArrayList<IENCBinding<IJAnnotationResource>>();
        if (collection != null) {
            for (IENCBinding<IJAnnotationResource> item : collection) {
                list.add(item);
            }
        }
        return list;
    }

    /**
     * Add a given EJB binding.
     * @param binding the given binding.
     */
    public void addEJBBinding(final IENCBinding<IJEjbEJB> binding) {
        this.ejbBindings.add(binding);
    }
    /**
     * @return the list of EJB bindings.
     */
    public List<IENCBinding<IJEjbEJB>> getEJBBindings() {
        return ejbBindings;
    }

    /**
     * Add a given WS binding.
     * @param binding the given binding.
     */
    public void addWebServicesBinding(final IENCBinding<IJaxwsWebServiceRef> binding) {
        this.wsBindings.add(binding);
    }
    /**
     * @return the list of WS bindings.
     */
    public List<IENCBinding<IJaxwsWebServiceRef>> getWebServicesBindings() {
        return wsBindings;
    }

    /**
     * Add a given field injection target for a given classname.
     * @param className the given class
     * @param injectionTargetEnc the given field injection target.
     */
    public void addFieldInjectionTarget(final String className, final IInjectionTargetEnc injectionTargetEnc) {
        List<IInjectionTargetEnc> fieldInjectionTargets = fieldInjections.get(className);
        // Init list if not already done
        if (fieldInjectionTargets == null) {
            fieldInjectionTargets = new ArrayList<IInjectionTargetEnc>();
            fieldInjections.put(className, fieldInjectionTargets);
        }

        // add in the list
        fieldInjectionTargets.add(injectionTargetEnc);
    }

    /**
     * Add a given method injection target for a given classname.
     * @param className the given class
     * @param injectionTargetEnc the given method injection target.
     */
    public void addMethodInjectionTarget(final String className, final IInjectionTargetEnc injectionTargetEnc) {
        List<IInjectionTargetEnc> methodInjectionTargets = methodInjections.get(className);
        // Init list if not already done
        if (methodInjectionTargets == null) {
            methodInjectionTargets = new ArrayList<IInjectionTargetEnc>();
            methodInjections.put(className, methodInjectionTargets);
        }

        // add in the list
        methodInjectionTargets.add(injectionTargetEnc);
    }


    /**
     * @param className the name of the class for which the fields are requested
     * @return the injection target enc objects for a given classname
     */
    public List<IInjectionTargetEnc> getFieldInjectionTargets(final String className) {
        return fieldInjections.get(className);
    }

    /**
     * @param className the name of the class for which the methods are requested
     * @return the injection target enc objects for a given classname
     */
    public List<IInjectionTargetEnc> getMethodInjectionTargets(final String className) {
        return methodInjections.get(className);
    }

    /**
     * List of fields/method that needs to be set for a given class.
     * @return a map with key = classname and then map = field/attribute name <--> value
     */
    public Map<String, Map<String, String>> getInjectionMap() {
        // Init map
        Map<String, Map<String, String>> injectionsMap = new HashMap<String, Map<String, String>>();

        // Iterate on fields
        completeInjectionMap(injectionsMap, fieldInjections.entrySet(), false);

        // Iterate on methods
        completeInjectionMap(injectionsMap, methodInjections.entrySet(), true);

        return injectionsMap;
    }

    public void addDataSourceBinding(final IENCBinding<IJAnnotationSqlDataSourceDefinition> dataSourceDefinitionENCBinding) {
        dataSourceBindings.add(dataSourceDefinitionENCBinding);
    }

    public List<IENCBinding<IJAnnotationSqlDataSourceDefinition>> getDataSourceBindings() {
        return dataSourceBindings;
    }

    /**
     * Fill the given map with the list of injection objects.
     * @param injectionsMap the map to complete
     * @param injections the set of injection
     * @param isMethod true if injection is for methods, false for fields
     */
    protected void completeInjectionMap(final Map<String, Map<String, String>> injectionsMap, final Set<Map.Entry<String, List<IInjectionTargetEnc>>> injections, final boolean isMethod) {
        for (Map.Entry<String, List<IInjectionTargetEnc>> injectionForClass : injections) {
            String classname = injectionForClass.getKey();
            // Key is the class name
            Map<String, String> injectionForGivenClass = injectionsMap.get(classname);
            // Not existing, add the entry
            if (injectionForGivenClass == null) {
                injectionForGivenClass = new HashMap<String, String>();
                injectionsMap.put(classname, injectionForGivenClass);
            }
            // Add entry
            for (IInjectionTargetEnc injectionTargetEnc : injectionForClass.getValue()) {
                if (isMethod) {
                    // Remove set prefix
                    injectionForGivenClass.put(injectionTargetEnc.getName().substring(4), injectionTargetEnc.getEncName());
                } else {
                    injectionForGivenClass.put(injectionTargetEnc.getName(), injectionTargetEnc.getEncName());
                }
            }

        }
    }


}
