/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.common.impl.helper;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import org.ow2.util.asm.Type;
import org.ow2.util.scan.api.metadata.IMethodMetadata;
import org.ow2.util.scan.api.metadata.structures.IMethod;

/**
 * Allow to load a java.lang.reflect.method from its metadata.
 * @author Florent Benoit
 */
public final class MethodHelper {

    /**
     * Utility class.
     */
    private MethodHelper() {

    }

    /**
     * Gets a signature entry for a given method.
     * @param methodMetadata the metadata used to extract signature
     * @return a signature string
     */
    public static String getSignature(final IMethodMetadata methodMetadata) {
        return methodMetadata.getJMethod().getName() + methodMetadata.getJMethod().getDescriptor().hashCode();
    }


    /**
     * Allow to get a reflect method from its metadata.
     * @param jMethod the method info
     * @param clazz the class that is containing the method
     * @return an instance of the method
     */
    public static Method getMethod(final IMethod jMethod, final Class<?> clazz) {
        // Get method name
        String methodName = jMethod.getName();
        // Get arguments
        Type[] argumentTypes = Type.getArgumentTypes(jMethod.getDescriptor());

        // Transform string arguments into class objects
        Class<?>[] parameterTypes = null;
        if (argumentTypes != null) {
            List<Class<?>> parameters = new ArrayList<Class<?>>();
            for (Type type : argumentTypes) {
                try {
                    String parameterClassName = type.getClassName().replace("/", ".");
                    Class<?> parameterClass = null;
                    // load parameters
                    if ("int".equals(parameterClassName)) {
                        parameterClass = int.class;
                    } else if ("byte".equals(parameterClassName)) {
                        parameterClass = byte.class;
                    } else if ("char".equals(parameterClassName)) {
                        parameterClass = char.class;
                    } else if ("long".equals(parameterClassName)) {
                        parameterClass = long.class;
                    } else if ("short".equals(parameterClassName)) {
                        parameterClass = short.class;
                    } else if ("float".equals(parameterClassName)) {
                        parameterClass = float.class;
                    } else if ("double".equals(parameterClassName)) {
                        parameterClass = double.class;
                    } else if ("boolean".equals(parameterClassName)) {
                        parameterClass = boolean.class;
                    } else if (Type.ARRAY == type.getSort()) {
                        parameterClass = Class.forName(type.getDescriptor().replace('/', '.'), true, clazz.getClassLoader());
                    } else {
                        parameterClass = clazz.getClassLoader().loadClass(parameterClassName);
                    }
                    parameters.add(parameterClass);
                } catch (ClassNotFoundException e) {
                    throw new IllegalStateException("Unable to load the class '" + type.getClassName() + "'.", e);
                }
            }
            parameterTypes = parameters.toArray(new Class[parameters.size()]);
        }

        // Load the method
        try {
            return clazz.getDeclaredMethod(methodName, parameterTypes);
        } catch (SecurityException e) {
            throw new IllegalStateException("Unable to get the method '" + methodName + "'.", e);
        } catch (NoSuchMethodException e) {
            throw new IllegalStateException("Unable to get the method '" + methodName + "'.", e);
        }
    }

    /**
     * Allow to get a reflect method from its metadata.
     * @param classname the name of the class that is containing the method
     * @param jMethod the method info
     * @param classLoader the classloader used to load the method's class
     * @return an instance of the method
     */
    public static Method getMethod(final String classname, final IMethod jMethod, final ClassLoader classLoader) {
        // load the class
        Class<?> clazz;
        try {
            clazz = classLoader.loadClass(classname.replace("/", "."));
        } catch (ClassNotFoundException e) {
            throw new IllegalStateException("Unable to load the class '" + classname + "'.", e);
        }

        return getMethod(jMethod, clazz);
    }

}
