/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.common.impl.struct;

import javax.annotation.Resource.AuthenticationType;

import org.ow2.util.ee.metadata.common.api.struct.IJAnnotationResource;


/**
 * Acts as an implementation of &#64;{@link javax.annotation.Resource} annotation.
 * @author Florent Benoit
 */
public class JAnnotationResource implements IJAnnotationResource {

    /**
     * Serial Version UID.
     */
    private static final long serialVersionUID = 8392442594223133257L;

    /**
     * Name.
     */
    private String name = null;

    /**
     * Type (class).
     */
    private String type = null;

    /**
     * Authentication type.
     */
    private AuthenticationType authenticationType = null;

    /**
     * Shareable (true/false).
     */
    private boolean shareable;

    /**
     * Description.
     */
    private String description = null;

    /**
     * lookup name.
     */
    private String lookup = null;

    /**
     * mapped name.
     */
    private String mappedName = null;

    /**
     * Message Destination Link.
     */
    private String messageDestinationLink = null;

    /**
     * Constructor.<br>
     * Build object with default values.
     */
    public JAnnotationResource() {
        // set default values
        this.name = "";
        this.type = "java.lang.Object";
        this.authenticationType = AuthenticationType.CONTAINER;
        shareable = true;
        description = "";
    }

    /**
     * @return Name (resource to be looked up).
     */
    public String getName() {
        return name;
    }

    /**
     * Sets Lookup Name (resource to be looked up).
     * @param lookup the given name.
     */
    public void setLookup(final String lookup) {
        this.lookup = lookup;
    }

    /**
     * @return Lookup Name (resource to be looked up).
     */
    public String getLookup() {
        return lookup;
    }

    /**
     * Sets Name (resource to be looked up).
     * @param name the given name.
     */
    public void setName(final String name) {
        this.name = name;
    }

    /**
     * @return the authentication type.
     */
    public AuthenticationType getAuthenticationType() {
        return authenticationType;
    }


    /**
     * Sets the authentication type.
     * @param authenticationType value to set.
     */
    public void setAuthenticationType(final AuthenticationType authenticationType) {
        this.authenticationType = authenticationType;
    }


    /**
     * @return the description.
     */
    public String getDescription() {
        return description;
    }


    /**
     * Sets the description.
     * @param description value to set.
     */
    public void setDescription(final String description) {
        this.description = description;
    }


    /**
     * @return true if it is shareable.
     */
    public boolean isShareable() {
        return shareable;
    }


    /**
     * Sets the shareable attribute (false/true).
     * @param shareable a boolean.
     */
    public void setShareable(final boolean shareable) {
        this.shareable = shareable;
    }


    /**
     * @return the type of resource (class).
     */
    public String getType() {
        return type;
    }


    /**
     * Sets the class type of this object.
     * @param type the class value (as string format).
     */
    public void setType(final String type) {
        this.type = type;
    }

    /**
     * @return MappedName.
     */
    public String getMappedName() {
        return mappedName;
    }

    /**
     * Sets mapped Name.
     * @param mappedName the given mappedName.
     */
    public void setMappedName(final String mappedName) {
        this.mappedName = mappedName;
    }

    /**
     * @return the message-Destination-Link.
     */
    public String getMessageDestinationLink() {
        return messageDestinationLink;
    }

    /**
     * Sets the message destination link.
     * @param messageDestinationLink the link value
     */
    public void setMessageDestinationLink(final String messageDestinationLink) {
        this.messageDestinationLink = messageDestinationLink;
    }

    /**
     * @return string representation
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        // classname
        sb.append(this.getClass().getName().substring(this.getClass().getPackage().getName().length() + 1));

        // name
        sb.append("[name=");
        sb.append(name);

        // type
        sb.append(", type=");
        sb.append(type);

        // Authentication type
        sb.append(", authenticationType=");
        sb.append(authenticationType);

        // Shareable
        sb.append(", shareable=");
        sb.append(shareable);

        // Description
        sb.append(", description=");
        sb.append(description);

        // Message Destination Link ?
        if (messageDestinationLink != null) {
            sb.append(", messageDestinationLink=");
            sb.append(messageDestinationLink);
        }

        // lookup
        if (lookup != null) {
            sb.append(", lookup=");
            sb.append(lookup);
        }

        // MappedName
        sb.append(", mappedName=");
        sb.append(mappedName);

        sb.append("]");
        return sb.toString();
    }

}
