/**
 * OW2 Utilities
 * Copyright (C) 2009-2012 Bull S.A.S.
 * Contact: utilities@ow2.org
 *
 * Licensed  under the  Apache License,  Version 2.0  (the "License");
 * you may not use  this file  except in  compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed  under the  License is distributed on an "AS IS" BASIS,
 * WITHOUT  WARRANTIES OR CONDITIONS  OF ANY KIND, either  express  or
 * implied.
 *
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.common.impl.struct;

import org.ow2.util.ee.metadata.common.api.struct.IJAnnotationSqlDataSourceDefinition;

/**
 *  Acts as an implementation of &#64;{@link javax.annotation.sql.DataSourceDefinition} annotation.
 * @author Loic Albertin
 */
public class JAnnotationSqlDataSourceDefinition implements IJAnnotationSqlDataSourceDefinition {

    private static final long serialVersionUID = 2805923776509918391L;
    /**
     * JNDI name by which the data source name will be registered.
     */
    private String name = null;

    /**
     * DataSource implementation class className.
     */
    private String className = null;

    /**
     * Description of the data source.
     */
    private String description = "";

    /**
     * A JDBC url. If the url property is specified along with other standard DataSource properties such as
     * serverName and
     * portNumber, the more specific properties will take precedence and the url will be ignored
     */
    private String url = "";

    /**
     * User name for connection authentications
     */
    private String user = "";

    /**
     * password for connection authentications
     */
    private String password = "";

    /**
     * Name of a database on a server
     */
    private String databaseName = "";

    /**
     * Port number where a server is listening for requests
     */
    private int portNumber = -1;

    /**
     * Database server name
     */
    private String serverName = "localhost";

    /**
     * Isolation level for connections.
     */
    private int isolationLevel = -1;

    /**
     * Indicates whether a connection is transactional or not.
     */
    private boolean transactional = true;

    /**
     * Number of connections that should be created when a connection pool is initialized
     */
    private int initialPoolSize = -1;

    /**
     * Maximum number of connections that should be concurrently allocated for a connection pool
     */
    private int maxPoolSize = -1;

    /**
     * Minimum number of connections that should be allocated for a connection pool
     */
    private int minPoolSize = -1;

    /**
     * The number of seconds that a physical connection should remain unused in the pool before the connection is
     * closed for a
     * connection pool
     */
    private int maxIdleTime = -1;

    /**
     * The total number of statements that a connection pool should keep open. A value of 0 indicates that the
     * caching of
     * statements is disabled for a connection pool.
     */
    int maxStatements = -1;

    /**
     * Used to specify vendor specific properties and less commonly used DataSource properties.
     */
    private String[] properties = {};

    /**
     * The maximum time in seconds that this data source will wait while attempting to connect to a database. A value
     * of 0
     * specifies that the timeout is the = system timeout if there is one, otherwise it specifies that there is no
     * timeout
     */
    private int loginTimeout = 0;


    public JAnnotationSqlDataSourceDefinition() {
    }

    public String getClassName() {
        return className;
    }

    public void setClassName(String className) {
        this.className = className;
    }

    public String getDatabaseName() {
        return databaseName;
    }

    public void setDatabaseName(String databaseName) {
        this.databaseName = databaseName;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public int getInitialPoolSize() {
        return initialPoolSize;
    }

    public void setInitialPoolSize(int initialPoolSize) {
        this.initialPoolSize = initialPoolSize;
    }

    public int getIsolationLevel() {
        return isolationLevel;
    }

    public void setIsolationLevel(int isolationLevel) {
        this.isolationLevel = isolationLevel;
    }

    public int getLoginTimeout() {
        return loginTimeout;
    }

    public void setLoginTimeout(int loginTimeout) {
        this.loginTimeout = loginTimeout;
    }

    public int getMaxIdleTime() {
        return maxIdleTime;
    }

    public void setMaxIdleTime(int maxIdleTime) {
        this.maxIdleTime = maxIdleTime;
    }

    public int getMaxPoolSize() {
        return maxPoolSize;
    }

    public void setMaxPoolSize(int maxPoolSize) {
        this.maxPoolSize = maxPoolSize;
    }

    public int getMaxStatements() {
        return maxStatements;
    }

    public void setMaxStatements(int maxStatements) {
        this.maxStatements = maxStatements;
    }

    public int getMinPoolSize() {
        return minPoolSize;
    }

    public void setMinPoolSize(int minPoolSize) {
        this.minPoolSize = minPoolSize;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public int getPortNumber() {
        return portNumber;
    }

    public void setPortNumber(int portNumber) {
        this.portNumber = portNumber;
    }

    public String[] getProperties() {
        return properties;
    }

    public void setProperties(String[] properties) {
        this.properties = properties;
    }

    public String getServerName() {
        return serverName;
    }

    public void setServerName(String serverName) {
        this.serverName = serverName;
    }

    public boolean isTransactional() {
        return transactional;
    }

    public void setTransactional(boolean transactional) {
        this.transactional = transactional;
    }

    public String getUrl() {
        return url;
    }

    public void setUrl(String url) {
        this.url = url;
    }

    public String getUser() {
        return user;
    }

    public void setUser(String user) {
        this.user = user;
    }

    /**
     * @return string representation
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        // classname
        sb.append(this.getClass().getName().substring(this.getClass().getPackage().getName().length() + 1));

        // name
        sb.append("[name=");
        sb.append(name);

        // className
        sb.append(", className=");
        sb.append(className);

        // description
        sb.append(", description=");
        sb.append(description);

        // url
        sb.append(", url=");
        sb.append(url);

        // user
        sb.append(", user=");
        sb.append(user);

        // password
        sb.append(", password=");
        sb.append(password);

        // databaseName
        sb.append(", databaseName=");
        sb.append(databaseName);

        // portNumber
        sb.append(", portNumber=");
        sb.append(portNumber);

        // serverName
        sb.append(", serverName=");
        sb.append(serverName);

        // isolationLevel
        sb.append(", isolationLevel=");
        sb.append(isolationLevel);

        // transactional
        sb.append(", transactional=");
        sb.append(transactional);

        // initialPoolSize
        sb.append(", initialPoolSize=");
        sb.append(initialPoolSize);

        // maxPoolSize
        sb.append(", maxPoolSize=");
        sb.append(maxPoolSize);


        // minPoolSize
        sb.append(", minPoolSize=");
        sb.append(minPoolSize);

        // maxIdleTime
        sb.append(", maxIdleTime=");
        sb.append(maxIdleTime);

        // maxStatements
        sb.append(", maxStatements=");
        sb.append(maxStatements);

        // properties
        sb.append(", properties=[");
        boolean first = true;
        for (String property : properties) {
            if (!first) {
                sb.append(", ");
            }
            sb.append(property);
            first = false;
        }
        sb.append("]");

        // loginTimeout
        sb.append(", loginTimeout=");
        sb.append(loginTimeout);


        sb.append("]");
        return sb.toString();
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }

        JAnnotationSqlDataSourceDefinition that = (JAnnotationSqlDataSourceDefinition) o;

        return !(name != null ? !name.equals(that.name) : that.name != null);

    }

    @Override
    public int hashCode() {
        return name != null ? name.hashCode() : 0;
    }
}
