/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.common.impl.struct;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.ow2.util.ee.metadata.common.api.struct.IJaxwsWebServiceRef;
import org.ow2.util.ee.metadata.common.api.xml.struct.IHandlerChains;
import org.ow2.util.ee.metadata.common.api.xml.struct.IPortComponentRef;

/**
 * This class allow to set informations on javax.xml.ws.WebServiceRef
 * annotation.
 * @author Guillaume Sauthier
 */
public class JaxwsWebServiceRef implements IJaxwsWebServiceRef {

    /**
     * Serial Version UID.
     */
    private static final long serialVersionUID = -1521926066946371643L;

    /**
     * Stores the 'mappedName' attribute.
     */
    private String mappedName;

    /**
     * Stores the 'name' attribute.
     */
    private String name;

    /**
     * Stores the 'type' attribute.
     */
    private String type;

    /**
     * Stores the 'value' attribute.
     */
    private String value;

    /**
     * Stores the 'wsdlLocation' attribute.
     */
    private String wsdlLocation;

    /**
     * The @HandlerChain.file attribute (if any).
     */
    private String handlerChainFile;

    /**
     * Declaring classname.
     */
    private String declaringClassname;

    /**
     * handlers-chains defined in the standard descriptor.
     */
    private IHandlerChains handlerChains;

    /**
     * Map between SEI <---> Port Component Ref.
     */
    private Map<String, IPortComponentRef> portComponentRefs = null;

    /**
     * Default constructor.
     */
    public JaxwsWebServiceRef() {
        this.portComponentRefs = new HashMap<String, IPortComponentRef>();
    }


    public String getMappedName() {
        return mappedName;
    }

    public void setMappedName(final String mappedName) {
        this.mappedName = mappedName;
    }

    public String getName() {
        return name;
    }

    public void setName(final String name) {
        this.name = name;
    }

    public String getType() {
        return type;
    }

    public void setType(final String type) {
        this.type = type;
    }

    public String getValue() {
        return value;
    }

    public void setValue(final String value) {
        this.value = value;
    }

    public String getWsdlLocation() {
        return wsdlLocation;
    }

    public void setWsdlLocation(final String wsdlLocation) {
        this.wsdlLocation = wsdlLocation;
    }

    /**
     * Store the value of the @HandlerChain.file attribute which is collocated
     * with this @WebServiceRef.
     *
     * @param fileAttribute attribute's value
     */
    public void setHandlerChainFile(final String fileAttribute) {
        this.handlerChainFile = fileAttribute;
    }

    /**
     * @return the value of the @HandlerChain.file attribute.
     */
    public String getHandlerChainFile() {
        return this.handlerChainFile;
    }

    /**
     *
     * @return handlers-chains defined in the standard descriptor
     *         (may be null, have precedence over @HandlerChain.file).
     */
    public IHandlerChains getHandlerChains() {
        return handlerChains;
    }

    /**
     * Set the handler-chains defined in the standard deployment descriptor.
     * @param handlerChains handler-chains defined in the standard deployment descriptor.
     */
    public void setHandlerChains(final IHandlerChains handlerChains) {
        this.handlerChains = handlerChains;
    }

    /**
     * Stores all port component refs.
     * @param portComponentRefs list of port component ref.
     */
    public void setPortComponentRefs(final List<IPortComponentRef> portComponentRefs) {
        if (portComponentRefs == null) {
            return;
        }
        // Add each port component ref
        for (IPortComponentRef portComponentRef : portComponentRefs) {
            this.portComponentRefs.put(portComponentRef.getServiceEndpointInterface(), portComponentRef);
        }

    }

    /**
     * @return the value of the list of port component refs.
     */
    public List<IPortComponentRef> getPortComponentRefs() {
        return new ArrayList<IPortComponentRef>(portComponentRefs.values());
    }

    /**
     * Gets a port component ref by providings its service endpoint interface.
     * @param serviceEndpointInterface the given SEI
     * @return a port component ref or null if not found
     */
    public IPortComponentRef getPortComponentRef(final String serviceEndpointInterface) {
        return portComponentRefs.get(serviceEndpointInterface);
    }

    /**
     * @param declaringClassname Class declaring the @HandlerChain annotation.
     */
    public void setDeclaringClass(String declaringClassname) {
        this.declaringClassname = declaringClassname;
    }

    /**
     * @return Name of the class declaring the @HandlerChain annotation (will be
     *         null if no @HC annotation is present).
     */
    public String getDeclaringClass() {
        return this.declaringClassname;
    }
}
