/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.common.impl.xml.merge;

import static org.ow2.util.ee.metadata.common.api.xml.struct.ILifeCycleCallback.POST_CONSTRUCT;
import static org.ow2.util.ee.metadata.common.api.xml.struct.ILifeCycleCallback.PRE_DESTROY;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.ow2.util.ee.metadata.common.api.interfaces.IEnvironmentRefAccessor;
import org.ow2.util.ee.metadata.common.api.interfaces.ISharedMetadataView;
import org.ow2.util.ee.metadata.common.api.struct.IEnvEntry;
import org.ow2.util.ee.metadata.common.api.struct.IJAnnotationResource;
import org.ow2.util.ee.metadata.common.api.struct.IJAnnotationSqlDataSourceDefinition;
import org.ow2.util.ee.metadata.common.api.struct.IJEjbEJB;
import org.ow2.util.ee.metadata.common.api.struct.IJavaxPersistenceContext;
import org.ow2.util.ee.metadata.common.api.struct.IJavaxPersistenceContextType;
import org.ow2.util.ee.metadata.common.api.struct.IJavaxPersistenceUnit;
import org.ow2.util.ee.metadata.common.api.struct.IJaxwsWebServiceRef;
import org.ow2.util.ee.metadata.common.api.struct.IJwsHandlerChain;
import org.ow2.util.ee.metadata.common.api.xml.struct.IAroundInvoke;
import org.ow2.util.ee.metadata.common.api.xml.struct.ICommonEJBRef;
import org.ow2.util.ee.metadata.common.api.xml.struct.IDataSource;
import org.ow2.util.ee.metadata.common.api.xml.struct.IEJBLocalRef;
import org.ow2.util.ee.metadata.common.api.xml.struct.IEJBRef;
import org.ow2.util.ee.metadata.common.api.xml.struct.IEnvironment;
import org.ow2.util.ee.metadata.common.api.xml.struct.IInjectionTarget;
import org.ow2.util.ee.metadata.common.api.xml.struct.ILifeCycleCallback;
import org.ow2.util.ee.metadata.common.api.xml.struct.IMessageDestinationRef;
import org.ow2.util.ee.metadata.common.api.xml.struct.IPersistenceContextRef;
import org.ow2.util.ee.metadata.common.api.xml.struct.IPersistenceUnitRef;
import org.ow2.util.ee.metadata.common.api.xml.struct.IResourceEnvRef;
import org.ow2.util.ee.metadata.common.api.xml.struct.IResourceRef;
import org.ow2.util.ee.metadata.common.api.xml.struct.IServiceRef;
import org.ow2.util.ee.metadata.common.impl.struct.JAnnotationResource;
import org.ow2.util.ee.metadata.common.impl.struct.JAnnotationSqlDataSourceDefinition;
import org.ow2.util.ee.metadata.common.impl.struct.JEjbEJB;
import org.ow2.util.ee.metadata.common.impl.struct.JavaxPersistenceContext;
import org.ow2.util.ee.metadata.common.impl.struct.JavaxPersistenceUnit;
import org.ow2.util.ee.metadata.common.impl.struct.JaxwsWebServiceRef;
import org.ow2.util.ee.metadata.common.impl.view.CommonView;
import org.ow2.util.ee.metadata.common.impl.xml.struct.AroundInvoke;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.scan.api.metadata.IClassMetadata;
import org.ow2.util.scan.api.metadata.IFieldMetadata;
import org.ow2.util.scan.api.metadata.IMetadata;
import org.ow2.util.scan.api.metadata.IMethodMetadata;

/**
 * Perform a common merge from XML files to metadata.
 * @author Gael Lalire
 */
public abstract class MetadataMerge {

    /**
     * Defines java.lang.Object class.
     */
    public static final String JAVA_LANG_OBJECT = "java/lang/Object";

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(MetadataMerge.class);

    /**
     * Apply rules for a an environment.
     * @param envGroup the struct (XML DD)
     * @param classAnnotationMetadata (Annotation metadata)
     */
    protected void applyJndiEnvironmentRefsGroup(final IEnvironment envGroup,
            final IEnvironmentRefAccessor classAnnotationMetadata) {

        // post-construct / pre-destroy
        applyLifeCycle(envGroup.getPostConstructCallbackList(), classAnnotationMetadata, POST_CONSTRUCT);
        applyLifeCycle(envGroup.getPreDestroyCallbackList(), classAnnotationMetadata, PRE_DESTROY);

        // ejb(local) ref
        applyEjbRef(envGroup.getEJBRefList(), classAnnotationMetadata);
        applyEjbLocalRef(envGroup.getEJBLocalRefList(), classAnnotationMetadata);

        // resource(env) ref
        applyResourceRef(envGroup.getResourceRefList(), classAnnotationMetadata);
        applyResourceEnvRef(envGroup.getResourceEnvRefList(), classAnnotationMetadata);

        // message-destination-ref
        applyMessageDestinationRef(envGroup.getMessageDestinationRefList(), classAnnotationMetadata);

        // env entry
        applyEnvEntry(envGroup.getEnvEntryList(), classAnnotationMetadata);

        // persistence unit ref
        applyPersistenceUnitRef(envGroup.getPersistenceUnitRefList(), classAnnotationMetadata);

        // persistence context ref
        applyPersistenceContextRef(envGroup.getPersistenceContextRefList(), classAnnotationMetadata);

        // service-ref
        applyServiceRef(envGroup.getServiceRefList(), classAnnotationMetadata);

        // Merge @HandlerChain annotations in their @WebServiceRef
        applyHandlerChain();

        //Apply merge on data sources
        applyDataSource(envGroup.getDataSourceList(), classAnnotationMetadata);

    }

    /**
     * Apply rules for a common bean (persistence-unit-ref).
     * @param persistenceUnitRefList the list of persistence-unit-ref
     * @param classAnnotationMetadata (Annotation metadata) of the bean using
     *        this interceptor (may be null)
     */
    private void applyPersistenceUnitRef(final List<IPersistenceUnitRef> persistenceUnitRefList,
            final IEnvironmentRefAccessor classAnnotationMetadata) {
        if (persistenceUnitRefList == null) {
            return;
        }

        // For each persistence-unit-ref, build a &#64;{@link javax.persistence.PersistenceUnit}
        for (IPersistenceUnitRef persistenceUnitRef : persistenceUnitRefList) {
            JavaxPersistenceUnit jPersistenceUnit =  new JavaxPersistenceUnit();
            // set ref-name
            jPersistenceUnit.setName(persistenceUnitRef.getPersistenceUnitRefName());

            // set unit-name
            jPersistenceUnit.setUnitName(persistenceUnitRef.getPersistenceUnitName());

            applyJPersistenceUnit(jPersistenceUnit, classAnnotationMetadata, persistenceUnitRef.getInjectionTargetList());
        }
    }

    /**
     * Apply rules to define a data source (data-source).
     * @param dataSourceList the list of persistence-unit-ref
     * @param classAnnotationMetadata (Annotation metadata) of the bean using
     *        this interceptor (may be null)
     */
    private void applyDataSource(final List<IDataSource> dataSourceList,
                                 final IEnvironmentRefAccessor classAnnotationMetadata) {
        if (dataSourceList == null) {
            return;
        }
        if (classAnnotationMetadata.getJAnnotationSqlDataSourceDefinitions() == null) {
            classAnnotationMetadata.setJAnnotationSqlDataSourceDefinitions(
                    new ArrayList<IJAnnotationSqlDataSourceDefinition>());
        }
        for (IDataSource dataSource : dataSourceList) {
            IJAnnotationSqlDataSourceDefinition dataSourceDefinition = new JAnnotationSqlDataSourceDefinition();
            dataSourceDefinition.setName(dataSource.getName());

            if (classAnnotationMetadata.getJAnnotationSqlDataSourceDefinitions().contains(dataSourceDefinition)) {
                //if object with same name exists update it
                dataSourceDefinition = classAnnotationMetadata.getJAnnotationSqlDataSourceDefinitions()
                                                              .get(classAnnotationMetadata
                                                                           .getJAnnotationSqlDataSourceDefinitions()
                                                                           .indexOf(dataSourceDefinition));
            } else {
                //else add the newly created object
                classAnnotationMetadata.getJAnnotationSqlDataSourceDefinitions().add(dataSourceDefinition);
            }
            if (dataSource.getClassName() != null) {
                dataSourceDefinition.setClassName(dataSource.getClassName());
            }
            if (dataSource.getDatabaseName() != null) {
                dataSourceDefinition.setDatabaseName(dataSource.getDatabaseName());
            }
            if (dataSource.getDescription() != null) {
                dataSourceDefinition.setDescription(dataSource.getDescription());
            }
            if (dataSource.getInitialPoolSize() != null) {
                dataSourceDefinition.setInitialPoolSize(dataSource.getInitialPoolSize());
            }
            if (dataSource.getIsolationLevel() != null) {
                dataSourceDefinition.setIsolationLevel(dataSource.getIsolationLevel());
            }
            if (dataSource.getLoginTimeout() != null) {
                dataSourceDefinition.setLoginTimeout(dataSource.getLoginTimeout());
            }
            if (dataSource.getMaxIdleTime() != null) {
                dataSourceDefinition.setMaxIdleTime(dataSource.getMaxIdleTime());
            }
            if (dataSource.getMaxPoolSize() != null) {
                dataSourceDefinition.setMaxPoolSize(dataSource.getMaxPoolSize());
            }
            if (dataSource.getMaxStatements() != null) {
                dataSourceDefinition.setMaxStatements(dataSource.getMaxStatements());
            }
            if (dataSource.getMinPoolSize() != null) {
                dataSourceDefinition.setMinPoolSize(dataSource.getMinPoolSize());
            }
            if (dataSource.getPassword() != null) {
                dataSourceDefinition.setPassword(dataSource.getPassword());
            }
            if (dataSource.getPortNumber() != null) {
                dataSourceDefinition.setPortNumber(dataSource.getPortNumber());
            }
            if (dataSource.getProperties() != null) {
                dataSourceDefinition.setProperties(dataSource.getProperties());
            }
            if (dataSource.getServerName() != null) {
                dataSourceDefinition.setServerName(dataSource.getServerName());
            }
            if (dataSource.isTransactional() != null) {
                dataSourceDefinition.setTransactional(dataSource.isTransactional());
            }
            if (dataSource.getUrl() != null) {
                dataSourceDefinition.setUrl(dataSource.getUrl());
            }
            if (dataSource.getUser() != null) {
                dataSourceDefinition.setUser(dataSource.getUser());
            }
        }

    }


    /**
     * Apply rules for a common bean (persistence-context-ref).
     * @param persistenceContextRefList the list of persistence-context-ref
     * @param classAnnotationMetadata (Annotation metadata) of the bean using
     *        this interceptor (may be null)
     */
    private void applyPersistenceContextRef(final List<IPersistenceContextRef> persistenceContextRefList,
            final IEnvironmentRefAccessor classAnnotationMetadata) {
        if (persistenceContextRefList == null) {
            return;
        }

        // For each persistence-context-ref, build a &#64;{@link javax.persistence.PersistenceContext}
        for (IPersistenceContextRef persistenceContextRef : persistenceContextRefList) {
            JavaxPersistenceContext jPersistenceContext =  new JavaxPersistenceContext();
            // set ref-name
            jPersistenceContext.setName(persistenceContextRef.getPersistenceContextRefName());

            // set unit-name
            jPersistenceContext.setUnitName(persistenceContextRef.getPersistenceUnitName());

            // set context type
            String type = persistenceContextRef.getPersistenceContextType();
            if (IJavaxPersistenceContextType.EXTENDED.toString().equals(type)) {
                jPersistenceContext.setType(IJavaxPersistenceContextType.EXTENDED);
            } else if (IJavaxPersistenceContextType.TRANSACTION.toString().equals(type)) {
                jPersistenceContext.setType(IJavaxPersistenceContextType.TRANSACTION);
            }

            applyJPersistenceContext(jPersistenceContext, classAnnotationMetadata, persistenceContextRef
                    .getInjectionTargetList());
        }
    }

    /**
     * Apply rules for a common bean (service-ref).
     * @param serviceRefList the list of service-ref.
     * @param classAnnotationMetadata (Annotation metadata) of the bean using
     *        this interceptor (may be null).
     */
    protected void applyServiceRef(final List<IServiceRef> serviceRefList,
                                   final IEnvironmentRefAccessor classAnnotationMetadata) {
        if (serviceRefList == null) {
            return;
        }

        // For each service-ref, build a &#64;{@link
        // javax.xml.ws.WebServiceRef}
        for (IServiceRef serviceRef : serviceRefList) {

            List<IInjectionTarget> injectionTargetList = serviceRef.getInjectionTargetList();

            // Honor the injection-target if specified
            if (injectionTargetList != null && injectionTargetList.size() > 0) {
                for (IInjectionTarget target : injectionTargetList) {
                    // Find the attribute / target
                    ISharedMetadataView commonClassView = getInjectionTarget(classAnnotationMetadata, target);

                    // If there is already a WebServiceRef annotation, overload
                    // it
                    IJaxwsWebServiceRef wsr = commonClassView.getJaxwsWebServiceRef();

                    if (wsr != null) {
                        // merge
                        mergeWebServiceRef(serviceRef, wsr);
                    } else {
                        // place a new annotation on the target
                        commonClassView.setJaxwsWebServiceRef(createJaxwsWebServiceRefFromXML(serviceRef));
                    }
                }
            } else {
                // We need to lookup the target annotation in the class
                // and overload it.
                String name = serviceRef.getName();
                List<IJaxwsWebServiceRef> metadatas = findJaxwsWebServiceRefMetadata(classAnnotationMetadata, name);

                if (metadatas.isEmpty()) {
                    // no annotation to override:
                    // -> equivalent to class-level annotation

                    // If there is an existing single class-level annotation,
                    // place it in the list
                    IJaxwsWebServiceRef existing = classAnnotationMetadata.getJaxwsWebServiceRef();
                    List<IJaxwsWebServiceRef> webServiceRefs = classAnnotationMetadata.getJaxwsWebServiceRefs();

                    // init the list if needed
                    if (webServiceRefs == null) {
                        webServiceRefs = new ArrayList<IJaxwsWebServiceRef>();
                        classAnnotationMetadata.setJaxwsWebServiceRefs(webServiceRefs);
                    }

                    if (existing != null) {
                        // place the annotation in the @WebServiceRefs list
                        webServiceRefs.add(existing);
                        // remove the single @WebServiceRef
                        classAnnotationMetadata.setJaxwsWebServiceRef(null);
                    }

                    // add our new annotation in the list
                    webServiceRefs.add(createJaxwsWebServiceRefFromXML(serviceRef));

                } else if (metadatas.size() > 1) {
                    // Not good, too many choices
                    // a smart tool may try to find a compatible existing
                    // metadata and perform the merge
                    // and a dumb tool will produce an Exception :)
                    throw new IllegalStateException("Too many matching WebServiceRef metadata for '" + name
                            + "' service-ref of class '" + classAnnotationMetadata.getEnvironmentName() + "'");
                } else {
                    // only 1 matching candidate
                    // merge
                    mergeWebServiceRef(serviceRef, metadatas.get(0));
                }
            }
        }
    }

    /**
     * Needs to search the matching WebServiceRef.
     * @param classAnnotationMetadata the metadata
     * @param name the xml name of the service ref
     * @return a list of matching web service ref
     */
    protected List<IJaxwsWebServiceRef> findJaxwsWebServiceRefMetadata(final IEnvironmentRefAccessor classAnnotationMetadata,
            final String name) {

        List<IJaxwsWebServiceRef> elements = new ArrayList<IJaxwsWebServiceRef>();

        // Find annotation on Class
        // Try @WebServiceRef
        IJaxwsWebServiceRef classAnnotation = classAnnotationMetadata.getJaxwsWebServiceRef();
        if ((classAnnotation != null) && (name.equals(classAnnotation.getName()))) {
            elements.add(classAnnotation);
        }
        // Try @WebServiceRefs
        List<IJaxwsWebServiceRef> webServiceRefs = classAnnotationMetadata.getJaxwsWebServiceRefs();
        if (webServiceRefs != null) {
            for (IJaxwsWebServiceRef ref : webServiceRefs) {
                if (name.equals(ref.getName())) {
                    elements.add(ref);
                }
            }
        }

        return elements;
    }

    /**
     * Create a new IJaxwsWebServiceRef object from the result of the XML parsing.
     * @param ref the XML struct
     * @return a new implementation of our metadata
     */
    protected IJaxwsWebServiceRef createJaxwsWebServiceRefFromXML(final IServiceRef ref) {
        // Creates an empty metadata element
        IJaxwsWebServiceRef metadata = new JaxwsWebServiceRef();

        // Fill it from XML values
        metadata.setName(ref.getName());
        metadata.setWsdlLocation(ref.getWsdlFile().toString());

        // service-interface
        metadata.setType(ref.getServiceInterface());
        metadata.setValue(ref.getServiceInterface());

        // service-ref-type
        String serviceRefType = ref.getServiceRefType();
        if (serviceRefType != null && !"".equals(serviceRefType)) {
            metadata.setType(serviceRefType);
        }

        // handler-chains
        metadata.setHandlerChains(ref.getHandlerChains());

        // Complete port-component-ref, etc.
        completeWebServiceRef(ref, metadata);

        // Return
        return metadata;
    }

    /**
     * Merge the values from the XML inside the metadata struct.
     * @param serviceRef XML
     * @param annotation the metadata structure to be updated
     */
    protected void mergeWebServiceRef(final IServiceRef serviceRef, final IJaxwsWebServiceRef annotation) {

        if (serviceRef.getWsdlFile() != null) {
            annotation.setWsdlLocation(serviceRef.getWsdlFile().toString());
        }

        if (serviceRef.getServiceInterface() != null) {
            if (annotation.getValue() == null) {
                // the annotation is used to declare a service
                annotation.setType(serviceRef.getServiceInterface());
            } else if (annotation.getType() == null) {
                // TODO check if the type is a Service subclass or not
                // the annotation is used to declare a service-endpoint
                annotation.setValue(serviceRef.getServiceInterface());
            }
        }

        if (serviceRef.getServiceRefType() != null) {
            annotation.setType(serviceRef.getServiceRefType());
        }

        // Override name
        annotation.setName(serviceRef.getName());

        // Override handler-chains
        annotation.setHandlerChains(serviceRef.getHandlerChains());

        // Complete port-component-ref, etc.
        completeWebServiceRef(serviceRef, annotation);

    }

    /**
     * Complete the values from the XML into the metadata struct.
     * @param serviceRef XML
     * @param annotation the metadata structure to be updated
     */
    protected void completeWebServiceRef(final IServiceRef serviceRef, final IJaxwsWebServiceRef annotation) {
        // Set the port component ref list
        annotation.setPortComponentRefs(serviceRef.getPortComponentRefList());

        // TODO complete update (service-qname)
    }

    /**
     * Apply rules for a common bean (message-destination-ref).
     * @param messageDestinationRefList the list of message-destination-ref.
     * @param classAnnotationMetadata (Annotation metadata) of the bean using
     *        this interceptor (may be null).
     */
    protected void applyMessageDestinationRef(final List<IMessageDestinationRef> messageDestinationRefList,
            final IEnvironmentRefAccessor classAnnotationMetadata) {
        if (messageDestinationRefList == null) {
            return;
        }

        // For each message-destination-ref, build a &#64;{@link
        // javax.annotation.Resource}
        for (IMessageDestinationRef messageDestinationRef : messageDestinationRefList) {
            JAnnotationResource jResource = new JAnnotationResource();
            // set name
            jResource.setName(messageDestinationRef.getName());

            // set type
            jResource.setType(messageDestinationRef.getType());

            // Mapped Name
            jResource.setMappedName(messageDestinationRef.getMappedName());

            // sets the lookup-name
            if (messageDestinationRef.getLookupName() != null) {
                jResource.setLookup(messageDestinationRef.getLookupName());
            }

            // Link ?
            jResource.setMessageDestinationLink(messageDestinationRef.getLink());

            // Apply the resource on the class
            applyJResource(jResource, classAnnotationMetadata, messageDestinationRef.getInjectionTargetList());
        }
    }

    /**
     * Apply rules for a common bean (resource-ref).
     * @param resourceRefList the list of resource-ref.
     * @param classAnnotationMetadata (Annotation metadata) of the bean using
     *        this interceptor (may be null).
     */
    protected void applyResourceRef(final List<IResourceRef> resourceRefList,
            final IEnvironmentRefAccessor classAnnotationMetadata) {
        if (resourceRefList == null) {
            return;
        }

        // For each resource-ref, build a &#64;{@link javax.annotation.Resource}
        for (IResourceRef resourceRef : resourceRefList) {
            JAnnotationResource jResource = new JAnnotationResource();
            // set name
            jResource.setName(resourceRef.getResRefName());

            // set type
            jResource.setType(resourceRef.getResRefType());

            // Mapped Name
            jResource.setMappedName(resourceRef.getMappedName());

            // sets the lookup-name
            if (resourceRef.getLookupName() != null) {
                jResource.setLookup(resourceRef.getLookupName());
            }

            // Apply the resource on the class
            applyJResource(jResource, classAnnotationMetadata, resourceRef.getInjectionTargetList());
        }
    }

    /**
     * Apply rules for a common bean (resource-env-ref).
     * @param resourceEnvRefList the list of resource-env-ref.
     * @param classAnnotationMetadata (Annotation metadata) of the bean using
     *        this interceptor (may be null).
     */
    protected void applyResourceEnvRef(final List<IResourceEnvRef> resourceEnvRefList,
            final IEnvironmentRefAccessor classAnnotationMetadata) {
        if (resourceEnvRefList == null) {
            return;
        }

        // For each resource-ref, build a &#64;{@link javax.annotation.Resource}
        for (IResourceEnvRef resourceEnvRef : resourceEnvRefList) {
            JAnnotationResource jResource = new JAnnotationResource();
            // set name
            jResource.setName(resourceEnvRef.getResourceEnvRefName());

            // set type
            jResource.setType(resourceEnvRef.getResourceEnvRefType());

            // Mapped Name
            jResource.setMappedName(resourceEnvRef.getMappedName());

            // sets the lookup-name
            if (resourceEnvRef.getLookupName() != null) {
                jResource.setLookup(resourceEnvRef.getLookupName());
            }

            // Apply the resource on the class
            applyJResource(jResource, classAnnotationMetadata, resourceEnvRef.getInjectionTargetList());
        }
    }

    /**
     * Apply rules for a common bean (env-entry).
     * @param envEntryList the list of env-entry.
     * @param classAnnotationMetadata (Annotation metadata) of the bean
     */
    protected void applyEnvEntry(final List<org.ow2.util.ee.metadata.common.api.xml.struct.IEnvEntry> envEntryList,
            final IEnvironmentRefAccessor classAnnotationMetadata) {
        if (envEntryList == null) {
            return;
        }

        List<IEnvEntry> structEnvEntryList = new ArrayList<IEnvEntry>();

        // Store the env-entry list in the metadata class
        for (org.ow2.util.ee.metadata.common.api.xml.struct.IEnvEntry envEntry : envEntryList) {
            IEnvEntry builtEntry = new org.ow2.util.ee.metadata.common.impl.struct.EnvEntry(envEntry.getEnvEntryName(),
                    envEntry.getEnvEntryType(), envEntry.getEnvEntryValue());
            structEnvEntryList.add(builtEntry);

            // for each injection-target field, add a resource annotation
            List<IInjectionTarget> injectionTargetList = envEntry.getInjectionTargetList();
            if (injectionTargetList != null) {
                for (IInjectionTarget injectionTarget : injectionTargetList) {

                    // build a new annotation
                    JAnnotationResource jResource = new JAnnotationResource();

                    // set name
                    jResource.setName(envEntry.getEnvEntryName());

                    // set type
                    jResource.setType(envEntry.getEnvEntryType());

                    // Add the JAnnotationResource on it
                    getInjectionTarget(classAnnotationMetadata, injectionTarget).setJAnnotationResource(jResource);
                }
            }

            // applyJResource(jResource, classAnnotationMetadata,
            // resourceEnvRef.getInjectionTargetList());
        }

        // Set the env entry collection
        classAnnotationMetadata.setEnvEntryCollection(structEnvEntryList);

    }

    /**
     * Adds the given &#64;{@link javax.persistence.PersistenceUnit} object to the
     * class and apply it also on injection target elements.
     * @param jPersistenceUnit an &#64;{@link javax.persistence.PersistenceUnit} impl
     * @param classAnnotationMetadata the bean's class
     * @param injectionTargetList the list of injection fields to set
     */
    protected void applyJPersistenceUnit(final JavaxPersistenceUnit jPersistenceUnit,
            final IEnvironmentRefAccessor classAnnotationMetadata, final List<IInjectionTarget> injectionTargetList) {

        // Add field on the class
        IJavaxPersistenceUnit existingJPersistentUnit = classAnnotationMetadata.getJavaxPersistenceUnit();

        // Needs to use a list instead.
        List<IJavaxPersistenceUnit> jPersistentUnits = classAnnotationMetadata.getJavaxPersistencePersistenceUnits();
        if (jPersistentUnits == null) {
            jPersistentUnits = new ArrayList<IJavaxPersistenceUnit>();
            classAnnotationMetadata.setJavaxPersistencePersistenceUnits(jPersistentUnits);
        }

        // Put existing existing element in the list
        if (existingJPersistentUnit != null) {
            jPersistentUnits.add(existingJPersistentUnit);
            classAnnotationMetadata.setJavaxPersistenceUnit(null);
        }

        // Add the new element
        jPersistentUnits.add(jPersistenceUnit);


        // Now, add elements on the injection fields
        if (injectionTargetList != null) {
            for (IInjectionTarget injectionTarget : injectionTargetList) {

                // Add the JavaxPersistenceUnit on it
                getInjectionTarget(classAnnotationMetadata, injectionTarget).setJavaxPersistenceUnit(jPersistenceUnit);
            }
        }
    }

    /**
     * Adds the given &#64;{@link javax.persistence.PersistenceContext} object to the
     * class and apply it also on injection target elements.
     * @param jPersistenceContext an &#64;{@link javax.persistence.PersistenceContext} impl
     * @param classAnnotationMetadata the bean's class
     * @param injectionTargetList the list of injection fields to set
     */
    protected void applyJPersistenceContext(final JavaxPersistenceContext jPersistenceContext,
            final IEnvironmentRefAccessor classAnnotationMetadata, final List<IInjectionTarget> injectionTargetList) {

        // Add field on the class
        IJavaxPersistenceContext existingJPersistentContext = classAnnotationMetadata.getJavaxPersistenceContext();

        // Needs to use a list instead.
        List<IJavaxPersistenceContext> jPersistentContexts = classAnnotationMetadata.getJavaxPersistencePersistenceContexts();
        if (jPersistentContexts == null) {
            jPersistentContexts = new ArrayList<IJavaxPersistenceContext>();
            classAnnotationMetadata.setJavaxPersistencePersistenceContexts(jPersistentContexts);
        }

        // Put existing existing element in the list
        if (existingJPersistentContext != null) {
            jPersistentContexts.add(existingJPersistentContext);
            classAnnotationMetadata.setJavaxPersistenceContext(null);
        }

        // Add the new element
        jPersistentContexts.add(jPersistenceContext);


        // Now, add elements on the injection fields
        if (injectionTargetList != null) {
            for (IInjectionTarget injectionTarget : injectionTargetList) {

                // Add the JavaxPersistenceContext on it
                getInjectionTarget(classAnnotationMetadata, injectionTarget).setJavaxPersistenceContext(jPersistenceContext);
            }
        }
    }
    /**
     * Apply rules for a common bean (ejb-ref).
     * @param ejbRefList the list of ejb-ref.
     * @param classAnnotationMetadata (Annotation metadata) of the bean using
     *        this interceptor (may be null).
     */
    protected void applyEjbRef(final List<IEJBRef> ejbRefList, final IEnvironmentRefAccessor classAnnotationMetadata) {
        if (ejbRefList == null) {
            return;
        }

        // For each ejb-ref, build a &#64;{@link javax.ejb.EJB}
        for (IEJBRef ejbRef : ejbRefList) {
            JEjbEJB jEJB = buildEJB(ejbRef);
            // set interface
            if (ejbRef.getHome() != null) {
                jEJB.setBeanInterface(ejbRef.getHome());
            } else if (ejbRef.getRemote() != null) {
                jEJB.setBeanInterface(ejbRef.getRemote());
            }
            applyJEJB(jEJB, classAnnotationMetadata, ejbRef.getInjectionTargetList());
        }
    }

    /**
     * Apply rules for a common bean (ejb-local-ref).
     * @param ejbLocalRefList the list of ejb-local-ref.
     * @param classAnnotationMetadata (Annotation metadata) of the bean using
     *        this interceptor (may be null).
     */
    protected void applyEjbLocalRef(final List<IEJBLocalRef> ejbLocalRefList,
            final IEnvironmentRefAccessor classAnnotationMetadata) {
        if (ejbLocalRefList == null) {
            return;
        }

        // For each ejb-local-ref, build a &#64;{@link javax.ejb.EJB}
        for (IEJBLocalRef ejbLocalRef : ejbLocalRefList) {
            JEjbEJB jEJB = buildEJB(ejbLocalRef);
            // set interface
            if (ejbLocalRef.getLocalHome() != null) {
                jEJB.setBeanInterface(ejbLocalRef.getLocalHome());
            } else if (ejbLocalRef.getLocal() != null) {
                jEJB.setBeanInterface(ejbLocalRef.getLocal());
            }
            applyJEJB(jEJB, classAnnotationMetadata, ejbLocalRef.getInjectionTargetList());
        }
    }

    /**
     * Adds the given &#64;{@link javax.annotation.Resource} object to the
     * class and apply it also on injection target elements.
     * @param jResource an &#64;{@link javax.annotation.Resource} impl
     * @param classAnnotationMetadata the bean's class
     * @param injectionTargetList the list of injection fields to set
     */
    private void applyJResource(final JAnnotationResource jResource, final IEnvironmentRefAccessor classAnnotationMetadata,
            final List<IInjectionTarget> injectionTargetList) {

        // Add field on the class
        IJAnnotationResource existingJResource = classAnnotationMetadata.getJAnnotationResource();

        // Needs to use a list instead.
        List<IJAnnotationResource> jResources = classAnnotationMetadata.getJAnnotationResources();
        if (jResources == null) {
            jResources = new ArrayList<IJAnnotationResource>();
            classAnnotationMetadata.setJAnnotationResources(jResources);
        }

        // put existing element in the list
        if (existingJResource != null) {
            jResources.add(existingJResource);
            // remove single element
            classAnnotationMetadata.setJAnnotationResource(null);
        }

        // Add our new element
        jResources.add(jResource);

        // Now, add elements on the injection fields
        if (injectionTargetList != null) {
            for (IInjectionTarget injectionTarget : injectionTargetList) {

                // Add the JAnnotationResource on it
                getInjectionTarget(classAnnotationMetadata, injectionTarget).setJAnnotationResource(jResource);
            }
        }

    }

    /**
     * Gets the metadata for a given injection target.
     * @param classAnnotationMetadata the class used to search the target
     *        element
     * @param injectionTarget the given target info element
     * @return a common annotation metadata if element is found
     */
    private ISharedMetadataView getInjectionTarget(final IEnvironmentRefAccessor classAnnotationMetadata,
            final IInjectionTarget injectionTarget) {
        String injectionTargetName = injectionTarget.getTargetName();

        // First, try with the field
        IFieldMetadata field = getField(classAnnotationMetadata, injectionTarget);
        // found ? return it
        if (field != null) {
            return field.as(CommonView.class);
        }

        // Else, try with the method
        IMethodMetadata method = getMethod(classAnnotationMetadata, injectionTarget);

        // else, try with the next item which should be a field
        if (method == null) {
            throw new IllegalArgumentException("No method or field for injection target name '" + injectionTargetName
                    + "' found in the class '" + classAnnotationMetadata.getEnvironmentName() + "'.");
        }

        return method.as(CommonView.class);

    }

    /**
     * Gets the method metadata for a given injection target.
     * @param classAnnotationMetadata the class used to search the target
     *        element
     * @param injectionTarget the given target info element
     * @return a method metadata if element is found
     */
    private IMethodMetadata getMethod(final IEnvironmentRefAccessor classAnnotationMetadata,
            final IInjectionTarget injectionTarget) {

        // get data from the injection target element
        String clName = injectionTarget.getClassname();

        if (clName == null) {
            clName = findDefaultClassForEnvironment(classAnnotationMetadata);
        } else {
            clName = clName.replace(".", "/");
        }

        // Get method name which is the injection target name
        String methodName = injectionTarget.getTargetName();

        // Now, search the element in the given class
        IClassMetadata foundClassMetadata = getClassMetadata(clName);
        if (foundClassMetadata == null) {
            throw new IllegalArgumentException("Cannot find the class metadata named '" + clName + "'.");
        }

        // Got the class metadata, now search the field
        List<IMethodMetadata> methods = foundClassMetadata.searchMethodMetadata(methodName);

        if (methods.size() == 0) {
            // no method
            return null;
        } else if (methods.size() > 1) {
            // Should be only one
            throw new IllegalArgumentException("Too many methods with name '" + methodName + "' found in the class '" + clName
                    + "'.");
        }

        return methods.get(0);
    }

    /**
     * Gets the field metadata for a given injection target.
     * @param classAnnotationMetadata the class used to search the target
     *        element
     * @param injectionTarget the given target info element
     * @return a field if element is found
     */
    private IFieldMetadata getField(final IEnvironmentRefAccessor classAnnotationMetadata,
            final IInjectionTarget injectionTarget) {
        // get data from the injection target element
        String clName = injectionTarget.getClassname();
        if (clName == null) {
            clName = findDefaultClassForEnvironment(classAnnotationMetadata);
        } else {
            clName = clName.replace(".", "/");
        }

        String fieldName = injectionTarget.getTargetName();

        // Now, search the element in the given class
        IClassMetadata foundClassMetadata = getClassMetadata(clName);
        if (foundClassMetadata == null) {
            throw new IllegalArgumentException("Cannot find the class metadata named '" + clName + "'.");
        }

        // Got the class metadata, now search the field
        List<IFieldMetadata> fields = foundClassMetadata.searchFieldMetadata(fieldName);

        if (fields.size() == 0) {
            // no field
            return null;
        } else if (fields.size() > 1) {
            // Should be only one
            throw new IllegalArgumentException("Too many fields with name '" + fieldName + "' found in the class '" + clName
                    + "'.");
        }

        return fields.get(0);
    }

    /**
     * Adds the given &#64;{@link javax.ejb.EJB} object to the class and apply
     * it also on injection target elements.
     * @param jEJB an &#64;{@link javax.ejb.EJB} impl
     * @param classAnnotationMetadata the bean's class
     * @param injectionTargetList the list of injection fields to set
     */
    private void applyJEJB(final JEjbEJB jEJB, final IEnvironmentRefAccessor classAnnotationMetadata,
            final List<IInjectionTarget> injectionTargetList) {

        // Add field on the class
        IJEjbEJB existingJEjbEJB = classAnnotationMetadata.getJEjbEJB();

        // Needs to use a list instead.
        List<IJEjbEJB> jEjbEJBs = classAnnotationMetadata.getJEjbEJBs();
        if (jEjbEJBs == null) {
            jEjbEJBs = new ArrayList<IJEjbEJB>();
            classAnnotationMetadata.setJEjbEJBs(jEjbEJBs);
        }

        // put existing element in the list
        if (existingJEjbEJB != null) {
            jEjbEJBs.add(existingJEjbEJB);
            // remove single element
            classAnnotationMetadata.setJEjbEJB(null);
        }

        // Add our new element
        jEjbEJBs.add(jEJB);

        // Now, add elements on the injection fields
        if (injectionTargetList != null) {
            for (IInjectionTarget injectionTarget : injectionTargetList) {

                // Add the JEjbEJB on it
                getInjectionTarget(classAnnotationMetadata, injectionTarget).setJEjbEJB(jEJB);
            }
        }

    }

    /**
     * Build an &#64;{@link javax.ejb.EJB} impl for the given
     * ejb-ref/ejb-local-ref.
     * @param ref an instance of a ejb-ref/ejb-local-ref.
     * @return a implementation of &#64;{@link javax.ejb.EJB} interface.
     */
    private JEjbEJB buildEJB(final ICommonEJBRef ref) {
        JEjbEJB jEJB = new JEjbEJB();
        // Sets the name
        if (ref.getEjbRefName() != null) {
            jEJB.setName(ref.getEjbRefName());
        }

        // sets the mapped-name
        if (ref.getMappedName() != null) {
            jEJB.setMappedName(ref.getMappedName());
        }

        // sets the lookup-name
        if (ref.getLookupName() != null) {
            jEJB.setLookup(ref.getLookupName());
        }

        // sets the bean-name
        if (ref.getEjbLink() != null) {
            jEJB.setBeanName(ref.getEjbLink());
        }

        return jEJB;
    }

    /**
     * Apply rules for a common bean (method callbacks).
     * @param aroundInvokeList the list of method/aroundInvoke callbacks.
     * @param classAnnotationMetadata (Annotation metadata) of the bean using
     *        this interceptor (may be null).
     * @param type the type of lifecycle.
     */
    protected void applyAroundInvoke(final List<AroundInvoke> aroundInvokeList,
            final IEnvironmentRefAccessor classAnnotationMetadata, final String type) {
        for (IAroundInvoke aroundInvoke : aroundInvokeList) {
            String interceptorClassName = null;

            // if no class, takes the bean's class
            if (aroundInvoke.getClassName() == null) {
                interceptorClassName = findDefaultClassForEnvironment(classAnnotationMetadata);
            } else {
                interceptorClassName = encode(aroundInvoke.getClassName());
            }

            // Now, search the Method on the class or its super-class if not
            // found
            String searchMethodName = aroundInvoke.getMethodName();
            if (searchMethodName == null) {
                throw new IllegalStateException("Invalid null method name for the callback '" + interceptorClassName + "'.");
            }

            // Apply interceptor
            applyInterceptor(type, interceptorClassName, searchMethodName, classAnnotationMetadata);

        }
    }

    /**
     * Apply rules for a common bean (lifecycle callbacks).
     * @param lifecycleList the list of lifecycle callbacks.
     * @param classAnnotationMetadata (Annotation metadata) of the bean using
     *        this interceptor (may be null).
     * @param type the type of lifecycle.
     */
    protected void applyLifeCycle(final List<ILifeCycleCallback> lifecycleList,
            final IEnvironmentRefAccessor classAnnotationMetadata, final String type) {

        if (lifecycleList == null) {
            return;
        }

        for (ILifeCycleCallback lifecycleCallback : lifecycleList) {

            String interceptorClassName = null;

            // if no class, takes the bean's class
            if (lifecycleCallback.getLifecycleCallbackClass() == null) {
                interceptorClassName = findDefaultClassForEnvironment(classAnnotationMetadata);
            } else {
                interceptorClassName = encode(lifecycleCallback.getLifecycleCallbackClass());
            }

            // Now, search the Method on the class or its super-class if not
            // found
            String searchMethodName = lifecycleCallback.getLifecycleCallbackMethod();
            if (searchMethodName == null) {
                throw new IllegalStateException("Invalid null method name for the callback '" + interceptorClassName + "'.");
            }

            // Apply interceptor
            applyInterceptor(type, interceptorClassName, searchMethodName, classAnnotationMetadata);

        }
    }

    /**
     * Finds a method in the interceptor class and set the correct type value.
     * @param type type of interceptor
     * @param interceptorClassName the name of the interceptor class
     * @param methodName the name of the method in the interceptor class
     * @param classAnnotationMetadata the class that is using the interceptor
     *        (may be null)
     */
    private void applyInterceptor(final String type, final String interceptorClassName, final String methodName,
            final IEnvironmentRefAccessor classAnnotationMetadata) {

        IMethodMetadata foundMethod = null;
        IClassMetadata interceptorMetadata = getClassMetadata(interceptorClassName);

        boolean loop = true;
        // While method is not found and that class has super class, try to
        // search
        while (foundMethod == null && loop) {
            List<IMethodMetadata> listMethods = interceptorMetadata.searchMethodMetadata(methodName);

            // Method present ?
            if (listMethods.size() == 1) {
                // first item
                foundMethod = listMethods.get(0);
                continue;
            } else if (listMethods.size() > 1) {
                throw new IllegalStateException("Method with name '" + methodName + "' was found more than once on the class '"
                        + interceptorMetadata.getJClass().getName() + "'.");
            }

            // else continue
            String superClassName = interceptorMetadata.getJClass().getSuperName();
            if (!superClassName.equals(JAVA_LANG_OBJECT)) {
                interceptorMetadata = getClassMetadata(superClassName);
            } else {
                // break loop
                loop = false;
            }
        }

        if (foundMethod == null) {
            throw new IllegalStateException("The method named '" + methodName + "' was not found in the class '"
                    + interceptorMetadata.getJClass().getName() + "' and all its super-classes.");
        }

        CommonView methodView = foundMethod.as(CommonView.class);

        // call setter to true and only if it is not already present.
        if (POST_CONSTRUCT.equals(type)) {
            if (!methodView.isPostConstruct()) {
                methodView.setPostConstruct(true);
            }
        } else if (PRE_DESTROY.equals(type)) {
            if (!methodView.isPreDestroy()) {
                methodView.setPreDestroy(true);
            }
        }
    }

    /**
     * Encode a classname (replace . by /) as it's the internal form of the
     * metadata
     * @param className the name of the class to encode
     * @return encoded name
     */
    protected static String encode(final String className) {
        return className.replace(".", "/");
    }

    /**
     * Decode a internal classname (replace '/' by '.').
     * @param className the name of the class to decode
     * @return decoded name
     */
    protected static String decode(final String className) {
        return className.replace("/", ".");
    }

    /**
     * @param className the class name
     * @return the common class metadata which name is classname
     */
    public abstract IClassMetadata getClassMetadata(String className);

    public abstract String findDefaultClassForEnvironment(IEnvironmentRefAccessor env);

    /**
     * Add the WebServiceRef element if the name of the annotation is matching.
     * @param sharedMetadata the shared metadata (field, method, ...)
     * @param name the name specified in the XML
     * @param jaxWSWebServiceRefs the list of elements that have been found
     */
    protected void addJaxwsWebServiceRefIfFound(final IMetadata sharedMetadata,
                                                final String name,
                                                final List<IJaxwsWebServiceRef> jaxWSWebServiceRefs) {
        CommonView commonView = sharedMetadata.as(CommonView.class);
        IJaxwsWebServiceRef webServiceRef = commonView.getJaxwsWebServiceRef();
        if (webServiceRef != null && name.equals(webServiceRef.getName())) {
            jaxWSWebServiceRefs.add(webServiceRef);
        }
    }

    /**
     * Merge the @HandlerChain annotations in their colocated @WebServiceRef
     */
    protected void applyHandlerChain() {

        // Browse HandlerChains and @WebServiceRef supported by classes
        Collection<IClassMetadata> allClassMetadata = getAllClassMetadatas();
        for (IClassMetadata metadata : allClassMetadata) {

            CommonView view = metadata.as(CommonView.class);

            IJwsHandlerChain handlerChain = view.getJwsHandlerChain();
            List<IJaxwsWebServiceRef> references = view.getJaxwsWebServiceRefs();

            if ((handlerChain != null) && ((references != null) && !references.isEmpty())) {
                // We have a HandlerChain and some @WebServiceRef annotations
                for (IJaxwsWebServiceRef reference : references) {
                    reference.setHandlerChainFile(handlerChain.getFile());

                    // Store declaring class
                    String classname = decode(metadata.getJClass().getName());
                    reference.setDeclaringClass(classname);
                }
            }
        }

        // Browse HandlerChains and @WebServiceRef supported by methods
        Collection<IMethodMetadata> allMethodMetadata = getAllMethodMetadatas();
        for (IMethodMetadata metadata : allMethodMetadata) {
            CommonView view = metadata.as(CommonView.class);

            IJwsHandlerChain handlerChain = view.getJwsHandlerChain();
            IJaxwsWebServiceRef reference = view.getJaxwsWebServiceRef();

            if ((handlerChain != null) && (reference != null)) {
                // We have a HandlerChain and some @WebServiceRef annotations
                reference.setHandlerChainFile(handlerChain.getFile());

                // Store declaring class
                String classname = decode(metadata.getClassMetadata().getJClass().getName());
                reference.setDeclaringClass(classname);
            }
        }


        // Browse HandlerChains and @WebServiceRef supported by fields
        Collection<IFieldMetadata> allFieldMetadatas = getAllFieldMetadatas();
        for (IFieldMetadata metadata : allFieldMetadatas) {
            CommonView view = metadata.as(CommonView.class);

            IJwsHandlerChain handlerChain = view.getJwsHandlerChain();
            IJaxwsWebServiceRef reference = view.getJaxwsWebServiceRef();

            if ((handlerChain != null) && (reference != null)) {
                // We have a HandlerChain and some @WebServiceRef annotations
                reference.setHandlerChainFile(handlerChain.getFile());

                // Store declaring class
                String classname = decode(metadata.getClassMetadata().getJClass().getName());
                reference.setDeclaringClass(classname);
            }
        }
    }

    /**
     * @return A collection of all ClassMetadata of the merged object.
     */
    protected abstract Collection<IClassMetadata> getAllClassMetadatas();

    /**
     * @return A collection of all MethodMetadata of the merged object.
     */
    protected abstract Collection<IMethodMetadata> getAllMethodMetadatas();

    /**
     * @return A collection of all FieldMetadata of the merged object.
     */
    protected abstract Collection<IFieldMetadata> getAllFieldMetadatas();

}
