/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.common.impl.xml.parsing;

import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.sql.Connection;
import javax.xml.namespace.QName;
import org.ow2.util.ee.metadata.common.api.xml.struct.IDataSource;
import org.ow2.util.ee.metadata.common.api.xml.struct.IEJBLocalRef;
import org.ow2.util.ee.metadata.common.api.xml.struct.IEnvEntry;
import org.ow2.util.ee.metadata.common.api.xml.struct.IEnvironment;
import org.ow2.util.ee.metadata.common.api.xml.struct.IMessageDestinationRef;
import org.ow2.util.ee.metadata.common.api.xml.struct.IPersistenceContextRef;
import org.ow2.util.ee.metadata.common.api.xml.struct.IPersistenceUnitRef;
import org.ow2.util.ee.metadata.common.api.xml.struct.IPortComponentRef;
import org.ow2.util.ee.metadata.common.api.xml.struct.IResourceEnvRef;
import org.ow2.util.ee.metadata.common.api.xml.struct.IResourceGroup;
import org.ow2.util.ee.metadata.common.api.xml.struct.IResourceRef;
import org.ow2.util.ee.metadata.common.api.xml.struct.ISecurityRoleRef;
import org.ow2.util.ee.metadata.common.api.xml.struct.IServiceRef;
import org.ow2.util.ee.metadata.common.impl.enc.ENCBinding;
import org.ow2.util.ee.metadata.common.impl.xml.struct.AbsEJBRef;
import org.ow2.util.ee.metadata.common.impl.xml.struct.DataSource;
import org.ow2.util.ee.metadata.common.impl.xml.struct.EJBLocalRef;
import org.ow2.util.ee.metadata.common.impl.xml.struct.EJBRef;
import org.ow2.util.ee.metadata.common.impl.xml.struct.EnvEntry;
import org.ow2.util.ee.metadata.common.impl.xml.struct.Handler;
import org.ow2.util.ee.metadata.common.impl.xml.struct.HandlerChain;
import org.ow2.util.ee.metadata.common.impl.xml.struct.HandlerChains;
import org.ow2.util.ee.metadata.common.impl.xml.struct.InjectionTarget;
import org.ow2.util.ee.metadata.common.impl.xml.struct.LifeCycleCallback;
import org.ow2.util.ee.metadata.common.impl.xml.struct.MessageDestinationRef;
import org.ow2.util.ee.metadata.common.impl.xml.struct.PersistenceContextRef;
import org.ow2.util.ee.metadata.common.impl.xml.struct.PersistenceUnitRef;
import org.ow2.util.ee.metadata.common.impl.xml.struct.PortComponentRef;
import org.ow2.util.ee.metadata.common.impl.xml.struct.ResourceEnvRef;
import org.ow2.util.ee.metadata.common.impl.xml.struct.ResourceRef;
import org.ow2.util.ee.metadata.common.impl.xml.struct.SecurityRoleRef;
import org.ow2.util.ee.metadata.common.impl.xml.struct.ServiceRef;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.xml.XMLUtils;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import static org.ow2.util.ee.metadata.common.api.xml.struct.ILifeCycleCallback.POST_ACTIVATE;
import static org.ow2.util.ee.metadata.common.api.xml.struct.ILifeCycleCallback.POST_CONSTRUCT;
import static org.ow2.util.ee.metadata.common.api.xml.struct.ILifeCycleCallback.PRE_DESTROY;
import static org.ow2.util.ee.metadata.common.api.xml.struct.ILifeCycleCallback.PRE_PASSIVATE;

/**
 * Provide a method to analyze an xml environnement.
 *
 * @author Florent Benoit
 */
public abstract class CommonDeploymentDescLoader extends AbsDeploymentDescLoader {

    /**
     * Logger.
     */
    private static final Log logger = LogFactory.getLog(CommonDeploymentDescLoader.class);

    /**
     * Utility class, no public constructor.
     *
     * @param url the url that is pointing to the EJB Xml file.
     */
    protected CommonDeploymentDescLoader(final URL url) {
        super(url);
    }

    /**
     * Analyze the environment (could be a bean, servlet, etc).
     *
     * @param envElement the dom element.
     * @param env the structure representing object.
     */
    protected void analyzeEnvironment(final Element envElement, final IEnvironment env) {
        analyzeLifeCycle(envElement, env, POST_CONSTRUCT);
        analyzeLifeCycle(envElement, env, PRE_DESTROY);
        analyzeEnvEnvtry(envElement, env);
        analyzeEJBRef(envElement, env);
        analyzeEJBLocalRef(envElement, env);
        analyzeResourceRef(envElement, env);
        analyzeResourceEnvRef(envElement, env);
        analyzeMessageDestinationRef(envElement, env);
        analyzePersistenceContextRef(envElement, env);
        analyzePersistenceUnitRef(envElement, env);
        analyzeServiceRef(envElement, env);
        analyzeSecurityRoleRef(envElement, env);
        analyzeDataSource(envElement, env);
    }


    /**
     * Analyze the ejb-ref/local-ref element and find the sub-elements.
     *
     * @param refElement given XML element which contains ejb-ref-name,
     *        ejb-ref-type, etc.
     * @param ref the struct object representing the XML element.
     */
    private void analyzeCommonEJBRef(final Element refElement, final AbsEJBRef ref) {
        // ejb-ref-name
        String ejbRefName = XMLUtils.getStringValueElement(getXMLNS(), refElement, "ejb-ref-name");
        ref.setEjbRefName(ejbRefName);

        // ejb-ref-type
        String ejbRefType = XMLUtils.getStringValueElement(getXMLNS(), refElement, "ejb-ref-type");
        ref.setEjbRefType(ejbRefType);

        // mapped-name
        String mappedName = XMLUtils.getStringValueElement(getXMLNS(), refElement, "mapped-name");
        ref.setMappedName(mappedName);
        
        // lookup-name
        String lookupName = XMLUtils.getStringValueElement(getXMLNS(), refElement, "lookup-name");
        ref.setLookupName(lookupName);        

        // ejb-link
        String ejbLink = XMLUtils.getStringValueElement(getXMLNS(), refElement, "ejb-link");
        ref.setEjbLink(ejbLink);

        // Analyze injection-target
        analyzeInjectionTarget(refElement, ref);
    }

    /**
     * Analyze the injection-target element and add them to the given resource
     * group object used by ejb-ref, ejb-local-ref, resource-ref, etc.
     *
     * @param refElement given XML element which contains a resource element.
     * @param resourceGroup the struct object representing the XML element.
     */
    private void analyzeInjectionTarget(final Element refElement, final IResourceGroup resourceGroup) {
        // injection target
        NodeList injectionTargetList = refElement.getElementsByTagNameNS(getXMLNS(), "injection-target");
        for (int j = 0; j < injectionTargetList.getLength(); j++) {
            Element injectionTargetElement = (Element) injectionTargetList.item(j);

            // Build Object and add it
            InjectionTarget injectionTarget = new InjectionTarget();

            // Get class name
            String className =
                XMLUtils.getStringValueElement(getXMLNS(), injectionTargetElement, "injection-target-class");
            injectionTarget.setClassname(className);

            // Get target
            String targetName =
                XMLUtils.getStringValueElement(getXMLNS(), injectionTargetElement, "injection-target-name");
            injectionTarget.setTargetName(targetName);

            resourceGroup.addInjectionTarget(injectionTarget);
        }

    }

    /**
     * Analyze the environment element and find the ejb-ref element.
     *
     * @param envElement given XML element which contains ejb-ref.
     * @param env the struct object representing the XML element.
     */
    private void analyzeEJBRef(final Element envElement, final IEnvironment env) {
        NodeList ejbRefList = envElement.getElementsByTagNameNS(getXMLNS(), EJBRef.NAME);

        // Loop on this list
        for (int i = 0; i < ejbRefList.getLength(); i++) {
            Element ejbRefElement = (Element) ejbRefList.item(i);
            EJBRef ejbRef = new EJBRef();

            analyzeCommonEJBRef(ejbRefElement, ejbRef);

            // remote
            String remote = XMLUtils.getStringValueElement(getXMLNS(), ejbRefElement, "remote");
            ejbRef.setRemote(remote);

            // home
            String home = XMLUtils.getStringValueElement(getXMLNS(), ejbRefElement, "home");
            ejbRef.setHome(home);

            // add ejb-ref in environment element
            env.addEJBRef(ejbRef);
        }
    }

    /**
     * Analyze the environment element and find the resource-ref element.
     *
     * @param envElement given XML element which contains resource-ref.
     * @param env the struct object representing the XML element.
     */
    private void analyzeResourceRef(final Element envElement, final IEnvironment env) {
        NodeList resourceRefList = envElement.getElementsByTagNameNS(getXMLNS(), IResourceRef.NAME);

        // Loop on this list
        for (int i = 0; i < resourceRefList.getLength(); i++) {
            Element resourceRefElement = (Element) resourceRefList.item(i);
            ResourceRef resourceRef = new ResourceRef();

            // name
            String resRefName = XMLUtils.getStringValueElement(getXMLNS(), resourceRefElement, "res-ref-name");
            resourceRef.setResRefName(resRefName);

            // type
            String resRefType = XMLUtils.getStringValueElement(getXMLNS(), resourceRefElement, "res-type");
            resourceRef.setResRefType(resRefType);

            // mapped-name
            String mappedName = XMLUtils.getStringValueElement(getXMLNS(), resourceRefElement, "mapped-name");
            resourceRef.setMappedName(mappedName);

            // lookup-name
            String lookupName = XMLUtils.getStringValueElement(getXMLNS(), resourceRefElement, "lookup-name");
            resourceRef.setLookupName(lookupName); 
            
            // Analyze injection-target
            analyzeInjectionTarget(resourceRefElement, resourceRef);

            // add resource-ref in environment element
            env.addResourceRef(resourceRef);
        }
    }

    /**
     * Analyze the environment element and find the security-role-ref element.
     *
     * @param envElement given XML element which contains security-role-ref.
     * @param env the struct object representing the XML element.
     */
    private void analyzeSecurityRoleRef(final Element envElement, final IEnvironment env) {
        NodeList securityRoleRefList = envElement.getElementsByTagNameNS(getXMLNS(), ISecurityRoleRef.NAME);

        // Loop on this list
        for (int i = 0; i < securityRoleRefList.getLength(); i++) {
            Element securityRoleRefElement = (Element) securityRoleRefList.item(i);
            SecurityRoleRef securityRoleRef = new SecurityRoleRef();

            // name
            String roleName = XMLUtils.getStringValueElement(getXMLNS(), securityRoleRefElement, "role-name");
            securityRoleRef.setRoleName(roleName);

            // link
            String roleLink = XMLUtils.getStringValueElement(getXMLNS(), securityRoleRefElement, "role-link");
            securityRoleRef.setRoleLink(roleLink);

            // add security-role-ref in environment element
            env.addSecurityRoleRef(securityRoleRef);
        }
    }


    /**
     * Analyze the environment element and find the message-destination-ref
     * element.
     *
     * @param envElement given XML element which contains resource-ref.
     * @param env the struct object representing the XML element.
     */
    private void analyzeMessageDestinationRef(final Element envElement, final IEnvironment env) {
        NodeList messageDestinationRefList = envElement.getElementsByTagNameNS(getXMLNS(), IMessageDestinationRef.NAME);

        // Loop on this list
        for (int i = 0; i < messageDestinationRefList.getLength(); i++) {
            Element messageDestinationRefElement = (Element) messageDestinationRefList.item(i);
            MessageDestinationRef messageDestinationRef = new MessageDestinationRef();

            // name
            String name = XMLUtils
                    .getStringValueElement(getXMLNS(), messageDestinationRefElement, "message-destination-ref-name");
            messageDestinationRef.setName(name);

            // type
            String type = XMLUtils.getStringValueElement(getXMLNS(), messageDestinationRefElement,
                "message-destination-type");
            messageDestinationRef.setType(type);

            // usage
            String usage = XMLUtils.getStringValueElement(getXMLNS(), messageDestinationRefElement,
                "message-destination-usage");
            messageDestinationRef.setUsage(usage);

            // link
            String link = XMLUtils.getStringValueElement(getXMLNS(), messageDestinationRefElement,
                "message-destination-link");
            messageDestinationRef.setLink(link);

            // mapped-name
            String mappedName = XMLUtils.getStringValueElement(getXMLNS(), messageDestinationRefElement, "mapped-name");
            messageDestinationRef.setMappedName(mappedName);

            // lookup-name
            String lookupName = XMLUtils.getStringValueElement(getXMLNS(), messageDestinationRefElement, "lookup-name");
            messageDestinationRef.setLookupName(lookupName);              
            
            // Analyze injection-target
            analyzeInjectionTarget(messageDestinationRefElement, messageDestinationRef);

            // add message-destination-ref in environment element
            env.addMessageDestinationRef(messageDestinationRef);
        }
    }

    /**
     * Analyze the environment element and find the persistence-unit-ref
     * element.
     *
     * @param envElement given XML element which contains persistence-unit-ref.
     * @param env the struct object representing the XML element.
     */
    private void analyzePersistenceUnitRef(final Element envElement, final IEnvironment env) {
        logger.debug("Analyze environment, looking for "+ IPersistenceUnitRef.NAME);
        NodeList persistenceUnitRefList = envElement.getElementsByTagNameNS(getXMLNS(), IPersistenceUnitRef.NAME);

        // Loop on this list
        for (int i = 0; i < persistenceUnitRefList.getLength(); i++) {
            Element persistenceUnitRefElement = (Element) persistenceUnitRefList.item(i);
            PersistenceUnitRef persistenceUnitRef = new PersistenceUnitRef();

            // persistence-unit-ref name
            String unitRefName = XMLUtils
                    .getStringValueElement(getXMLNS(), persistenceUnitRefElement, "persistence-unit-ref-name");
            if (unitRefName != null) {
                persistenceUnitRef.setPersistenceUnitRefName(unitRefName);
            }
            logger.debug("Analyze environment, found persistence-unit-ref-name " + unitRefName);
            // persistence-unit name
            String unitName = XMLUtils
                    .getStringValueElement(getXMLNS(), persistenceUnitRefElement, "persistence-unit-name");
            if (unitName != null) {
                persistenceUnitRef.setPersistenceUnitName(unitName);
            }

            // add persistence-unit-ref in environment element
            env.addPersistenceUnitRef(persistenceUnitRef);
        }
    }

    /**
     * Analyze the environment element and find the persistence-context-ref
     * element.
     *
     * @param envElement given XML element which contains persistence-context-ref.
     * @param env the struct object representing the XML element.
     */
    private void analyzePersistenceContextRef(final Element envElement, final IEnvironment env) {
        logger.debug("Analyze environment, looking for "+ IPersistenceContextRef.NAME);
        NodeList persistenceContextRefList = envElement.getElementsByTagNameNS(getXMLNS(), IPersistenceContextRef.NAME);

        // Loop on this list
        for (int i = 0; i < persistenceContextRefList.getLength(); i++) {
            Element persistenceContextRefElement = (Element) persistenceContextRefList.item(i);
            PersistenceContextRef persistenceContextRef = new PersistenceContextRef();

            // persistence-context-ref name
            String contextRefName = XMLUtils
                    .getStringValueElement(getXMLNS(), persistenceContextRefElement, "persistence-context-ref-name");
            if (contextRefName != null) {
                persistenceContextRef.setPersistenceContextRefName(contextRefName);
            }
            logger.debug("Analyze environment, found persistence-context-ref-name " + contextRefName);
            // persistence-unit name
            String unitName = XMLUtils
                    .getStringValueElement(getXMLNS(), persistenceContextRefElement, "persistence-unit-name");
            if (unitName != null) {
                persistenceContextRef.setPersistenceUnitName(unitName);
            }
            // persistence-context-type
            String contextType = XMLUtils
                    .getStringValueElement(getXMLNS(), persistenceContextRefElement, "persistence-context-type");
            if (contextType != null) {
                persistenceContextRef.setPersistenceContextType(contextType);
            }

            // add persistence-unit-ref in environment element
            env.addPersistenceContextRef(persistenceContextRef);
        }
    }

    /**
     * Analyze the environment element and find the service-ref
     * element.
     *
     * @param envElement given XML element which contains service-ref.
     * @param env the struct object representing the XML element.
     */
    private void analyzeServiceRef(final Element envElement, final IEnvironment env) {
        // Use Java EE namespace as we only manage JAX-WS service-ref
        NodeList serviceRefList = envElement.getElementsByTagNameNS(JAVAEE_NS, IServiceRef.NAME);

        // Loop on this list
        for (int i = 0; i < serviceRefList.getLength(); i++) {
            Element serviceRefElement = (Element) serviceRefList.item(i);
            ServiceRef serviceRef = new ServiceRef();

            // name
            String name = XMLUtils
                    .getStringValueElement(getXMLNS(), serviceRefElement, "service-ref-name");
            serviceRef.setName(name);

            // service-interface
            String type = XMLUtils.getStringValueElement(getXMLNS(), serviceRefElement, "service-interface");
            serviceRef.setServiceInterface(type);

            // service-ref-type
            String usage = XMLUtils.getStringValueElement(getXMLNS(), serviceRefElement, "service-ref-type");
            serviceRef.setServiceRefType(usage);

            // wsdl-file
            String link = XMLUtils.getStringValueElement(getXMLNS(), serviceRefElement, "wsdl-file");
            if (link != null) {
                URI location = null;
                try {
                    location = new URI(link);
                } catch (URISyntaxException e) {
                    throw new IllegalStateException("Element 'wsdl-file' is not a valid URI", e);
                }
                serviceRef.setWsdlFile(location);
            }

            // service-qname
            NodeList serviceQNameList = envElement.getElementsByTagNameNS(getXMLNS(), "service-qname");
            if (serviceQNameList.getLength() == 1) {
                QName serviceQName = XMLUtils.getValueAsQName((Element) serviceQNameList.item(0));
                serviceRef.setServiceQName(serviceQName);
            }

            // port-component-ref
            analyzePortComponentRef(serviceRefElement, serviceRef);

            // handler-chains
            analyzeHandlerChains(serviceRefElement, serviceRef);

            // Analyze injection-target
            analyzeInjectionTarget(serviceRefElement, serviceRef);

            // add service-ref in environment element
            env.addServiceRef(serviceRef);
        }
    }

    /**
     * Analyze the service-ref element and find the handler(s) or handler-chains
     * element.
     *
     * @param serviceRefElement given XML element which contains port-component-ref.
     * @param serviceRef the struct object representing the XML element.
     */
    private void analyzeHandlerChains(final Element serviceRefElement, final IServiceRef serviceRef) {
        NodeList handlerChainsList = serviceRefElement.getElementsByTagNameNS(getXMLNS(), "handler-chains");

        if (handlerChainsList.getLength() == 1) {
            // There is a handler-chains element
            Element handlerChainsElement = (Element) handlerChainsList.item(0);

            HandlerChains handlerChains = new HandlerChains();
            serviceRef.setHandlerChains(handlerChains);
            analyzeHandlerChain(handlerChainsElement, handlerChains);

        }
    }

    /**
     * Analyze the handler-chains element and find the handler-chain
     * element.
     *
     * @param handlerChainsElement given XML element which contains handler-chains.
     * @param handlerChains the struct object representing the XML element.
     */
    protected void analyzeHandlerChain(final Element handlerChainsElement, final HandlerChains handlerChains) {

        NodeList handlerChainList = handlerChainsElement.getElementsByTagNameNS(getXMLNS(), "handler-chain");

        for (int i = 0; i < handlerChainList.getLength(); i++) {
            // There is a handler-chain element
            Element handlerChainElement = (Element) handlerChainList.item(i);

            HandlerChain handlerChain = new HandlerChain();
            handlerChains.addHandlerChain(handlerChain);

            // service-name-pattern
            QName servicePattern = XMLUtils.getChildValueAsQName(handlerChainElement, "service-name-pattern");
            handlerChain.setServiceNamePattern(servicePattern);

            // port-name-pattern
            QName portPattern = XMLUtils.getChildValueAsQName(handlerChainElement, "port-name-pattern");
            handlerChain.setPortNamePattern(portPattern);

            // protocol-bindings
            analyzeProtocolBindings(handlerChainElement, handlerChain);

            // handler
            analyzeHandlers(handlerChainElement, handlerChain);

        }


    }

    private void analyzeHandlers(final Element handlerChainElement, final HandlerChain handlerChain) {

        NodeList handlerList = handlerChainElement.getElementsByTagNameNS(getXMLNS(), "handler");

        for (int i = 0; i < handlerList.getLength(); i++) {
            // There is a handler element
            Element handlerElement = (Element) handlerList.item(i);

            // handler-name
            String name = XMLUtils.getStringValueElement(getXMLNS(), handlerElement, "handler-name");

            // handler-class
            String classname = XMLUtils.getStringValueElement(getXMLNS(), handlerElement, "handler-class");

            Handler handler = new Handler(name, classname);
            handlerChain.addHandler(handler);

            // soap-header
            QName soapHeader = XMLUtils.getChildValueAsQName(handlerElement, "soap-header");
            handler.setSoapHeader(soapHeader);

            // soap-role
            String soapRole = XMLUtils.getStringValueElement(getXMLNS(), handlerElement, "soap-role");
            handler.setSoapRole(soapRole);

            // init-param
            analyzeInitParam(handlerElement, handler);

        }


    }

    /**
     * Analyze the handler element and fill the init values Map.
     *
     * @param handlerElement given XML element which contains init-param.
     * @param handler the struct object representing the XML element.
     */
    private void analyzeInitParam(final Element handlerElement, final Handler handler) {

        NodeList initParamList = handlerElement.getElementsByTagNameNS(getXMLNS(), "init-param");

        for (int i = 0; i < initParamList.getLength(); i++) {
            // There is a init-param element
            Element initParamElement = (Element) initParamList.item(i);

            // param-name
            String name = XMLUtils.getStringValueElement(getXMLNS(), handlerElement, "param-name");

            // param-value
            String value = XMLUtils.getStringValueElement(getXMLNS(), handlerElement, "param-value");

            handler.addInitParam(name, value);
        }

    }

    /**
     * Analyze the handler-chain element and find the protocol-bindings values.
     *
     * @param handlerChainElement given XML element which contains handler-chain.
     * @param handlerChain the struct object representing the XML element.
     */
    private void analyzeProtocolBindings(final Element handlerChainElement, final HandlerChain handlerChain) {

        // protocol-bindings
        String bindingsValue = XMLUtils.getStringValueElement(getXMLNS(), handlerChainElement, "protocol-bindings");
        if (bindingsValue != null) {
            String[] bindings = bindingsValue.split(" ");
            for (String binding : bindings) {
                handlerChain.addProtocolBinding(binding);
            }
        }

    }

    /**
     * Analyze the service-ref element and find the port-component-ref
     * element.
     *
     * @param serviceRefElement given XML element which contains port-component-ref.
     * @param serviceRef the struct object representing the XML element.
     */
    private void analyzePortComponentRef(final Element serviceRefElement, final IServiceRef serviceRef) {
        NodeList portComponentRefList = serviceRefElement.getElementsByTagNameNS(getXMLNS(), IPortComponentRef.NAME);

        // Loop on this list
        for (int i = 0; i < portComponentRefList.getLength(); i++) {
            Element portComponentRefElement = (Element) portComponentRefList.item(i);
            PortComponentRef portComponentRef = new PortComponentRef();

            // service-endpoint-interface
            String sei =
                XMLUtils.getStringValueElement(getXMLNS(), portComponentRefElement, "service-endpoint-interface");
            portComponentRef.setServiceEndpointInterface(sei);

            // enable-mtom
            String enableMTOM = XMLUtils.getStringValueElement(getXMLNS(), portComponentRefElement, "enable-mtom");
            if (enableMTOM != null) {
                portComponentRef.setEnableMTOM(Boolean.parseBoolean(enableMTOM));
            }

            // port-component-link
            String link = XMLUtils.getStringValueElement(getXMLNS(), portComponentRefElement, "port-component-link");
            portComponentRef.setPortComponentLink(link);

            // add port-component-ref in service-ref element
            serviceRef.addPortComponentRef(portComponentRef);
        }
    }

    /**
     * Analyze the environment element and find the resource-env-ref element.
     *
     * @param envElement given XML element which contains resource-ref.
     * @param env the struct object representing the XML element.
     */
    private void analyzeResourceEnvRef(final Element envElement, final IEnvironment env) {
        NodeList resourceEnvRefList = envElement.getElementsByTagNameNS(getXMLNS(), IResourceEnvRef.NAME);

        // Loop on this list
        for (int i = 0; i < resourceEnvRefList.getLength(); i++) {
            Element resourceEnvRefElement = (Element) resourceEnvRefList.item(i);
            ResourceEnvRef resourceEnvRef = new ResourceEnvRef();

            // name
            String resourceEnvRefName = XMLUtils
                    .getStringValueElement(getXMLNS(), resourceEnvRefElement, "resource-env-ref-name");
            resourceEnvRef.setResourceEnvRefName(resourceEnvRefName);

            // type
            String resourceEnvRefType = XMLUtils
                    .getStringValueElement(getXMLNS(), resourceEnvRefElement, "resource-env-ref-type");
            resourceEnvRef.setResourceEnvRefType(resourceEnvRefType);

            // mapped-name
            String mappedName = XMLUtils.getStringValueElement(getXMLNS(), resourceEnvRefElement, "mapped-name");
            resourceEnvRef.setMappedName(mappedName);

            // lookup-name
            String lookupName = XMLUtils.getStringValueElement(getXMLNS(), resourceEnvRefElement, "lookup-name");
            resourceEnvRef.setLookupName(lookupName);                     
            
            // Analyze injection-target
            analyzeInjectionTarget(resourceEnvRefElement, resourceEnvRef);

            // add resource-ref in environment element
            env.addResourceEnvRef(resourceEnvRef);
        }
    }

    /**
     * Analyze the environment element and find the ejb-local-ref element.
     *
     * @param envElement given XML element which contains ejb-local-ref.
     * @param env the struct object representing the XML element.
     */
    private void analyzeEJBLocalRef(final Element envElement, final IEnvironment env) {
        NodeList ejbLocalRefList = envElement.getElementsByTagNameNS(getXMLNS(), IEJBLocalRef.NAME);

        // Loop on this list
        for (int i = 0; i < ejbLocalRefList.getLength(); i++) {
            Element ejbRefElement = (Element) ejbLocalRefList.item(i);
            EJBLocalRef ejbLocalRef = new EJBLocalRef();

            analyzeCommonEJBRef(ejbRefElement, ejbLocalRef);

            // local
            String local = XMLUtils.getStringValueElement(getXMLNS(), ejbRefElement, "local");
            ejbLocalRef.setLocal(local);

            // local-home
            String localHome = XMLUtils.getStringValueElement(getXMLNS(), ejbRefElement, "local-home");
            ejbLocalRef.setLocalHome(localHome);

            // add ejb-local-ref in environment element
            env.addEJBLocalRef(ejbLocalRef);
        }
    }

    /**
     * Analyze the env-entry element.
     *
     * @param envElement the dom element.
     * @param bean the structure representing object.
     */
    private void analyzeEnvEnvtry(final Element envElement, final IEnvironment bean) {
        // Get env-entry
        NodeList envEntryList = envElement.getElementsByTagNameNS(getXMLNS(), IEnvEntry.NAME);

        // Loop on env entry
        for (int i = 0; i < envEntryList.getLength(); i++) {
            Element envEntryElement = (Element) envEntryList.item(i);

            // Build instance that is created.
            EnvEntry envEntry = new EnvEntry();
            bean.addEnvEntry(envEntry);

            // Env-entry name
            String envEntryName = XMLUtils.getStringValueElement(getXMLNS(), envEntryElement, "env-entry-name");
            envEntry.setEnvEntryName(envEntryName);

            // Env-entry type
            String envEntryType = XMLUtils.getStringValueElement(getXMLNS(), envEntryElement, "env-entry-type");
            envEntry.setEnvEntryType(envEntryType);

            // Env-entry value
            String envEntryValue =
                XMLUtils.getStringValueElement(getXMLNS(), envEntryElement, "env-entry-value", false);
            envEntry.setEnvEntryValue(envEntryValue);
            
            // Env-entry lookup name
            String envEntryLookupName =
                XMLUtils.getStringValueElement(getXMLNS(), envEntryElement, "lookup-name", false);
            envEntry.setLookupName(envEntryLookupName);

            // Analyze injection-target
            analyzeInjectionTarget(envEntryElement, envEntry);

        }
    }

    /**
     * Analyze the lifecycle elements.
     *
     * @param envElement the dom element.
     * @param env the structure representing object.
     * @param lifecycleName the name of lifecycle (post-construct, etc)
     */
    protected void analyzeLifeCycle(final Element envElement, final IEnvironment env, final String lifecycleName) {
        // Get the lifecycle elements
        NodeList lifecycleList = envElement.getElementsByTagNameNS(getXMLNS(), lifecycleName);

        // Loop on the lifecycle
        for (int i = 0; i < lifecycleList.getLength(); i++) {
            Element lifeCycleElement = (Element) lifecycleList.item(i);

            // Build instance that is created.
            LifeCycleCallback lifecycleCallback = new LifeCycleCallback();
            if (POST_CONSTRUCT.equals(lifecycleName)) {
                env.addPostConstructCallback(lifecycleCallback);
            } else if (PRE_DESTROY.equals(lifecycleName)) {
                env.addPreDestroyCallback(lifecycleCallback);
            } else if (PRE_PASSIVATE.equals(lifecycleName)) {
                env.addPrePassivateCallback(lifecycleCallback);
            } else if (POST_ACTIVATE.equals(lifecycleName)) {
                env.addPostActivateCallback(lifecycleCallback);
            } else {
                throw new IllegalArgumentException("Unknown lifecycle named '" + lifecycleName + "'.");
            }

            // lifecycle-callback-class
            String className = XMLUtils.getStringValueElement(getXMLNS(), lifeCycleElement, "lifecycle-callback-class");
            lifecycleCallback.setLifecycleCallbackClass(className);

            // lifecycle-callback-method
            String method = XMLUtils.getStringValueElement(getXMLNS(), lifeCycleElement, "lifecycle-callback-method");
            lifecycleCallback.setMethodName(method);

        }

    }

    private void analyzeDataSource(Element envElement, IEnvironment env) {
        // Get data-source list
        NodeList dataSourceList = envElement.getElementsByTagNameNS(getXMLNS(), IDataSource.NAME);

        for (int i = 0; i < dataSourceList.getLength(); i++) {
            Element dataSourceElement = (Element) dataSourceList.item(i);

            // Build instance that is created.
            DataSource dataSource = new DataSource();

            // data-source name
            String dataSourceName = null;
            NodeList dataSourceNamesList = dataSourceElement.getElementsByTagNameNS(getXMLNS(), "name");
            for (int n = 0; n < dataSourceNamesList.getLength(); n++) {
                Element dataSourceNameElement = (Element) dataSourceNamesList.item(n);
                if (dataSourceNameElement.getParentNode() == dataSourceElement) {
                    Node node = dataSourceNameElement.getFirstChild();
                    if (node != null) {
                        dataSourceName = node.getNodeValue();
                        if (dataSourceName != null) {
                            dataSourceName = dataSourceName.trim();
                            break;
                        }
                    }
                }
            }
            if (dataSourceName == null) {
                logger.warn("Malformed environment: data-source name element missing");
                continue;
            }
            //In order to ease XML desc and annotations merging adding default ns prefix on names without explicit
            // ns declaration
            if (!dataSourceName.startsWith(ENCBinding.NS_JAVA_APP_PREFIX) &&
                !dataSourceName.startsWith(ENCBinding.NS_JAVA_COMP_PREFIX) &&
                !dataSourceName.startsWith(ENCBinding.NS_JAVA_GLOBAL_PREFIX) &&
                !dataSourceName.startsWith(ENCBinding.NS_JAVA_MODULE_PREFIX)) {
                dataSourceName = ENCBinding.DEFAULT_NS_PREFIX + dataSourceName;
            }
            dataSource.setName(dataSourceName);

            // data-source description
            String dataSourceDescription = XMLUtils.getStringValueElement(getXMLNS(), dataSourceElement, "description");
            if (dataSourceDescription != null) {
                dataSource.setDescription(dataSourceDescription);
            }

            // data-source class-name
            String dataSourceClassName = XMLUtils.getStringValueElement(getXMLNS(), dataSourceElement, "class-name");
            dataSource.setClassName(dataSourceClassName);

            // data-source server-name
            String dataSourceServerName = XMLUtils.getStringValueElement(getXMLNS(), dataSourceElement, "server-name");
            if (dataSourceServerName != null) {
                dataSource.setServerName(dataSourceServerName);
            }

            // data-source port-number
            String dataSourcePortNumber = XMLUtils.getStringValueElement(getXMLNS(), dataSourceElement, "port-number");
            if (dataSourcePortNumber != null) {
                try {
                    dataSource.setPortNumber(Integer.parseInt(dataSourcePortNumber));
                } catch (NumberFormatException e) {
                    logger.warn("port-number element for data-source {0} is not a valid integer", dataSource.getName());
                    logger.debug(e);
                }
            }

            // data-source database-name
            String dataSourceDatabaseName =
                XMLUtils.getStringValueElement(getXMLNS(), dataSourceElement, "database-name");
            if (dataSourceDatabaseName != null) {
                dataSource.setDatabaseName(dataSourceDatabaseName);
            }

            // data-source url
            String dataSourceUrl =
                XMLUtils.getStringValueElement(getXMLNS(), dataSourceElement, "url");
            if (dataSourceUrl != null) {
                dataSource.setUrl(dataSourceUrl);
            }

            // data-source user
            String dataSourceUser =
                XMLUtils.getStringValueElement(getXMLNS(), dataSourceElement, "user");
            if (dataSourceUser != null) {
                dataSource.setUser(dataSourceUser);
            }

            // data-source password
            String dataSourcePassword =
                XMLUtils.getStringValueElement(getXMLNS(), dataSourceElement, "password");
            if (dataSourcePassword != null) {
                dataSource.setPassword(dataSourcePassword);
            }


            // Get property elements

            NodeList propertiesList = dataSourceElement.getElementsByTagNameNS(getXMLNS(), "property");
            final String[] dataSourceProperties = new String[propertiesList.getLength()];
            // If properties is present, analyze them and add them
            if (propertiesList.getLength() > 0) {
                for (int j = 0; j < propertiesList.getLength(); j++) {
                    Element elemProperty = (Element) propertiesList.item(j);
                    String pName = XMLUtils.getStringValueElement(elemProperty, "name");
                    String pValue = XMLUtils.getStringValueElement(elemProperty, "value");
                    if (pName != null && pValue != null) {
                        dataSourceProperties[j] = pName + "=" + pValue;
                    }

                }
            }
            dataSource.setProperties(dataSourceProperties);

            // data-source login-timeout
            String dataSourceLoginTimeout =
                XMLUtils.getStringValueElement(getXMLNS(), dataSourceElement, "login-timeout");
            if (dataSourceLoginTimeout != null) {
                try {
                    dataSource.setLoginTimeout(Integer.parseInt(dataSourceLoginTimeout));
                } catch (NumberFormatException e) {
                    logger.warn("login-timeout element for data-source {0} is not a valid integer",
                        dataSource.getName());
                    logger.debug(e);
                }
            }

            // data-source transactional
            String dataSourceTransactional =
                XMLUtils.getStringValueElement(getXMLNS(), dataSourceElement, "transactional");
            if (dataSourceTransactional != null) {
                if (dataSourceTransactional.equalsIgnoreCase("true") || dataSourceTransactional.equalsIgnoreCase("1")) {
                    dataSource.setTransactional(true);
                } else if (dataSourceTransactional.equalsIgnoreCase("false") ||
                    dataSourceTransactional.equalsIgnoreCase("0")) {
                    dataSource.setTransactional(false);
                } else {
                    logger.warn("transactional element for data-source {0} is not a valid boolean. Using default value",
                        dataSource.getName());
                }
            }

            // data-source login-timeout
            String dataSourceIsolationLevel =
                XMLUtils.getStringValueElement(getXMLNS(), dataSourceElement, "isolation-level");
            if (dataSourceIsolationLevel != null) {
                if (dataSourceIsolationLevel.equals("TRANSACTION_READ_UNCOMMITTED")) {
                    dataSource.setIsolationLevel(Connection.TRANSACTION_READ_UNCOMMITTED);
                } else if (dataSourceIsolationLevel.equals("TRANSACTION_READ_COMMITTED")) {
                    dataSource.setIsolationLevel(Connection.TRANSACTION_READ_COMMITTED);
                } else if (dataSourceIsolationLevel.equals("TRANSACTION_REPEATABLE_READ")) {
                    dataSource.setIsolationLevel(Connection.TRANSACTION_REPEATABLE_READ);
                } else if (dataSourceIsolationLevel.equals("TRANSACTION_SERIALIZABLE")) {
                    dataSource.setIsolationLevel(Connection.TRANSACTION_SERIALIZABLE);
                }

            }

            // data-source initial-pool-size
            String dataSourceInitialPoolSize =
                XMLUtils.getStringValueElement(getXMLNS(), dataSourceElement, "initial-pool-size");
            if (dataSourceInitialPoolSize != null) {
                try {
                    dataSource.setInitialPoolSize(Integer.parseInt(dataSourceInitialPoolSize));
                } catch (NumberFormatException e) {
                    logger.warn("initial-pool-size element for data-source {0} is not a valid integer",
                        dataSource.getName());
                    logger.debug(e);
                }
            }

            // data-source max-pool-size
            String dataSourceMaxPoolSize =
                XMLUtils.getStringValueElement(getXMLNS(), dataSourceElement, "max-pool-size");
            if (dataSourceMaxPoolSize != null) {
                try {
                    dataSource.setMaxPoolSize(Integer.parseInt(dataSourceMaxPoolSize));
                } catch (NumberFormatException e) {
                    logger.warn("max-pool-size element for data-source {0} is not a valid integer",
                        dataSource.getName());
                    logger.debug(e);
                }
            }

            // data-source min-pool-size
            String dataSourceMinPoolSize =
                XMLUtils.getStringValueElement(getXMLNS(), dataSourceElement, "min-pool-size");
            if (dataSourceMinPoolSize != null) {
                try {
                    dataSource.setMinPoolSize(Integer.parseInt(dataSourceMinPoolSize));
                } catch (NumberFormatException e) {
                    logger.warn("min-pool-size element for data-source {0} is not a valid integer",
                        dataSource.getName());
                    logger.debug(e);
                }
            }

            // data-source max-idle-time
            String dataSourceMaxIdleTime =
                XMLUtils.getStringValueElement(getXMLNS(), dataSourceElement, "max-idle-time");
            if (dataSourceMaxIdleTime != null) {
                try {
                    dataSource.setMaxIdleTime(Integer.parseInt(dataSourceMaxIdleTime));
                } catch (NumberFormatException e) {
                    logger.warn("max-idle-time element for data-source {0} is not a valid integer",
                        dataSource.getName());
                    logger.debug(e);
                }
            }

            // data-source max-statements
            String dataSourceMaxStatements =
                XMLUtils.getStringValueElement(getXMLNS(), dataSourceElement, "max-statements");
            if (dataSourceMaxStatements != null) {
                try {
                    dataSource.setMaxStatements(Integer.parseInt(dataSourceMaxStatements));
                } catch (NumberFormatException e) {
                    logger.warn("max-statements element for data-source {0} is not a valid integer",
                        dataSource.getName());
                    logger.debug(e);
                }
            }

            env.addDataSource(dataSource);
        }
    }


}
