/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.common.impl.xml.struct;

import java.util.ArrayList;
import java.util.List;
import org.ow2.util.ee.metadata.common.api.xml.struct.IAroundInvoke;
import org.ow2.util.ee.metadata.common.api.xml.struct.IDataSource;
import org.ow2.util.ee.metadata.common.api.xml.struct.IEJBLocalRef;
import org.ow2.util.ee.metadata.common.api.xml.struct.IEJBRef;
import org.ow2.util.ee.metadata.common.api.xml.struct.IEnvEntry;
import org.ow2.util.ee.metadata.common.api.xml.struct.IEnvironment;
import org.ow2.util.ee.metadata.common.api.xml.struct.ILifeCycleCallback;
import org.ow2.util.ee.metadata.common.api.xml.struct.IMessageDestinationRef;
import org.ow2.util.ee.metadata.common.api.xml.struct.IPersistenceContextRef;
import org.ow2.util.ee.metadata.common.api.xml.struct.IPersistenceUnitRef;
import org.ow2.util.ee.metadata.common.api.xml.struct.IResourceEnvRef;
import org.ow2.util.ee.metadata.common.api.xml.struct.IResourceRef;
import org.ow2.util.ee.metadata.common.api.xml.struct.ISecurityRoleRef;
import org.ow2.util.ee.metadata.common.api.xml.struct.IServiceRef;

/**
 * This class defines an implementation for an environment element.
 *
 * @author Florent Benoit
 */
public class AbsEnvironment implements IEnvironment {

    /**
     * Serial Version UID.
     */
    private static final long serialVersionUID = 6234694208035054870L;

    /**
     * List of &lt;env-entry&gt; elements.
     */
    private List<IEnvEntry> envEntryList = null;

    /**
     * List of &lt;ejb-ref&gt; elements.
     */
    private List<IEJBRef> ejbRefList = null;

    /**
     * List of &lt;ejb-local-ref&gt; elements.
     */
    private List<IEJBLocalRef> ejbLocalRefList = null;

    /**
     * List of &lt;resource-ref&gt; elements.
     */
    private List<IResourceRef> resourceRefList = null;

    /**
     * List of &lt;resource-env-ref&gt; elements.
     */
    private List<IResourceEnvRef> resourceEnvRefList = null;

    /**
     * List of &lt;message-destination-ref&gt; elements.
     */
    private List<IMessageDestinationRef> messageDestinationRefList = null;

    /**
     * List of &lt;service-ref&gt; elements.
     */
    private List<IServiceRef> serviceRefList = null;
    /**
     * List of &lt;around-invoke&gt; elements.
     */
    private List<IAroundInvoke> aroundInvokeList = null;

    /**
     * List of &lt;post-activate&gt; elements.
     */
    private List<ILifeCycleCallback> postActivateList = null;

    /**
     * List of &lt;pre-passivate&gt; elements.
     */
    private List<ILifeCycleCallback> prePassivateList = null;

    /**
     * List of &lt;post-construct&gt; elements.
     */
    private List<ILifeCycleCallback> postConstructList = null;

    /**
     * List of &lt;pre-destroy&gt; elements.
     */
    private List<ILifeCycleCallback> preDestroyList = null;

    /**
     * List of &lt;persistent-unit-ref&gt; elements.
     */
    private List<IPersistenceUnitRef> persistenceUnitRefList = null;

    /**
     * List of &lt;persistent-unit-ref&gt; elements.
     */
    private List<IPersistenceContextRef> persistenceContextRefList = null;

    /**
     * List of &lt;security-role-ref&gt; elements.
     */
    private List<ISecurityRoleRef> securityRoleRefList = null;

    /**
     * List of &lt;data-source&gt; elements.
     */
    private List<IDataSource> dataSourceList = null;

    /**
     * Constructor : build a new object for environment.
     */
    public AbsEnvironment() {
        this.envEntryList = new ArrayList<IEnvEntry>();
        this.ejbRefList = new ArrayList<IEJBRef>();
        this.ejbLocalRefList = new ArrayList<IEJBLocalRef>();
        this.resourceRefList = new ArrayList<IResourceRef>();
        this.resourceEnvRefList = new ArrayList<IResourceEnvRef>();
        this.messageDestinationRefList = new ArrayList<IMessageDestinationRef>();
        this.serviceRefList = new ArrayList<IServiceRef>();
        this.aroundInvokeList = new ArrayList<IAroundInvoke>();
        this.postActivateList = new ArrayList<ILifeCycleCallback>();
        this.postConstructList = new ArrayList<ILifeCycleCallback>();
        this.preDestroyList = new ArrayList<ILifeCycleCallback>();
        this.prePassivateList = new ArrayList<ILifeCycleCallback>();
        this.persistenceUnitRefList = new ArrayList<IPersistenceUnitRef>();
        this.persistenceContextRefList = new ArrayList<IPersistenceContextRef>();
        this.securityRoleRefList = new ArrayList<ISecurityRoleRef>();
        this.dataSourceList = new ArrayList<IDataSource>();
    }

    /**
     * Add a new env-entry element to this object.
     *
     * @param envEntry the ejb-ref object
     */
    public void addEnvEntry(final IEnvEntry envEntry) {
        envEntryList.add(envEntry);
    }

    /**
     * @return the list of all env-entry elements
     */
    public List<IEnvEntry> getEnvEntryList() {
        return envEntryList;
    }


    /**
     * Gets the list of &lt;around-invoke&gt; elements.
     *
     * @return list of &lt;around-invoke&gt; elements.
     */
    public List<IAroundInvoke> getAroundInvokeList() {
        return aroundInvokeList;
    }

    /**
     * Adds a new &lt;around-invoke&gt; element to assembly descriptor.
     *
     * @param aroundInvoke the &lt;around-invoke&gt; element
     */
    public void addAroundInvoke(final IAroundInvoke aroundInvoke) {
        aroundInvokeList.add(aroundInvoke);
    }

    /**
     * Gets the list of &lt;post-construct&gt; elements.
     *
     * @return list of &lt;post-construct&gt; elements.
     */
    public List<ILifeCycleCallback> getPostConstructCallbackList() {
        return postConstructList;
    }

    /**
     * Adds a new &lt;post-construct&gt; element to assembly descriptor.
     *
     * @param postConstructCallback the &lt;post-construct&gt; element
     */
    public void addPostConstructCallback(final ILifeCycleCallback postConstructCallback) {
        postConstructList.add(postConstructCallback);
    }

    /**
     * Gets the list of &lt;post-activate&gt; elements.
     *
     * @return list of &lt;post-activate&gt; elements.
     */
    public List<ILifeCycleCallback> getPostActivateCallbackList() {
        return postActivateList;
    }

    /**
     * Adds a new &lt;post-activate&gt; element to assembly descriptor.
     *
     * @param postActivateCallback the &lt;post-activate&gt; element
     */
    public void addPostActivateCallback(final ILifeCycleCallback postActivateCallback) {
        postActivateList.add(postActivateCallback);
    }

    /**
     * Gets the list of &lt;pre-destroy&gt; elements.
     *
     * @return list of &lt;pre-destroy&gt; elements.
     */
    public List<ILifeCycleCallback> getPreDestroyCallbackList() {
        return preDestroyList;
    }

    /**
     * Adds a new &lt;pre-destroy&gt; element to assembly descriptor.
     *
     * @param preDestroyCallback the &lt;pre-destroy&gt; element
     */
    public void addPreDestroyCallback(final ILifeCycleCallback preDestroyCallback) {
        preDestroyList.add(preDestroyCallback);
    }

    /**
     * Gets the list of &lt;pre-passivate&gt; elements.
     *
     * @return list of &lt;pre-passivate&gt; elements.
     */
    public List<ILifeCycleCallback> getPrePassivateCallbackList() {
        return prePassivateList;
    }

    /**
     * Adds a new &lt;pre-passivate&gt; element to assembly descriptor.
     *
     * @param prePassivateCallback the &lt;pre-passivate&gt; element
     */
    public void addPrePassivateCallback(final ILifeCycleCallback prePassivateCallback) {
        prePassivateList.add(prePassivateCallback);
    }

    /**
     * Gets the list of &lt;ejb-ref&gt; elements.
     *
     * @return list of &lt;ejb-ref&gt; elements.
     */
    public List<IEJBRef> getEJBRefList() {
        return ejbRefList;
    }

    /**
     * Adds a new &lt;ejb-ref&gt; element.
     *
     * @param ejbRef the &lt;ejb-ref&gt; element
     */
    public void addEJBRef(final IEJBRef ejbRef) {
        ejbRefList.add(ejbRef);
    }

    /**
     * Gets the list of &lt;resource-ref&gt; elements.
     *
     * @return list of &lt;resource-ref&gt; elements.
     */
    public List<IResourceRef> getResourceRefList() {
        return resourceRefList;
    }

    /**
     * Adds a new &lt;resource-ref&gt; element.
     *
     * @param resourceRef the &lt;resource-ref&gt; element
     */
    public void addResourceRef(final IResourceRef resourceRef) {
        resourceRefList.add(resourceRef);
    }

    /**
     * Gets the list of &lt;message-destination-ref&gt; elements.
     *
     * @return list of &lt;message-destination-ref&gt; elements.
     */
    public List<IMessageDestinationRef> getMessageDestinationRefList() {
        return messageDestinationRefList;
    }

    /**
     * Adds a new &lt;message-destination-ref&gt; element.
     *
     * @param messageDestinationRef the &lt;message-destination-ref&gt; element
     */
    public void addMessageDestinationRef(final IMessageDestinationRef messageDestinationRef) {
        messageDestinationRefList.add(messageDestinationRef);
    }


    /**
     * Gets the list of &lt;service-ref&gt; elements.
     *
     * @return list of &lt;service-ref&gt; elements.
     */
    public List<IServiceRef> getServiceRefList() {
        return serviceRefList;
    }

    /**
     * Adds a new &lt;service-ref&gt; element.
     *
     * @param serviceRef the &lt;service-ref&gt; element
     */
    public void addServiceRef(final IServiceRef serviceRef) {
        serviceRefList.add(serviceRef);
    }

    /**
     * Gets the list of &lt;persistent-unit-ref&gt; elements.
     *
     * @return list of &lt;persistent-unit-ref&gt; elements.
     */
    public List<IPersistenceUnitRef> getPersistenceUnitRefList() {
        return persistenceUnitRefList;
    }

    /**
     * Adds a new &lt;persistent-unit-ref&gt; element.
     *
     * @param persistenceUnitRef the &lt;persistent-unit-ref&gt; element
     */
    public void addPersistenceUnitRef(final IPersistenceUnitRef persistenceUnitRef) {
        persistenceUnitRefList.add(persistenceUnitRef);
    }

    /**
     * Gets the list of &lt;persistent-unit-ref&gt; elements.
     *
     * @return list of &lt;persistent-unit-ref&gt; elements.
     */
    public List<IPersistenceContextRef> getPersistenceContextRefList() {
        return persistenceContextRefList;
    }

    /**
     * Adds a new &lt;persistent-unit-ref&gt; element.
     *
     * @param persistenceContextRef the &lt;persistent-unit-ref&gt; element
     */
    public void addPersistenceContextRef(final IPersistenceContextRef persistenceContextRef) {
        persistenceContextRefList.add(persistenceContextRef);
    }

    /**
     * Gets the list of &lt;resource-env-ref&gt; elements.
     *
     * @return list of &lt;resource-env-ref&gt; elements.
     */
    public List<IResourceEnvRef> getResourceEnvRefList() {
        return resourceEnvRefList;
    }

    /**
     * Adds a new &lt;resource-ref&gt; element.
     *
     * @param resourceEnvRef the &lt;resource-ref&gt; element
     */
    public void addResourceEnvRef(final IResourceEnvRef resourceEnvRef) {
        resourceEnvRefList.add(resourceEnvRef);
    }

    /**
     * Gets the list of &lt;ejb-local-ref&gt; elements.
     *
     * @return list of &lt;ejb-local-ref&gt; elements.
     */
    public List<IEJBLocalRef> getEJBLocalRefList() {
        return ejbLocalRefList;
    }

    /**
     * Adds a new &lt;ejb-local-ref&gt; element.
     *
     * @param ejLocalRef the &lt;ejb-local-ref&gt; element
     */
    public void addEJBLocalRef(final IEJBLocalRef ejLocalRef) {
        ejbLocalRefList.add(ejLocalRef);
    }

    /**
     * Gets the list of &lt;security-role-ref&gt; elements.
     *
     * @return list of &lt;security-role-ref&gt; elements.
     */
    public List<ISecurityRoleRef> getSecurityRoleRefList() {
        return securityRoleRefList;
    }

    /**
     * Adds a new &lt;security-role-ref&gt; element.
     *
     * @param ejLocalRef the &lt;security-role-ref&gt; element
     */
    public void addSecurityRoleRef(final ISecurityRoleRef securityRoleRef) {
        this.securityRoleRefList.add(securityRoleRef);
    }

    public List<IDataSource> getDataSourceList() {
        return dataSourceList;
    }

    public void addDataSource(IDataSource dataSource) {
        this.dataSourceList.add(dataSource);
    }

}
