/**
 * OW2 Utilities
 * Copyright (C) 2009-2012 Bull S.A.S.
 * Contact: utilities@ow2.org
 *
 * Licensed  under the  Apache License,  Version 2.0  (the "License");
 * you may not use  this file  except in  compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed  under the  License is distributed on an "AS IS" BASIS,
 * WITHOUT  WARRANTIES OR CONDITIONS  OF ANY KIND, either  express  or
 * implied.
 *
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.common.impl;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertFalse;
import static org.testng.Assert.assertNotNull;

import java.net.URL;
import java.sql.Connection;
import java.util.List;

import org.ow2.util.ee.metadata.common.api.struct.IJAnnotationSqlDataSourceDefinition;
import org.ow2.util.ee.metadata.common.api.xml.struct.IDataSource;
import org.ow2.util.ee.metadata.common.api.xml.struct.IEnvironment;
import org.ow2.util.ee.metadata.common.impl.clazz.DataSourceDefinitionClass;
import org.ow2.util.ee.metadata.common.impl.view.CommonView;
import org.ow2.util.ee.metadata.common.impl.xml.parsing.ParsingException;
import org.ow2.util.ee.metadata.common.impl.xml.struct.AbsEnvironment;
import org.testng.annotations.BeforeTest;
import org.testng.annotations.Test;

/**
 * @author Loic Albertin
 */
public class DataSourceMergingTestCase extends AbstractAnnotationParsingTestCase {
    /**
     * This is the component environment filled by the Loader. We'll check the
     * loader behavior by checking values "injected" in that object.
     */
    private IEnvironment env = null;

    /**
     * Create an empty object that will be filled automatically.
     */
    @Override
    @BeforeTest(alwaysRun = true)
    protected void setUp() {
        super.setUp();
        env = new AbsEnvironment();
    }

    /**
     * Gets an url for a given filename.
     *
     * @param fileName the name of the file to search.
     *
     * @return an URL if found
     *
     * @throws org.ow2.util.ee.metadata.common.impl.xml.parsing.ParsingException
     *          if no URL is found.
     */
    private static URL getURL(final String fileName) throws ParsingException {
        URL url = DataSourceMergingTestCase.class.getResource(fileName);
        if (url == null) {
            throw new ParsingException("URL for '" + fileName + "' was not found.");
        }
        return url;
    }

    @Override
    protected String getParsedClassName() {
        return DataSourceDefinitionClass.class.getName();
    }

    @Override
    protected ClassLoader getParsedClassClassLoader() {
        return DataSourceDefinitionClass.class.getClassLoader();
    }

    @Test
    public void testXMLParsing() throws ParsingException {

        // Init
        URL url = getURL("dataSource01.xml");

        CommonDeploymentDescLoaderTested loader = new CommonDeploymentDescLoaderTested(url);

        // parse + map to Java Objects
        loader.analyzeEnvironment(loader.parse(), env);

        //        simple checks
        assertNotNull(env.getDataSourceList());
        assertEquals(env.getDataSourceList().size(), 2);
        IDataSource dataSource = env.getDataSourceList().get(0);
        assertEquals(dataSource.getClassName(), "com.acme.jdbc.Driver");
        assertEquals(dataSource.getName(), "java:app/AcmeDataSourceName");
        assertEquals(dataSource.getDescription(), "Data source desc");
        assertEquals(dataSource.getServerName(), "acmeServer");
        assertEquals(dataSource.getPortNumber(), Integer.valueOf(3307));
        assertEquals(dataSource.getDatabaseName(), "testdb");
        assertEquals(dataSource.getUrl(), "jdbc:mysql://localhost:3307/testdb");
        assertEquals(dataSource.getUser(), "u1");
        assertEquals(dataSource.getPassword(), "p2");
        assertEquals(dataSource.getProperties().length, 2);
        assertEquals(dataSource.getProperties()[0], "vendor.prop.1=true");
        assertEquals(dataSource.getProperties()[1], "vendor.prop.2=foo");
        assertEquals(dataSource.getLoginTimeout(), Integer.valueOf(10));
        assertFalse(dataSource.isTransactional());
        assertEquals(dataSource.getIsolationLevel(), Integer.valueOf(Connection.TRANSACTION_SERIALIZABLE));
        assertEquals(dataSource.getInitialPoolSize(), Integer.valueOf(12));
        assertEquals(dataSource.getMaxPoolSize(), Integer.valueOf(13));
        assertEquals(dataSource.getMinPoolSize(), Integer.valueOf(14));
        assertEquals(dataSource.getMaxIdleTime(), Integer.valueOf(15));
        assertEquals(dataSource.getMaxStatements(), Integer.valueOf(16));
    }

    @Test(dependsOnMethods = {"createMetadata", "testXMLParsing"})
    public void mergeMetadatas() {
        TestCommonMetadataMerge merger = new TestCommonMetadataMerge(classMetadata);
        CommonView commonClassView = classMetadata.as(CommonView.class);
        merger.applyJndiEnvironmentRefsGroup(env, commonClassView);
    }

    @Test(dependsOnMethods = {"mergeMetadatas"})
    public void testMergeOnClass() {

        CommonView view = classMetadata.as(CommonView.class);

        List<IJAnnotationSqlDataSourceDefinition> jAnnotationSqlDataSourceDefinitions =
                view.getJAnnotationSqlDataSourceDefinitions();
        assertNotNull(jAnnotationSqlDataSourceDefinitions);
        assertEquals(jAnnotationSqlDataSourceDefinitions.size(), 2);
        IJAnnotationSqlDataSourceDefinition jAnnotationSqlDataSourceDefinition =
                jAnnotationSqlDataSourceDefinitions.get(0);
        assertEquals(jAnnotationSqlDataSourceDefinition.getClassName(), "com.acme.jdbc.Driver");
        assertEquals(jAnnotationSqlDataSourceDefinition.getName(), "java:app/AcmeDataSourceName");
        assertEquals(jAnnotationSqlDataSourceDefinition.getDescription(), "Data source desc");
        assertEquals(jAnnotationSqlDataSourceDefinition.getServerName(), "acmeServer");
        assertEquals(jAnnotationSqlDataSourceDefinition.getPortNumber(), 3307);
        assertEquals(jAnnotationSqlDataSourceDefinition.getDatabaseName(), "testdb");
        assertEquals(jAnnotationSqlDataSourceDefinition.getUrl(), "jdbc:mysql://localhost:3307/testdb");
        assertEquals(jAnnotationSqlDataSourceDefinition.getUser(), "u1");
        assertEquals(jAnnotationSqlDataSourceDefinition.getPassword(), "p2");
        assertEquals(jAnnotationSqlDataSourceDefinition.getProperties().length, 2);
        assertEquals(jAnnotationSqlDataSourceDefinition.getProperties()[0], "vendor.prop.1=true");
        assertEquals(jAnnotationSqlDataSourceDefinition.getProperties()[1], "vendor.prop.2=foo");
        assertEquals(jAnnotationSqlDataSourceDefinition.getLoginTimeout(), 10);
        assertFalse(jAnnotationSqlDataSourceDefinition.isTransactional());
        assertEquals(jAnnotationSqlDataSourceDefinition.getIsolationLevel(), Connection.TRANSACTION_SERIALIZABLE);
        assertEquals(jAnnotationSqlDataSourceDefinition.getInitialPoolSize(), 12);
        assertEquals(jAnnotationSqlDataSourceDefinition.getMaxPoolSize(), 13);
        assertEquals(jAnnotationSqlDataSourceDefinition.getMinPoolSize(), 14);
        assertEquals(jAnnotationSqlDataSourceDefinition.getMaxIdleTime(), 15);
        assertEquals(jAnnotationSqlDataSourceDefinition.getMaxStatements(), 16);
    }
}
