/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.common.impl;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertNotNull;
import static org.testng.Assert.assertTrue;

import java.net.URI;
import java.net.URL;
import java.util.List;

import javax.xml.namespace.QName;

import org.ow2.util.ee.metadata.common.api.xml.struct.IEnvironment;
import org.ow2.util.ee.metadata.common.api.xml.struct.IPortComponentRef;
import org.ow2.util.ee.metadata.common.api.xml.struct.IServiceRef;
import org.ow2.util.ee.metadata.common.api.xml.struct.IHandlerChains;
import org.ow2.util.ee.metadata.common.api.xml.struct.IHandlerChain;
import org.ow2.util.ee.metadata.common.api.xml.struct.IHandler;
import org.ow2.util.ee.metadata.common.impl.xml.parsing.ParsingException;
import org.ow2.util.ee.metadata.common.impl.xml.struct.AbsEnvironment;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

/**
 * Tests the parsing of an pseudo environment with only service-ref.
 * @author Guillaume Sauthier
 */
public class ServiceRefLoadingTestCase {

    /**
     * This is the component environment filled by the Loader. We'll check the
     * loader behavior by checking values "injected" in that object.
     */
    private IEnvironment env = null;

    /**
     * Create an empty object that will be filled automatically.
     */
    @BeforeMethod
    private void setUp() {
        env = new AbsEnvironment();
    }

    /**
     * Gets an url for a given filename.
     * @param fileName the name of the file to search.
     * @return an URL if found
     * @throws org.ow2.util.ee.metadata.common.impl.xml.parsing.ParsingException
     *         if no URL is found.
     */
    private static URL getURL(final String fileName) throws ParsingException {
        URL url = ServiceRefLoadingTestCase.class.getResource(fileName);
        if (url == null) {
            throw new ParsingException("URL for '" + fileName + "' was not found.");
        }
        return url;
    }

    /**
     * Parse a given filename and gets the resulting parsing.
     * @param fileName the name of the file to parse
     * @return a struct representing the EJB
     * @throws org.ow2.util.ee.metadata.common.impl.xml.parsing.ParsingException
     *         if parsing has failed.
     */
    private List<IServiceRef> getServiceRefs(final String fileName) throws ParsingException {

        // Init
        URL url = getURL(fileName);
        CommonDeploymentDescLoaderTested loader = new CommonDeploymentDescLoaderTested(url);

        // parse + map to Java Objects
        loader.analyzeEnvironment(loader.parse(), env);

        // get the ServiceRefs
        return env.getServiceRefList();

    }

    @Test
    public void testSimpleServiceReference() throws Exception {

        List<IServiceRef> refs = getServiceRefs("service-ref01.xml");

        assertNotNull(refs);
        assertEquals(1, refs.size(), "Expected 1 reference");

        IServiceRef ref = refs.get(0);
        assertEquals(ref.getName(), "SR01", "Wrong name");
        assertEquals(ref.getServiceInterface(), "org.ow2.dummy.ServiceInterface", "Wrong service-interface");
        assertEquals(ref.getServiceRefType(), "org.ow2.dummy.WebServiceEndpoint", "Wrong service-ref-type");
        assertEquals(ref.getWsdlFile(), new URI("META-INF/dummy.wsdl"), "Wrong wsdl-file");

        QName expected = new QName("urn:org.ow2.util.ee", "Service", "test");
        assertEquals(ref.getServiceQName(), expected, "Wrong service-qname");

    }

    @Test(dependsOnMethods = "testSimpleServiceReference")
    public void testServiceReferenceWithPortComponentRef() throws Exception {

        List<IServiceRef> refs = getServiceRefs("service-ref02.xml");

        assertNotNull(refs);
        assertEquals(1, refs.size(), "Expected 1 reference");

        IServiceRef ref = refs.get(0);
        assertEquals(ref.getName(), "SR02", "Wrong name");

        List<IPortComponentRef> portRefs = ref.getPortComponentRefList();
        assertNotNull(portRefs);
        assertEquals(1, portRefs.size(), "Expected 1 port-component-ref");

        IPortComponentRef portRef = portRefs.get(0);

        assertEquals(portRef.getServiceEndpointInterface(), "org.ow2.dummy.IEndpoint", "Wrong endpoint interface");
        assertTrue(portRef.isEnableMTOM(), "Wrong enable-mtom value");
        assertEquals(portRef.getPortComponentLink(), "link-name", "Wrong port-component-link");
    }

    @Test(dependsOnMethods = "testSimpleServiceReference")
    public void testServiceReferenceWithHandlerChains() throws Exception {

        List<IServiceRef> refs = getServiceRefs("service-ref03.xml");

        assertNotNull(refs);
        assertEquals(1, refs.size(), "Expected 1 reference");

        IServiceRef ref = refs.get(0);
        assertEquals(ref.getName(), "SR04", "Wrong name");

        IHandlerChains handlerChains = ref.getHandlerChains();
        assertNotNull(handlerChains);

        // Check handler-chains
        List<IHandlerChain> chains = handlerChains.getHandlerChains();
        assertNotNull(chains);
        assertEquals(chains.size(), 2, "Should only have 2 handler-chain");

        // Check handler-chain
        IHandlerChain handlerChain = chains.get(0);
        assertEquals(handlerChain.getServiceNamePattern(), new QName("urn:pattern1", "Service"));
        assertEquals(handlerChain.getPortNamePattern(), new QName("urn:pattern2", "*Port"));

        // Check bindings
        List<String> bindings = handlerChain.getProtocolBindings();
        assertNotNull(bindings);
        assertEquals(bindings.size(), 1, "Should only have 1 protocol-binding");
        assertEquals(bindings.get(0), "##SOAP11_HTTP");

        // Check handlers
        List<IHandler> handlers = handlerChain.getHandlers();
        assertNotNull(handlers);
        assertEquals(handlers.size(), 2, "Should only have 2 handlers");

        IHandler h1 = handlers.get(0);
        assertEquals(h1.getHandlerName(), "H1");
        assertEquals(h1.getHandlerClass(), "org.ow2.dummy.Handler");
        assertEquals(h1.getInitParams().size(), 0);
        assertEquals(h1.getSoapHeader(), null);
        assertEquals(h1.getSoapRole(), null);

        IHandler h2 = handlers.get(1);
        assertEquals(h2.getHandlerName(), "H2");
        assertEquals(h2.getHandlerClass(), "org.ow2.dummy.Handler2");
        assertEquals(h2.getSoapHeader(), new QName("urn:org.ow2.util.ee", "Header"));
        assertEquals(h2.getSoapRole(), null);
        assertEquals(h2.getInitParams().size(), 1);
        assertEquals(h2.getInitParams().get("P1"), "V1");
        
        IHandlerChain handlerChain2 = chains.get(1);
        assertEquals(handlerChain2.getServiceNamePattern(), new QName("*"));
        List<String> bindings2 = handlerChain2.getProtocolBindings();
        assertNotNull(bindings2);
        assertEquals(bindings2.size(), 2, "Should only have 2 protocol-binding");
        assertEquals(bindings2.get(0), "##SOAP11_HTTP");
        assertEquals(bindings2.get(1), "##SOAP12_HTTP");


    }

    @Test
    public void testMultipleServiceReferences() throws Exception {

        List<IServiceRef> refs = getServiceRefs("service-ref04.xml");

        assertNotNull(refs);
        assertEquals(2, refs.size(), "Expected 2 reference");

        IServiceRef ref1 = refs.get(0);
        assertEquals(ref1.getName(), "SR05.1", "Wrong name for ref1");

        IServiceRef ref2 = refs.get(1);
        assertEquals(ref2.getName(), "SR05.2", "Wrong name for ref2");
    }

}
