/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.common.impl;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertNotNull;
import static org.testng.Assert.assertNull;

import java.net.URL;

import org.ow2.util.ee.metadata.common.api.struct.IJaxwsWebServiceRef;
import org.ow2.util.ee.metadata.common.api.xml.struct.IEnvironment;
import org.ow2.util.ee.metadata.common.impl.clazz.ClassForMerge;
import org.ow2.util.ee.metadata.common.impl.view.CommonView;
import org.ow2.util.ee.metadata.common.impl.xml.parsing.ParsingException;
import org.ow2.util.ee.metadata.common.impl.xml.struct.AbsEnvironment;
import org.ow2.util.scan.api.metadata.IFieldMetadata;
import org.ow2.util.scan.api.metadata.IMethodMetadata;
import org.testng.annotations.BeforeTest;
import org.testng.annotations.Test;

/**
 * Test annotation parsing.
 * @author Guillaume Sauthier
 */
public class WebServiceRefsMergingTestCase extends AbstractAnnotationParsingTestCase {

    /**
     * This is the component environment filled by the Loader. We'll check the
     * loader behavior by checking values "injected" in that object.
     */
    private IEnvironment env = null;

    /**
     * Create an empty object that will be filled automatically.
     */
    @Override
    @BeforeTest(alwaysRun=true)
    protected void setUp() {
        super.setUp();
        env = new AbsEnvironment();
    }

    /**
     * Gets an url for a given filename.
     * @param fileName the name of the file to search.
     * @return an URL if found
     * @throws org.ow2.util.ee.metadata.common.impl.xml.parsing.ParsingException
     *         if no URL is found.
     */
    private static URL getURL(final String fileName) throws ParsingException {
        URL url = WebServiceRefsMergingTestCase.class.getResource(fileName);
        if (url == null) {
            throw new ParsingException("URL for '" + fileName + "' was not found.");
        }
        return url;
    }

    @Override
    protected String getParsedClassName() {
        return ClassForMerge.class.getName();
    }

    @Override
    protected ClassLoader getParsedClassClassLoader() {
        return ClassForMerge.class.getClassLoader();
    }

    @Test
    public void testXMLParsing() throws ParsingException {

        // Init
        URL url = getURL("service-refs-for-merge.xml");

        CommonDeploymentDescLoaderTested loader = new CommonDeploymentDescLoaderTested(url);

        // parse + map to Java Objects
        loader.analyzeEnvironment(loader.parse(), env);

        // simple checks
        assertNotNull(env.getServiceRefList());
    }

    @Test(dependsOnMethods = {"createMetadata", "testXMLParsing"})
    public void mergeMetadatas() {
        TestCommonMetadataMerge merger = new TestCommonMetadataMerge(classMetadata);
        merger.applyJndiEnvironmentRefsGroup(env, new AbsEnvironmentRefAccessor());
    }

    @Test(dependsOnMethods = "mergeMetadatas")
    public void testMergeOnField1() throws Exception {
        IFieldMetadata field = getUniqueFieldMetadata("pseudoService");
        CommonView view = field.as(CommonView.class);

        IJaxwsWebServiceRef ref = view.getJaxwsWebServiceRef();
        assertNotNull(ref);
        assertEquals(ref.getMappedName(), "Global", "mappedName attribute overridded");
        assertEquals(ref.getName(), "SR01", "name attribute was not overridded");
        assertEquals(ref.getHandlerChainFile(),
                     "META-INF/field-level.xml",
                     "@HandlerChain.file not associated with field 'pseudoService'");
        assertEquals(ref.getHandlerChains(), null);

        assertNotNull(view.getJwsHandlerChain());
    }

    @Test(dependsOnMethods = "mergeMetadatas")
    public void testNonMergedField() throws Exception {
        IFieldMetadata field = getUniqueFieldMetadata("onlyAnnotation");
        CommonView view = field.as(CommonView.class);

        IJaxwsWebServiceRef ref = view.getJaxwsWebServiceRef();
        assertNotNull(ref);
        assertEquals(ref.getName(), "override", "name attribute was not overridded");
        assertEquals(ref.getHandlerChains(), null);
    }

    @Test(dependsOnMethods = "mergeMetadatas")
    public void testMergeOnField2() throws Exception {
        IFieldMetadata field = getUniqueFieldMetadata("pseudoServiceWithoutHC");
        CommonView view = field.as(CommonView.class);

        IJaxwsWebServiceRef ref = view.getJaxwsWebServiceRef();
        assertNotNull(ref);
        assertEquals(ref.getType(), "org.ow2.dummy.WebServiceEndpoint", "type attribute not overridded");
        assertEquals(ref.getWsdlLocation(), "META-INF/dummy.wsdl", "wsdlLocation attribute was not overridded");
        assertEquals(ref.getHandlerChainFile(), null);
        assertEquals(ref.getHandlerChains(), null);

        assertNull(view.getJwsHandlerChain());
    }

    @Test(dependsOnMethods = "mergeMetadatas")
    public void testMergeOnMethod1() throws Exception {
        IMethodMetadata methodMetadata = getUniqueMethodMetadata("setPseudoService2");
        CommonView view = methodMetadata.as(CommonView.class);

        IJaxwsWebServiceRef ref = view.getJaxwsWebServiceRef();
        assertNotNull(ref);
        assertEquals(ref.getHandlerChainFile(), "META-INF/method-level.xml");
        assertEquals(ref.getHandlerChains(), null);

        assertNotNull(view.getJwsHandlerChain());
    }

    @Test(dependsOnMethods = "mergeMetadatas")
    public void testMergeOnClass() throws Exception {
        CommonView view = classMetadata.as(CommonView.class);
        assertEquals(view.getJaxwsWebServiceRef(), null);
        assertEquals(view.getJaxwsWebServiceRefs().size(), 2);

        // Check ref1
        IJaxwsWebServiceRef ref1 = view.getJaxwsWebServiceRefs().get(0);
        assertEquals(ref1.getName(), "WSRs.wsr1");
        assertEquals(ref1.getHandlerChainFile(), "META-INF/class-level.xml");
        assertEquals(ref1.getHandlerChains(), null);

        // Check ref2
        IJaxwsWebServiceRef ref2 = view.getJaxwsWebServiceRefs().get(1);
        assertEquals(ref2.getName(), "WSRs.wsr2");
        assertEquals(ref2.getHandlerChainFile(), "META-INF/class-level.xml");
        assertEquals(ref2.getHandlerChains(), null);
    }

}
