/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.ear.impl;

import java.util.ArrayList;
import java.util.List;

import org.ow2.util.archive.api.ArchiveException;
import org.ow2.util.archive.api.IArchive;
import org.ow2.util.archive.impl.ArchiveManager;
import org.ow2.util.ee.metadata.car.api.ICarMetadataFactory;
import org.ow2.util.ee.metadata.car.impl.CarMetadataFactory;
import org.ow2.util.ee.metadata.ear.api.IEarMetadata;
import org.ow2.util.ee.metadata.ear.api.IEarMetadataFactory;
import org.ow2.util.ee.metadata.ear.api.exception.EARMetadataException;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarMetadataFactory;
import org.ow2.util.ee.metadata.ejbjar.impl.EjbJarMetadataFactory;
import org.ow2.util.ee.metadata.war.impl.WarMetadataFactory;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.scan.api.IClassesLocator;
import org.ow2.util.scan.api.IScanner;
import org.ow2.util.scan.api.classlocator.ClassLoaderClassesLocator;
import org.ow2.util.url.URLUtils;
import org.ow2.util.url.URLUtilsException;

/**
 * Implementation of ear metadata.
 * @author Gael Lalire
 */
public class EarMetadataFactory implements IEarMetadataFactory {


    /**
     * Application.xml file entry in the EAR file.
     */
    private static final String APPLICATION_XML_ENTRY = "META-INF/application.xml";

    /**
     * Logger.
     */
    private static final Log LOGGER = LogFactory.getLog(EarMetadataFactory.class);

    /**
     * The factory which manage war.
     */
    private final WarMetadataFactory warMetadataFactory;

    /**
     * The factory which manage client archive.
     */
    private final ICarMetadataFactory carMetadataFactory;

    /**
     * The factory which manage ejbjar.
     */
    private final IEjbJarMetadataFactory ejbJarMetadataFactory;


    /**
     * Constructor.
     * @param scanner the scanner which analyze class bytecode.
     */
    public EarMetadataFactory(final IScanner scanner) {
        warMetadataFactory = new WarMetadataFactory(scanner, ArchiveManager.getInstance());
        carMetadataFactory = new CarMetadataFactory(scanner);
        ejbJarMetadataFactory = new EjbJarMetadataFactory(scanner);
    }

    /**
     * @param deployable the ear deployable (getOriginalDeployable must not be
     *        null ie it's already decompressed archive)
     * @return a new ear deployable metadata
     * @throws EARMetadataException if ear is not uncompressed
     */
    @Override
    public IEarMetadata createArchiveMetadata(final IArchive archive) throws EARMetadataException {
        return createArchiveMetadata(archive, null);
    }

    /**
     * @param deployable the ear deployable (getOriginalDeployable must not be
     *        null ie it's already decompressed archive)
     * @param classLoader a classloader to use if a class is not in archive
     * @return a new ear deployable metadata
     * @throws EARMetadataException if ear is not uncompressed
     */
    @Override
    public IEarMetadata createArchiveMetadata(final IArchive archive, final ClassLoader classLoader)
            throws EARMetadataException {

        long tAnalyzeStart = System.currentTimeMillis();

        try {
            if (!URLUtils.urlToFile2(archive.getURL()).isDirectory()) {
                throw new EARMetadataException("ear must be uncompressed (local directory expected)");
            }
        } catch (URLUtilsException | ArchiveException e) {
            throw new EARMetadataException("ear must be uncompressed (local directory expected)");
        }

        List<IClassesLocator> libraryList = new ArrayList<IClassesLocator>();
        /*for (LibDeployable libDeployable : libDeployableList) {
            libraryList.add(new ArchiveClassesLocator(libDeployable.getArchive()));
        }*/
        if (classLoader != null) {
            libraryList.add(new ClassLoaderClassesLocator(classLoader));
        } else {
            libraryList.add(new ClassLoaderClassesLocator(ClassLoader.getSystemClassLoader()));
        }
        EarMetadata earDeployableMetadata = new EarMetadata(archive);


        if (LOGGER.isDebugEnabled()) {
            long tAnalyzeStartEnd = System.currentTimeMillis();
            LOGGER.debug("Analyze of file {0} took {1} ms.", archive.getName(),
                    (tAnalyzeStartEnd - tAnalyzeStart));
        }

        return earDeployableMetadata;
    }





}
