/**
 * Copyright 2013 Peergreen S.A.S.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ow2.util.ee.metadata.ejbjar.api.view;

import java.util.List;
import java.util.Map;

import org.ow2.util.ee.metadata.common.api.struct.ITransactionAttributeType;
import org.ow2.util.ee.metadata.common.api.struct.ITransactionManagementType;
import org.ow2.util.ee.metadata.common.api.xml.struct.ISecurityRoleRef;
import org.ow2.util.ee.metadata.ejbjar.api.ClassType;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IJClassInterceptor;
import org.ow2.util.ee.metadata.ejbjar.api.ILock;
import org.ow2.util.ee.metadata.ejbjar.api.InterceptorType;
import org.ow2.util.ee.metadata.ejbjar.api.interfaces.IEjbStatefulTimeout;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IApplicationException;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IConcurrencyManagementType;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IDependsOn;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJCommonBean;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJLocal;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJMessageDriven;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJRemote;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJSingleton;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJStateful;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJStateless;
import org.ow2.util.scan.api.metadata.IClassMetadata;

public interface IEJBClassView extends IEjbStatefulTimeout, ILock {

    /**
     * Sets the local interfaces of this class.
     * @param jLocal list of interfaces.
     */
    void setLocalInterfaces(final IJLocal jLocal);

    /**
     * Sets the remote interfaces of this class.
     * @param jRemote list of interfaces.
     */
    void setRemoteInterfaces(final IJRemote jRemote);

    /**
     * @return the local interfaces of this class.
     */
    IJLocal getLocalInterfaces();

    /**
     * @return the remote interfaces of this class.
     */
    IJRemote getRemoteInterfaces();

    /**
     * @return true if the class is a stateless class
     */
    boolean isStateless();

    /**
     * @return true if the class is a singleton class
     */
    boolean isSingleton();

    /**
     * @return true if the class is a stateful class
     */
    boolean isStateful();

    /**
     * @return true if the class is a session bean class
     */
    boolean isSession();

    /**
     * @return true if the class is an MDB class
     */
    boolean isMdb();

    /**
     * @return true if the class is a local bean.
     */
    boolean isLocalBean();

    /**
     * Defines if the bean is a local bean or not.
     * @param isLocalBean true if the bean is a local bean (and expose a no-interface local view)
     */
    void setLocalBean(boolean isLocalBean);

    /**
     * Sets the type of this class.
     * @param cType a type from enum class ClassType.
     * @see org.ow2.easybeans.deployment.annotations.ClassType
     */
    void setClassType(final ClassType cType);

    /**
     * @return Message driven attribute.
     */
    IJMessageDriven getJMessageDriven();

    /**
     * Sets the message driven bean object.
     * @param messageDriven attributes of message driven bean.
     */
    void setJMessageDriven(final IJMessageDriven messageDriven);

    /**
     * @return the &#64;{@link javax.ejb.RemoteHome} class name.
     */
    String getRemoteHome();

    /**
     * Sets the &#64;{@link javax.ejb.RemoteHome} class name.
     * @param remoteHome the class name.
     */
    void setRemoteHome(final String remoteHome);

    /**
     * @return the &#64;{@link javax.ejb.LocalHome} class name.
     */
    String getLocalHome();

    /**
     * Sets the &#64;{@link javax.ejb.LocalHome} class name.
     * @param localHome the class name.
     */
    void setLocalHome(final String localHome);

    /**
     * @return transaction management type from @see TransactionManagementType.
     */
    ITransactionManagementType getTransactionManagementType();

    /**
     * Sets transaction management type.
     * @see javax.ejb.TransactionManagementType
     * @param transactionManagementType value.
     *      (BEAN, CONTAINER)
     */
    void setTransactionManagementType(final ITransactionManagementType transactionManagementType);

    /**
     * @return transaction management type from
     * @see javax.ejb.ConcurrencyManagementType.
     */
    IConcurrencyManagementType getConcurrencyManagementType();

    /**
     * Sets concurrency management type.
     * @see javax.ejb.ConcurrencyManagementType
     * @param concurrencyManagementType value.
     *      BEAN,CONCURRENCY_NOT_ALLOWED, CONTAINER
     */
    void setConcurrencyManagementType(final IConcurrencyManagementType concurrencyManagementType);

    /**
     * @return transaction Attribute type.
     * @see javax.ejb.TransactionAttributeType
     */
    ITransactionAttributeType getTransactionAttributeType();

    /**
     * Set Transaction Attribute Type.
     * @see javax.ejb.TransactionAttributeType
     * @param transactionAttributeType the type of transaction.
     */
    void setTransactionAttributeType(final ITransactionAttributeType transactionAttributeType);

    /**
     * @return the &#64;{@link javax.ejb.ApplicationException} annotation.
     */
    IApplicationException getApplicationException();

    /**
     * Sets the object representing the &#64;{@link javax.ejb.ApplicationException} annotation.
     * @param applicationException object representation
     */
    void setApplicationException(final IApplicationException applicationException);

    /**
     * @return true if the class is a Bean
     */
    boolean isBean();

    /**
     * @return Map&lt;interceptor type &lt;--&gt; List of methods/class corresponding to the interceptor&gt; (interceptor classes)
     * of user interceptors that enhancer will use.
     */
    Map<InterceptorType, List<? extends IJClassInterceptor>> getExternalUserEasyBeansInterceptors();

    /**
     * Sets the list of user interceptors that enhancers will use.<br>
     * These interceptors are defined outside the bean class (interceptor classes).
     * @param externalUserInterceptors list of interceptors that enhancer will use.
     */
    void setExternalUserInterceptors(final Map<InterceptorType, List<? extends IJClassInterceptor>> externalUserInterceptors);

    /**
     * @return Map&lt;interceptor type &lt;--&gt; List of methods/class corresponding to the interceptor&gt; (bean classes)
     * of user interceptors that enhancer will use.
     */
    Map<InterceptorType, List<? extends IJClassInterceptor>> getInternalUserEasyBeansInterceptors();

    /**
     * Sets the list of user interceptors that enhancers will use.<br>
     * These interceptors are defined in bean class.
     * @param internalUserInterceptors list of interceptors that enhancer will use.
     */
    void setInternalUserInterceptors(final Map<InterceptorType, List<? extends IJClassInterceptor>> internalUserInterceptors);

    /**
     * @return list of global interceptors that enhancer will use. (ie : ENC)
     */
    List<? extends IJClassInterceptor> getGlobalEasyBeansInterceptors();

    /**
     * Sets the list of global interceptors that enhancers will use.
     * @param globalEasyBeansInterceptors list of interceptors that enhancer will use.
     */
    void setGlobalEasyBeansInterceptors(final List<? extends IJClassInterceptor> globalEasyBeansInterceptors);

    /**
     * @return the attributes for a Stateless/Stateful/MDB
     */
    IJCommonBean getJCommonBean();

    /**
     * Sets the attributes for a Stateless/Stateful/MDB.
     * @param commonBean the attributes
     */
    void setJCommonBean(final IJCommonBean commonBean);

    /**
     * @return the attributes for a Stateful
     */
    IJStateful getJStateful();

    /**
     * Sets the attributes for a Stateful.
     * @param jStateful the attributes
     */
    void setJStateful(final IJStateful jStateful);

    /**
     * @return the attributes for a Stateless
     */
    IJStateless getJStateless();

    /**
     * Sets the attributes for a Stateless.
     * @param jStateless the attributes
     */
    void setJStateless(final IJStateless jStateless);


    /**
     * @return the attributes for a Singleton
     */
    IJSingleton getJSingleton();

    /**
     * Sets the attributes for a Singleton.
     * @param jSingleton the attributes
     */
    void setJSingleton(final IJSingleton jSingleton);

    /**
     * @return true if the class has been modified.
     */
    boolean wasModified();

    /**
     * Defines that this class has been modified.
     */
    void setModified();

    /**
     * Sets the list of roles declared on this class.
     * @param declareRoles the list of roles.
     */
    void setDeclareRoles(final List<String> declareRoles);

    /**
     * @return the list of roles declared on this class.
     */
    List<String> getDeclareRoles();


    /**
     * Gets the list of &lt;security-role-ref&gt; elements.
     * @return list of &lt;security-role-ref&gt; elements.
     */
    List<ISecurityRoleRef> getSecurityRoleRefList();

    /**
     * Sets the list of &lt;security-role-ref&gt; element.
     * @param securityRoleRefList the list of &lt;security-role-ref&gt; element
     */
    void setSecurityRoleRefList(final List<ISecurityRoleRef> securityRoleRefList);

    /**
     * Set the list of roles allowed on this class/method.
     * @param rolesAllowed the list of roles.
     */
    void setRolesAllowed(final List<String> rolesAllowed);

    /**
     * @return the list of roles allowed on this class/method.
     */
    List<String> getRolesAllowed();

    /**
     * @return true if the interceptors have been ordered for this class.
     */
    boolean isOrderedInterceptors();

    /**
     * Sets flag to true if interceptors are ordered.
     * @param orderedInterceptors boolean with true/false.
     */
    void setOrderedInterceptors(final boolean orderedInterceptors);

    /**
     * Set the value of the remote Interface property.
     * @param remoteInterface the remote Interface property.
     */
    void setRemoteInterface(final String remoteInterface);

    /**
     * @return the name of the remote interface.
     */
    String getRemoteInterface();

    /**
     * construct a new JClass.
     * @param array interface list
     */
    void setInterfaces(final String[] array);

    /**
     * @return the deployable
     */
    IEjbJarMetadata getEjbJarDeployableMetadata();

    /**
     * Set the deployable metadata.
     * @param ejbJarArchiveMetadata the given deployable metadata
     */
    //void setEjbJarArchiveMetadata(IEjbJarMetadata ejbJarArchiveMetadata);

    /**
     * @return the name of the bean on which we are linked.
     * This is used for cloned classmetadata that are for the classes.
     */
    String getLinkedBean();

    /**
     * This class is linked to the given bean.
     * @param linkedBeanName the bean name
     */
    void setLinkedBean(final String linkedBeanName);

    /**
     * Get a classmetadata linked from this one.
     * @param className the name of the class that is wanted
     * @return the class metadata
     */
    IClassMetadata getLinkedClassMetadata(final String className);

    /**
     * This class needs eager initialization.
     * @param startup the boolean value.
     */
    void setStartup(final boolean startup);

    /**
     * @return true if this class(singleton) needs eager initialization.
     */
    boolean isStartup();

    /**
     * @return object representing list of &#64;
     *         {@link javax.interceptor.DependsOn}.
     */
    IDependsOn getDependsOn();

    /**
     * Sets the object representing the &#64;
     * {@link javax.ejb.DependsOn} annotation.
     * @param dependsOn list of beans
     */
    void setDependsOn(final IDependsOn dependsOn);

    /**
     * This bean is linked to a session synchronization and then should report TX lifecycle to the callbacks.
     */
    void setSessionSynchronization();

    /**
     * @return true if the bean is part of a session synchronization
     */
    boolean hasSessionSynchronization();


}