/**
 * Copyright 2013 Peergreen S.A.S.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ow2.util.ee.metadata.ejbjar.api.view;

import java.util.List;
import java.util.Map;

import org.ow2.util.ee.metadata.common.api.interfaces.IAnnotationSecurityRolesAllowed;
import org.ow2.util.ee.metadata.common.api.interfaces.ITransactionAttribute;
import org.ow2.util.ee.metadata.common.api.interfaces.IWebMethod;
import org.ow2.util.ee.metadata.common.api.struct.ITransactionAttributeType;
import org.ow2.util.ee.metadata.ejbjar.api.IJClassInterceptor;
import org.ow2.util.ee.metadata.ejbjar.api.ILock;
import org.ow2.util.ee.metadata.ejbjar.api.InterceptorType;
import org.ow2.util.ee.metadata.ejbjar.api.TransactionAttributeLevel;
import org.ow2.util.ee.metadata.ejbjar.api.interfaces.IEjbAccessTimeout;
import org.ow2.util.ee.metadata.ejbjar.api.interfaces.IEjbSchedule;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJEjbRemove;

/**
 * Defines view available on the methods
 * @author Florent Benoit
 */
public interface IEJBMethodView extends ITransactionAttribute,
ILock,
IEjbAccessTimeout,
IEjbSchedule,
IWebMethod,
IAnnotationSecurityRolesAllowed {


    /**
     * @return transaction Attribute type.
     * @see javax.ejb.TransactionAttributeType
     */
    @Override
    ITransactionAttributeType getTransactionAttributeType();

    /**
     * Sets Transaction Attribute Level.
     * @param transactionAttributeLevel the level of transaction.
     */
    void setTransactionAttributeLevel(final TransactionAttributeLevel transactionAttributeLevel);

    /**
     * @return transaction Attribute Level.
     */
    TransactionAttributeLevel getTransactionAttributeLevel();

    /**
     * Sets Transaction Attribute Type.
     * @see javax.ejb.TransactionAttributeType
     * @param transactionAttributeType the type of transaction.
     */
    @Override
    void setTransactionAttributeType(final ITransactionAttributeType transactionAttributeType);

    /**
     * @return true if the method is a business method.
     */
    boolean isBusinessMethod();

    /**
     * This method is a business method.
     * @param flag true/false if method is a business method.
     */
    void setBusinessMethod(final boolean flag);

    /**
     * @return true if the method is a lifecycle method.
     */
    boolean isLifeCycleMethod();

    /**
     * @return remove attributes  &#64;{@link javax.ejb.Remove} attributes
     */
    IJEjbRemove getJRemove();

    /**
     * Sets &#64;{@link javax.ejb.Remove} attribute.
     * @param remove contains the attribute with retainIfException.
     */
    void setRemove(final IJEjbRemove remove);

    /**
     * @return true if method has  &#64;{@link javax.ejb.PostActivate}
     */
    boolean isPostActivate();

    /**
     * Sets true if method has &#64;{@link javax.ejb.PostActivate}.
     * @param postActivate true/false.
     */
    void setPostActivate(final boolean postActivate);

    /**
     * @return true if method has &#64;{@link javax.ejb.PrePassivate}.
     */

    boolean isPrePassivate();

    /**
     * Sets true if method has &#64;{@link javax.ejb.PrePassivate}.
     * @param prePassivate true/false.
     */
    void setPrePassivate(final boolean prePassivate);

    /**
     * @return true if method has &#64;{@link javax.ejb.Timeout}.
     */
    boolean isTimeout();

    /**
     * Sets true if method has &#64;{@link javax.ejb.Timeout}.
     * @param timeout true/false.
     */
    void setTimeout(final boolean timeout);

    /**
     * @return true if method has &#64;{@link javax.interceptor.AroundInvoke}.
     */
    boolean isAroundInvoke();

    /**
     * Sets true if method has &#64;{@link javax.interceptor.AroundInvoke}.
     * @param aroundInvoke true/false
     */
    void setAroundInvoke(final boolean aroundInvoke);

    /**
     * @return true if this method won't use user interceptors.
     */
    boolean isExcludedClassInterceptors();

    /**
     * Flag this method as a method which exclude user interceptors.
     * @param excludeClassInterceptors true if this method is a method which exclude user interceptors.
     */
    void setExcludeClassInterceptors(final boolean excludeClassInterceptors);

    /**
     * @return Map&lt;interceptor type &lt;--&gt; List of methods/class corresponding to the interceptor&gt;
     * of user interceptors that enhancer will use.
     */
    Map<InterceptorType, List<? extends IJClassInterceptor>> getUserEasyBeansInterceptors();

    /**
     * Sets the list of user interceptors that enhancers will use.<br>
     * These interceptors are defined in bean class.
     * @param userInterceptors list of interceptors that enhancer will use.
     */
    void setUserInterceptors(final Map<InterceptorType, List<? extends IJClassInterceptor>> userInterceptors);

    /**
     * @return list of global interceptors that enhancer will use. (ie : Remove interceptor)
     */
    List<? extends IJClassInterceptor> getGlobalEasyBeansInterceptors();

    /**
     * Sets the list of global interceptors that enhancers will use.
     * @param globalEasyBeansInterceptors list of interceptors that enhancer will use.
     */
    void setGlobalEasyBeansInterceptors(final List<? extends IJClassInterceptor> globalEasyBeansInterceptors);

    /**
     * Set the list of roles allowed on this class/method.
     * @param rolesAllowed the list of roles.
     */
    @Override
    void setRolesAllowed(final List<String> rolesAllowed);

    /**
     * @return the list of roles allowed on this class/method.
     */
    @Override
    List<String> getRolesAllowed();

    /**
     * This method has DenyAll annotation.
     * @param denyAll the boolean value.
     */
    void setDenyAll(final boolean denyAll);

    /**
     * @return true if DenyAll annotation.
     */
    boolean hasDenyAll();

    /**
     * @return true if method is an afterBegin method.
     */
    boolean isAfterBegin();

    /**
     * Sets true if method is an afterBegin method.
     */
    void setAfterBegin();

    /**
     * @return true if method is an AfterCompletion method.
     */
    boolean isAfterCompletion();

    /**
     * Sets true if method is an AfterCompletion method.
     */
    void setAfterCompletion();

    /**
     * @return true if method is an beforeCompletion method.
     */
    boolean isBeforeCompletion();

    /**
     * Sets true if method is an beforeCompletion method.
     */
    void setBeforeCompletion();
}
