/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.ejbjar.impl;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.ow2.util.archive.api.ArchiveException;
import org.ow2.util.archive.api.IArchive;
import org.ow2.util.ee.metadata.common.impl.xml.parsing.ParsingException;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarMetadataFactory;
import org.ow2.util.ee.metadata.ejbjar.api.exceptions.EJBJARMetadataException;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IApplicationException;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IAssemblyDescriptor;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IEnterpriseBeans;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IInterceptor;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IInterceptors;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IMessageDriven;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.ISession;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.EjbJarSessionConfigurator;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.helper.EjbJarMetadataMerge;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.parsing.EJB3DeploymentDesc;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.EJB3;
import org.ow2.util.ee.metadata.ws.api.view.IWebservicesView;
import org.ow2.util.ee.metadata.ws.api.xml.merge.IClassMetadataFinder;
import org.ow2.util.ee.metadata.ws.api.xml.struct.IWebservices;
import org.ow2.util.ee.metadata.ws.impl.xml.merge.WebservicesMetadataMerger;
import org.ow2.util.ee.metadata.ws.impl.xml.parsing.Webservices12DeploymentDesc;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.scan.api.IClassesLocator;
import org.ow2.util.scan.api.IScanner;
import org.ow2.util.scan.api.ScanException;
import org.ow2.util.scan.api.classlocator.ArchiveClassesLocator;
import org.ow2.util.scan.api.classlocator.ClassLoaderClassesLocator;
import org.ow2.util.scan.api.metadata.IClassMetadata;

/**
 */
public class EjbJarMetadataFactory implements IEjbJarMetadataFactory {

    /**
     * Logger.
     */
    private static final Log LOGGER = LogFactory.getLog(EjbJarMetadataFactory.class);

    /**
     * Parsed classes, in a web-app are located in WEB-INF/classes.
     */
    private static final String WEBINF_CLASSES_PREFIX = "WEB-INF/classes/";

    private static final String CLASS_SUFFIX = ".class";

    private final IScanner scanner;

    /**
     * Constructor with a scanner.
     * @param scanner the scanner which analyze class bytecode.
     */
    public EjbJarMetadataFactory(final IScanner scanner) {
        this.scanner = scanner;
    }

    /**
     * Create war metadata.
     * @param archive the war archive
     * @return the metadata
     * @throws EJBJARMetadataException if error occurs
     */
    @Override
    public IEjbJarMetadata createArchiveMetadata(final IArchive archive) throws EJBJARMetadataException {
        return createArchiveMetadata(archive, (ClassLoader) null);
    }

    /**
     * Create war metadata.
     * @param archive the war archive
     * @param classLoader a classloader to use if a class is not in archive
     * @return the metadata
     * @throws EJBJARMetadataException if error occurs
     */
    @Override
    public IEjbJarMetadata createArchiveMetadata(final IArchive archive, final ClassLoader classLoader)
            throws EJBJARMetadataException {
        List<? extends IClassesLocator> classesLocatorList = null;
        if (classLoader != null) {
            classesLocatorList = Arrays.asList(new ClassLoaderClassesLocator(classLoader));
        } else {
            classesLocatorList = Arrays.asList(new ClassLoaderClassesLocator(ClassLoader.getSystemClassLoader()));
        }
        return createArchiveMetadata(archive, classesLocatorList);
    }

    /**
     * Extract a class list from xml.
     * @param war a object representation of web.xml
     * @param classNameSet a set to fill with class name inside web.xml
     */
    private void completeClassList(final EJB3 ejb3, final Set<String> classNameSet) {
        if (ejb3 != null) {

            // Get enterprise beans classes
            IEnterpriseBeans enterpriseBeans = ejb3.getEnterpriseBeans();
            if (enterpriseBeans != null) {
                // get session beans
                List<ISession> sessionList = enterpriseBeans.getSessionList();
                for (ISession session : sessionList) {
                    // get ejb-class
                    addClassToSetIfNotNull(session.getEjbClass(), classNameSet);
                }
                List<IMessageDriven> messageDrivenList = enterpriseBeans.getMessageDrivenList();
                for (IMessageDriven messageDriven : messageDrivenList) {
                    // get ejb-class
                    addClassToSetIfNotNull(messageDriven.getEjbClass(), classNameSet);
                }
            }

            // Get interceptor classes
            IInterceptors interceptors = ejb3.getInterceptors();
            if (interceptors != null) {
                List<IInterceptor> interceptorList = interceptors.getInterceptorList();
                if (interceptorList != null) {
                    for (IInterceptor interceptor : interceptorList) {
                        addClassToSetIfNotNull(interceptor.getInterceptorClass(), classNameSet);
                    }
                }
            }

            // Get application exception classes
            IAssemblyDescriptor assemblyDescriptor = ejb3.getAssemblyDescriptor();
            if (assemblyDescriptor != null) {
                List<IApplicationException> applicationExceptionList = assemblyDescriptor.getApplicationExceptionList();
                if (applicationExceptionList != null) {
                    for (IApplicationException applicationException : applicationExceptionList) {
                        addClassToSetIfNotNull(applicationException.getClassName(), classNameSet);
                    }
                }
            }

        }

    }

    /**
     * Create ejb jar metadata.
     * @param archive the ejbjar archive
     * @param ejbJarExternLibraries extern libraries
     * @return the metadata
     */
    @Override
    public IEjbJarMetadata createArchiveMetadata(final IArchive archive,
            final List<? extends IClassesLocator> ejbJarExternLibraries)
            throws EJBJARMetadataException {

        // for time debugging
        long tAnalyzeStart = System.currentTimeMillis();


        // Get EJB3 DD.
        EJB3 ejb3;
        try {
            ejb3 = EJB3DeploymentDesc.getEjb3(archive);
        } catch (ParsingException e) {
            throw new EJBJARMetadataException(e);
        }

        // Get Webservices DD.
        IWebservices webservices = null;
        try {
            webservices = Webservices12DeploymentDesc.getWebservices12(archive,
                    Webservices12DeploymentDesc.Type.EJBJAR);
        } catch (ParsingException e) {
            throw new EJBJARMetadataException(e);
        }

        EjbJarSessionConfigurator ejbjarSessionConfigurator = new EjbJarSessionConfigurator(archive);
        Set<String> classNameSet = new HashSet<String>();

        beforeScan(ejbjarSessionConfigurator.getEjbJarMetadata());

        // if metadata is complete then filter class to scan
        if (ejb3 != null && ejb3.isMetadataComplete()) {
            // disable annotation parsing
            ejbjarSessionConfigurator.setAnnotationParsingDesactived(true);

        } else {
            // enable annotation parsing because metadata descriptor is not
            // complete
            ejbjarSessionConfigurator.setAnnotationParsingDesactived(false);

            Iterator<String> entryIterator;
            try {
                entryIterator = archive.getEntries();
            } catch (ArchiveException e) {
                throw new EJBJARMetadataException(e);
            }
            while (entryIterator.hasNext()) {
                String entry = entryIterator.next();
                if (entry.endsWith(CLASS_SUFFIX)) {
                    classNameSet.add(entry.substring(0, entry.length() - CLASS_SUFFIX.length()));
                }
            }
        }

        // get class to scan from deployment descriptor
        completeClassList(ejb3, classNameSet);

        // do annotation scan
        try {
            this.scanner.scan(classNameSet, new ArchiveClassesLocator(archive), ejbJarExternLibraries, ejbjarSessionConfigurator);
        } catch (ScanException e) {
            throw new EJBJARMetadataException("Unable to scan", e);
        }

        // Prepare merge
        final IEjbJarMetadata ejbJarMetadata = ejbjarSessionConfigurator.getEjbJarMetadata();

        // prepare the metadata for merge
        ejbJarMetadata.setEjb3(ejb3);

        // Merge with XML descriptor
        IWebservicesView wsView = ejbJarMetadata.as(IWebservicesView.class);
        if (wsView != null) {
            wsView.setWebservices12(webservices);
        }
        // Call after scan
        afterScan(ejbJarMetadata);

        // Merge DD with annotations metadata
        EjbJarMetadataMerge.merge(ejbJarMetadata);

        if (wsView != null) {
            WebservicesMetadataMerger.merge(wsView, new IClassMetadataFinder() {

                @Override
                public IClassMetadata findClassMetadata(String link) {
                    return ejbJarMetadata.getEjbJarClassMetadataForEjbName(link);
                }
            });
        }
        afterMerge(ejbJarMetadata);

        // time if debugging
        if (LOGGER.isDebugEnabled()) {
            long tAnalyzeStartEnd = System.currentTimeMillis();
            LOGGER.debug("Analyze of file {0} took {1} ms.", archive.getName(), (tAnalyzeStartEnd - tAnalyzeStart));
        }

        return ejbJarMetadata;
    }

    protected void addClassToSetIfNotNull(final String className, final Set<String> stringSet) {
        if (className != null) {
            stringSet.add(className.replace('.', '/'));
        }
    }

    protected void afterMerge(IEjbJarMetadata ejbJarMetadata) {

    }

    protected void afterScan(IEjbJarMetadata ejbJarMetadata) {

    }


    protected void beforeScan(IEjbJarMetadata ejbJarMetadata) {

    }


}
