/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.ejbjar.impl;

import org.ow2.util.ee.metadata.ejbjar.api.IJClassInterceptor;
import org.ow2.util.scan.api.metadata.structures.IMethod;
import org.ow2.util.scan.impl.metadata.JMethod;

/**
 * This class defines a Bean interceptor with the name of the class and the
 * method which is the interceptor.
 * @author Florent Benoit
 */
public class JClassInterceptor implements IJClassInterceptor {

    /**
     * String name of the class (internal name) where is the interceptor.
     */
    private String className = null;

    /**
     * Method with &#64;{@link javax.interceptor.AroundInvoke} annotation.
     */
    private IMethod jMethod = null;

    /**
     * NO ID.
     */
    private static final int NO_ID = -1;

    /**
     * Id of this interceptor (0 = no id).
     */
    private int id = NO_ID;

    /**
     * Constructor.
     * @param className String name of the class (internal name).
     * @param jMethod the method with aroundInvoke annotation.
     * @param id the id of this interceptor
     */
    public JClassInterceptor(final String className, final IMethod jMethod, final int id) {
        this.className = className;
        this.jMethod = jMethod;
        this.id = id;
    }

    /**
     * Constructor.
     * @param className String name of the class (internal name).
     * @param jMethod the method with aroundInvoke annotation.
     */
    public JClassInterceptor(final String className, final JMethod jMethod) {
       this(className, jMethod, NO_ID);
    }

    /**
     * @return class (internal name) where is the interceptor.
     */
    @Override
    public String getClassName() {
        return this.className;
    }

    /**
     * @return Method with &#64;{@link javax.interceptor.AroundInvoke} annotation
     */
    @Override
    public IMethod getJMethod() {
        return this.jMethod;
    }

    /**
     * Equals method.
     * @param another object to compare.
     * @return true if the objects are the same.
     */
    @Override
    public boolean equals(final Object another) {
        if (!(another instanceof JClassInterceptor)) {
            return false;
        }
        JClassInterceptor anotherItcp = (JClassInterceptor) another;
        if (this.id == NO_ID) {
            return (anotherItcp.className.equals(this.className) && anotherItcp.jMethod.equals(this.jMethod));
        }
        return (anotherItcp.className.equals(this.className)
                && anotherItcp.jMethod.equals(this.jMethod) && anotherItcp.id == this.id);
    }

    /**
     * @return hashCode of the object.
     */
    @Override
    public int hashCode() {
        return this.className.hashCode() + this.jMethod.hashCode();
    }

    /**
     * @return a string representing the interceptor
     */
    @Override
    public String toString() {
        return "Interceptor on " + className + "/" + jMethod.getName();
    }

}
