/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor;

import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.ElementType.TYPE;

import java.util.concurrent.TimeUnit;

import org.ow2.util.ee.metadata.ejbjar.api.struct.IJEjbAccessTimeout;
import org.ow2.util.ee.metadata.ejbjar.api.view.IEJBView;
import org.ow2.util.ee.metadata.ejbjar.impl.struct.JEJBAccessTimeout;
import org.ow2.util.scan.api.IAnnotationVisitorContext;
import org.ow2.util.scan.api.IType;
import org.ow2.util.scan.api.annotation.VisitorTarget;
import org.ow2.util.scan.api.annotation.VisitorType;
import org.ow2.util.scan.api.visitor.DefaultAnnotationVisitor;

/**
 * This class manages the handling of &#64;{@link javax.ejb.AccessTimeout}
 * annotation.
 * @author Florent Benoit
 */
@VisitorTarget({TYPE, METHOD})
@VisitorType("javax.ejb.AccessTimeout")
public class JavaxEjbAccessTimeoutVisitor extends DefaultAnnotationVisitor<IJEjbAccessTimeout> {

    /**
     * value attribute of the annotation.
     */
    private static final String VALUE = "value";


    /**
     * unit attribute of the annotation.
     */
    private static final String UNIT = "unit";

    /**
     * Visits a primitive value of the annotation.<br>
     * @param name the value name.
     * @param value the actual value, whose type must be {@link Byte},
     *        {@link Boolean}, {@link Character}, {@link Short},
     *        {@link Integer}, {@link Long}, {@link Float}, {@link Double},
     *        {@link String} or {@link IType}.
     */
    @Override
    public void visit(final String name, final Object value, IAnnotationVisitorContext<IJEjbAccessTimeout> annotationVisitorContext) {
        IJEjbAccessTimeout jEjbAccessTimeout = annotationVisitorContext.getLocal();
        if (VALUE.equals(name)) {
            jEjbAccessTimeout.setValue((Long) value);
        }
    }

    /**
     * Visits an enumeration value of the annotation.
     * @param name the value name.
     * @param desc the class descriptor of the enumeration class.
     * @param value the actual enumeration value.
     */
    @Override
    public void visitEnum(final String name, final String desc, final String value, IAnnotationVisitorContext<IJEjbAccessTimeout> annotationVisitorContext) {
        IJEjbAccessTimeout jEjbAccessTimeout = annotationVisitorContext.getLocal();
        if (UNIT.equals(name)) {
            jEjbAccessTimeout.setUnit(TimeUnit.valueOf(value.toUpperCase()));
        }
    }

    /**
     * Visits the end of the annotation.<br>
     * Creates the object and store it.
     */
    @Override
    public void visitEnd(IAnnotationVisitorContext<IJEjbAccessTimeout> annotationVisitorContext) {
        // set flag on field
        annotationVisitorContext.getView(IEJBView.class).setJavaxEjbAccessTimeout(annotationVisitorContext.getLocal());
    }

    @Override
    public IJEjbAccessTimeout buildInstance() {
        return new JEJBAccessTimeout();
    }

}
