/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.clazz;

import static java.lang.annotation.ElementType.TYPE;
import static org.ow2.util.ee.metadata.ejbjar.api.ClassType.MDB;

import org.ow2.util.ee.metadata.ejbjar.api.struct.IJMessageDriven;
import org.ow2.util.ee.metadata.ejbjar.api.view.IEJBClassView;
import org.ow2.util.ee.metadata.ejbjar.impl.struct.JActivationConfigProperty;
import org.ow2.util.ee.metadata.ejbjar.impl.struct.JMessageDriven;
import org.ow2.util.scan.api.IAnnotationVisitor;
import org.ow2.util.scan.api.IAnnotationVisitorContext;
import org.ow2.util.scan.api.IType;
import org.ow2.util.scan.api.annotation.VisitorTarget;
import org.ow2.util.scan.api.annotation.VisitorType;
import org.ow2.util.scan.api.visitor.DefaultAnnotationVisitor;

/**
 * This class manages the handling of &#64;{@link javax.ejb.MessageDriven}
 * annotation.
 * @author Florent Benoit
 */
@VisitorTarget(TYPE)
@VisitorType("javax.ejb.MessageDriven")
public class JavaxEjbMessageDrivenVisitor extends DefaultAnnotationVisitor<IJMessageDriven> {

    /**
     * messageListenerInterface attribute of the annotation.
     */
    private static final String MESSAGE_LISTENER_INTERFACE = "messageListenerInterface";

    /**
     * activationConfig attribute of the annotation.
     */
    private static final String ACTIVATION_CONFIG = "activationConfig";

    /**
     * Message listener Interface.
     */
    private String messageListenerInterface = null;


    /**
     * Visits a primitive value of the annotation.<br>
     * @param name the value name.
     * @param value the actual value, whose type must be {@link Byte},
     *        {@link Boolean}, {@link Character}, {@link Short},
     *        {@link Integer}, {@link Long}, {@link Float}, {@link Double},
     *        {@link String} or {@link IType}.
     */
    @Override
    public void visit(final String name, final Object value, IAnnotationVisitorContext<IJMessageDriven> annotationVisitorContext) {
        if (name.equals(MESSAGE_LISTENER_INTERFACE)) {
            this.messageListenerInterface = ((IType) value).getInternalName();
        }
    }

    /**
     * Visits an array value of the annotation.
     * @param name the value name.
     * @return a non null visitor to visit the actual array value elements. The
     *         'name' parameters passed to the methods of this visitor are
     *         ignored. <i>All the array values must be visited before calling
     *         other methods on this annotation visitor</i>.
     */
    @Override
    public IAnnotationVisitor<IJMessageDriven> visitArray(final String name, IAnnotationVisitorContext<IJMessageDriven> annotationVisitorContext) {
        if (name.equals(ACTIVATION_CONFIG)) {
            return new ActivationConfigPropertyVisitor();
        }
        return this;
    }

    /**
     * Visits the end of the annotation. Creates the object and store it
     */
    @Override
    public void visitEnd(IAnnotationVisitorContext<IJMessageDriven> annotationVisitorContext) {
        IEJBClassView classView = annotationVisitorContext.getView(IEJBClassView.class);

        // message listener interface
        annotationVisitorContext.getLocal().setMessageListenerInterface(this.messageListenerInterface);
        classView.setJMessageDriven(annotationVisitorContext.getLocal());
        classView.setClassType(MDB);

    }


    /**
     * Classes manages the parsing of activationConfig[] array of &#64;{@link javax.ejb.MessageDriven}
     * annotation.
     * @author Florent Benoit
     */
    class ActivationConfigPropertyVisitor extends DefaultAnnotationVisitor<IJMessageDriven> {

        /**
         * Attribute for property name.
         */
        private static final String PROPERTY_NAME = "propertyName";

        /**
         * Attribute for property value.
         */
        private static final String PROPERTY_VALUE = "propertyValue";

        /**
         * Property name.
         */
        private String propertyName = null;

        /**
         * Property value.
         */
        private String propertyValue = null;

        /**
         * Visits a primitive value of the annotation.
         * @param name the value name.
         * @param value the actual value, whose type must be {@link Byte},
         *        {@link Boolean}, {@link Character}, {@link Short},
         *        {@link Integer}, {@link Long}, {@link Float},
         *        {@link Double}, {@link String} or {@link IType}.
         */
        @Override
        public void visit(final String name, final Object value, IAnnotationVisitorContext<IJMessageDriven> annotationVisitorContext) {
            if (name.equals(PROPERTY_NAME)) {
                this.propertyName = (String) value;
            } else if (name.equals(PROPERTY_VALUE)) {
                this.propertyValue = (String) value;
            }
        }

        /**
         * Visits the end of the annotation. Creates the object and store it
         */
        @Override
        public void visitEnd(IAnnotationVisitorContext<IJMessageDriven> annotationVisitorContext) {
            // Add an activation property
            JActivationConfigProperty jActivationConfigProperty = new JActivationConfigProperty(this.propertyName,
                    this.propertyValue);
            annotationVisitorContext.getLocal().addActivationConfigProperty(jActivationConfigProperty);

        }

        @Override
        public IJMessageDriven buildInstance() {
            return new JMessageDriven();
        }

    }

    @Override
    public IJMessageDriven buildInstance() {
        return new JMessageDriven();
    }

}
