/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.ejbjar.impl.struct;

import org.ow2.util.ee.metadata.ejbjar.api.struct.IApplicationException;

/**
 * Acts as an implementation of &#64;{javax.ejb.ApplicationException}
 * annotation.
 * @author Florent Benoit
 */
public class JApplicationException implements IApplicationException {

    /**
     * Serial Version UID.
     */
    private static final long serialVersionUID = -442184337225643333L;

    /**
     * Name of this element (Deployment desc).
     */
    public static final String NAME = "application-exception";

    /**
     * Container rollback the transaction ? if an exception is thrown.
     */
    private boolean rollback = false;

    /**
     * Inherited ?
     */
    private boolean inherited = true;

    /**
     * Name of the class that is flagged as ApplicationException.
     */
    private String className = null;

    /**
     * Build an object which represents &#64;{@link javax.ejb.ApplicationException}
     * annotation with a given boolean.<br>
     */
    public JApplicationException() {
        this(false, true);
    }


    /**
     * Build an object which represents &#64;{@link javax.ejb.ApplicationException}
     * annotation with a given boolean.<br>
     * @param rollback true/false (false is a default value)
     * @param inherited true/false (true is a default value)
     */
    public JApplicationException(final boolean rollback, final boolean inherited) {
        this(null, rollback, inherited);
    }

    /**
     * Build an object which represents &#64;{@link javax.ejb.ApplicationException}
     * annotation with a given boolean.<br>
     * @param className the name of the class which is the application
     *        exception.
     * @param rollback true/false (false is a default value)
     * @param inherited true/false (true is a default value)
     */
    public JApplicationException(final String className, final boolean rollback, final boolean inherited) {
        this.className = className;
        this.rollback = rollback;
        this.inherited = inherited;
    }

    /**
     * @return the retainIfException value (true/false)
     */
    @Override
    public boolean rollback() {
        return rollback;
    }

    /**
     * @return the inherited value (true/false)
     */
    @Override
    public boolean inherited() {
        return inherited;
    }

    /**
     * @return the class name.
     */
    @Override
    public String getClassName() {
        return className;
    }

    /**
     * @return string representation
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        // classname
        sb.append(this.getClass().getName().substring(this.getClass().getPackage().getName().length() + 1));
        // rollback
        sb.append("[rollback=");
        sb.append(rollback);
        sb.append(", inherited=");
        sb.append(inherited);
        if (className != null) {
            sb.append(", className=");
            sb.append(className);
        }
        sb.append("]");
        return sb.toString();
    }


    @Override
    public void setRollback(boolean rollback) {
        this.rollback = rollback;
    }


    @Override
    public void setInherited(boolean inherited) {
        this.inherited = inherited;
    }
}
