/**
 * Copyright 2013 Peergreen S.A.S.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ow2.util.ee.metadata.ejbjar.impl.view;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import org.ow2.util.ee.metadata.common.api.struct.IEnvEntry;
import org.ow2.util.ee.metadata.common.api.struct.IJavaxJwsWebMethod;
import org.ow2.util.ee.metadata.common.api.struct.ITransactionAttributeType;
import org.ow2.util.ee.metadata.common.api.struct.ITransactionManagementType;
import org.ow2.util.ee.metadata.common.api.xml.struct.ISecurityRoleRef;
import org.ow2.util.ee.metadata.common.impl.view.CommonView;
import org.ow2.util.ee.metadata.ejbjar.api.ClassType;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IJClassInterceptor;
import org.ow2.util.ee.metadata.ejbjar.api.InterceptorType;
import org.ow2.util.ee.metadata.ejbjar.api.TransactionAttributeLevel;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IApplicationException;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IConcurrencyManagementType;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IDependsOn;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJCommonBean;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJEjbAccessTimeout;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJEjbRemove;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJEjbSchedule;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJEjbStatefulTimeout;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJLocal;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJMessageDriven;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJRemote;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJSingleton;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJStateful;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJStateless;
import org.ow2.util.ee.metadata.ejbjar.api.struct.ILockType;
import org.ow2.util.ee.metadata.ejbjar.api.view.IEJBClassView;
import org.ow2.util.ee.metadata.ejbjar.api.view.IEJBMethodView;
import org.ow2.util.ee.metadata.ejbjar.api.view.IEJBView;
import org.ow2.util.scan.api.metadata.IClassMetadata;
import org.ow2.util.scan.api.metadata.IMetadata;
import org.ow2.util.scan.api.metadata.structures.IClass;
import org.ow2.util.scan.impl.metadata.JClass;

public class EJBView implements IEJBClassView, IEJBMethodView, IEJBView {

    private final IMetadata metadata;

    private final CommonView commonView;

    public EJBView(IMetadata metadata) {
        this.metadata = metadata;
        this.commonView = metadata.as(CommonView.class);
    }


    /**
     * @return the name of the bean on which we are linked.
     * This is used for cloned classmetadata that are for the classes.
     */
    @Override
    public String getLinkedBean() {
        return metadata.getProperty("ejb.metadata.linkedBean");
    }

    /**
     * This class is linked to the given bean.
     * @param linkedBeanName the bean name
     */
    @Override
    public void setLinkedBean(final String linkedBeanName) {
        this.metadata.setProperty("ejb.metadata.linkedBean", linkedBeanName);
    }


    /**
     * Sets the local interfaces of this class.
     * @param jLocal list of interfaces.
     */
    @Override
    public void setLocalInterfaces(final IJLocal jLocal) {
        this.metadata.set(IJLocal.class, jLocal);
    }

    /**
     * Sets the remote interfaces of this class.
     * @param jRemote list of interfaces.
     */
    @Override
    public void setRemoteInterfaces(final IJRemote jRemote) {
        this.metadata.set(IJRemote.class, jRemote);
    }

    /**
     * @return the local interfaces of this class.
     */
    @Override
    public IJLocal getLocalInterfaces() {
        return this.metadata.get(IJLocal.class);
    }

    /**
     * @return the remote interfaces of this class.
     */
    @Override
    public IJRemote getRemoteInterfaces() {
        return this.metadata.get(IJRemote.class);
    }


    protected ClassType getClassType() {
        return this.metadata.get(ClassType.class);
    }

    /**
     * @return true if the class is a stateless class
     */
    @Override
    public boolean isStateless() {
        return (getClassType() != null && getClassType() == ClassType.STATELESS);
    }

    /**
     * @return true if the class is a singleton class
     */
    @Override
    public boolean isSingleton() {
        return (getClassType() != null && getClassType() == ClassType.SINGLETON);
    }

    /**
     * @return true if the class is a stateful class
     */
    @Override
    public boolean isStateful() {
        return (getClassType() != null && getClassType() == ClassType.STATEFUL);
    }

    /**
     * @return true if the class is a session bean class
     */
    @Override
    public boolean isSession() {
        return (getClassType() != null
                && (getClassType() == ClassType.STATELESS
                        || getClassType() == ClassType.STATEFUL
                        || getClassType() == ClassType.SINGLETON));
    }

    /**
     * @return true if the class is an MDB class
     */
    @Override
    public boolean isMdb() {
        return (getClassType() != null && getClassType() == ClassType.MDB);
    }

    /**
     * Sets the type of this class.
     * @param cType a type from enum class ClassType.
     * @see org.ow2.easybeans.deployment.annotations.ClassType
     */
    @Override
    public void setClassType(final ClassType cType) {
        this.metadata.set(ClassType.class,  cType);
    }

    /**
     * @return Message driven attribute.
     */
    @Override
    public IJMessageDriven getJMessageDriven() {
        return this.metadata.get(IJMessageDriven.class);
    }

    /**
     * Sets the message driven bean object.
     * @param messageDriven attributes of message driven bean.
     */
    @Override
    public void setJMessageDriven(final IJMessageDriven messageDriven) {
        this.metadata.set(IJMessageDriven.class, messageDriven);
    }

    /**
     * @return the &#64;{@link javax.ejb.RemoteHome} class name.
     */
    @Override
    public String getRemoteHome() {
        return this.metadata.getProperty("javax.ejb.RemoteHome");
    }

    /**
     * Sets the &#64;{@link javax.ejb.RemoteHome} class name.
     * @param remoteHome the class name.
     */
    @Override
    public void setRemoteHome(final String remoteHome) {
        this.metadata.setProperty("javax.ejb.RemoteHome", remoteHome);
    }

    /**
     * @return the &#64;{@link javax.ejb.LocalHome} class name.
     */
    @Override
    public String getLocalHome() {
        return this.metadata.getProperty("javax.ejb.LocalHome");
    }

    /**
     * Sets the &#64;{@link javax.ejb.LocalHome} class name.
     * @param localHome the class name.
     */
    @Override
    public void setLocalHome(final String localHome) {
        this.metadata.setProperty("javax.ejb.LocalHome", localHome);
    }

    /**
     * @return transaction management type from
     * @see TransactionManagementType.
     */
    @Override
    public ITransactionManagementType getTransactionManagementType() {
        return this.metadata.get(ITransactionManagementType.class);
    }

    /**
     * Sets transaction management type.
     * @see javax.ejb.TransactionManagementType
     * @param transactionManagementType value. (BEAN, CONTAINER)
     */
    @Override
    public void setTransactionManagementType(final ITransactionManagementType transactionManagementType) {
        this.metadata.set(ITransactionManagementType.class,  transactionManagementType);
    }

    /**
     * @return transaction Attribute type.
     * @see ITransactionAttributeType
     */
    @Override
    public ITransactionAttributeType getTransactionAttributeType() {
        return this.metadata.get(ITransactionAttributeType.class);
    }

    /**
     * Set Transaction Attribute Type.
     * @see ITransactionAttributeType
     * @param transactionAttributeType the type of transaction.
     */
    @Override
    public void setTransactionAttributeType(final ITransactionAttributeType transactionAttributeType) {
        this.metadata.set(ITransactionAttributeType.class,  transactionAttributeType);
    }


    /**
     * @return object representing list of &#64;
     *         {@link javax.interceptor.DependsOn}.
     */
    @Override
    public IDependsOn getDependsOn() {
        return this.metadata.get(IDependsOn.class);
    }

    /**
     * Sets the object representing the &#64;
     * {@link javax.ejb.DependsOn} annotation.
     * @param dependsOn list of beans
     */
    @Override
    public void setDependsOn(final IDependsOn dependsOn) {
        this.metadata.set(IDependsOn.class, dependsOn);
    }


    /**
     * @return the &#64;{@link javax.ejb.ApplicationException} annotation.
     */
    @Override
    public IApplicationException getApplicationException() {
        return this.metadata.get(IApplicationException.class);
    }

    /**
     * Sets the object representing the &#64;
     * {@link javax.ejb.ApplicationException} annotation.
     * @param applicationException object representation
     */
    @Override
    public void setApplicationException(final IApplicationException applicationException) {
        this.metadata.set(IApplicationException.class, applicationException);
    }

    /**
     * @return true if the classs is a Bean
     */
    @Override
    public boolean isBean() {
        return isSession() || isMdb() || commonView.isManagedBean();
    }

    /**
     * @return Map&lt;interceptor type &lt;--&gt; List of methods/class
     *         corresponding to the interceptor&gt; (interceptor classes) of
     *         user interceptors that enhancer will use.
     */
    @Override
    public Map<InterceptorType, List<? extends IJClassInterceptor>> getExternalUserEasyBeansInterceptors() {
        return this.metadata.getProperty("ejb.metadata.externalUserInterceptors");
    }

    /**
     * Sets the list of user interceptors that enhancers will use.<br>
     * These interceptors are defined outside the bean class (interceptor
     * classes).
     * @param externalUserInterceptors list of interceptors that enhancer will
     *        use.
     */
    @Override
    public void setExternalUserInterceptors(
            final Map<InterceptorType, List<? extends IJClassInterceptor>> externalUserInterceptors) {
        this.metadata.setProperty("ejb.metadata.externalUserInterceptors", externalUserInterceptors);
    }

    /**
     * @return Map&lt;interceptor type &lt;--&gt; List of methods/class
     *         corresponding to the interceptor&gt; (bean classes) of user
     *         interceptors that enhancer will use.
     */
    @Override
    public Map<InterceptorType, List<? extends IJClassInterceptor>> getInternalUserEasyBeansInterceptors() {
        return this.metadata.getProperty("ejb.metadata.internalUserInterceptors");
    }

    /**
     * Sets the list of user interceptors that enhancers will use.<br>
     * These interceptors are defined in bean class.
     * @param internalUserInterceptors list of interceptors that enhancer will
     *        use.
     */
    @Override
    public void setInternalUserInterceptors(
            final Map<InterceptorType, List<? extends IJClassInterceptor>> internalUserInterceptors) {
        this.metadata.setProperty("ejb.metadata.internalUserInterceptors", internalUserInterceptors);
    }

    /**
     * @return list of global interceptors that enhancer will use. (ie : ENC)
     */
    @Override
    public List<? extends IJClassInterceptor> getGlobalEasyBeansInterceptors() {
        return this.metadata.getProperty("ejb.metadata.globalEasyBeansInterceptors");
    }

    /**
     * Sets the list of global interceptors that enhancers will use.
     * @param globalEasyBeansInterceptors list of interceptors that enhancer
     *        will use.
     */
    @Override
    public void setGlobalEasyBeansInterceptors(final List<? extends IJClassInterceptor> globalEasyBeansInterceptors) {
        this.metadata.setProperty("ejb.metadata.globalEasyBeansInterceptors", globalEasyBeansInterceptors);
    }

    /**
     * @return the attributes for a Stateless/Stateful/MDB
     */
    @Override
    public IJCommonBean getJCommonBean() {
        return this.metadata.get(IJCommonBean.class);
    }

    /**
     * Sets the attributes for a Stateless/Stateful/MDB.
     * @param commonBean the attributes
     */
    @Override
    public void setJCommonBean(final IJCommonBean commonBean) {
        // ensure that there is a bean name
        String ejbName = commonBean.getName();
        if (ejbName == null || "".equals(ejbName)) {
            // if not set, it has to be the unqualified name of the bean class.
            // (simplified spec : 10.1.1)
            if (metadata instanceof IClassMetadata) {
                IClassMetadata classMetadata = (IClassMetadata) metadata;
                String className = classMetadata.getInternalClassName();
                commonBean.setName(className.substring(className.lastIndexOf("/") + 1));
            } else {
                throw new IllegalArgumentException("Invalid metadata : " + this.metadata);
            }
        }
        this.metadata.set(IJCommonBean.class,  commonBean);
    }

    /**
     * @return the attributes for a Stateful
     */
    @Override
    public IJStateful getJStateful() {
        return this.metadata.get(IJStateful.class);
    }

    /**
     * Sets the attributes for a Stateful.
     * @param jStateful the attributes
     */
    @Override
    public void setJStateful(final IJStateful jStateful) {
        this.metadata.set(IJStateful.class, jStateful);
    }

    /**
     * @return the attributes for a Singleton
     */
    @Override
    public IJSingleton getJSingleton() {
        return this.metadata.get(IJSingleton.class);
    }

    /**
     * Sets the attributes for a Singleton.
     * @param jSingleton the attributes
     */
    @Override
    public void setJSingleton(final IJSingleton jSingleton) {
        this.metadata.set(IJSingleton.class, jSingleton);
    }


    /**
     * @return the attributes for a Stateless
     */
    @Override
    public IJStateless getJStateless() {
        return this.metadata.get(IJStateless.class);

    }

    /**
     * Sets the attributes for a Stateless.
     * @param jStateless the attributes
     */
    @Override
    public void setJStateless(final IJStateless jStateless) {
        this.metadata.set(IJStateless.class, jStateless);
    }

    /**
     * @return true if the class has been modified.
     */
    @Override
    public boolean wasModified() {
        return this.metadata.getBoolean("metadata.ejbjar.wasModified");
    }

    /**
     * Defines that this class has been modified.
     */
    @Override
    public void setModified() {
        this.metadata.setProperty("metadata.ejbjar.wasModified", true);
    }

    /**
     * Sets the list of roles declared on this class.
     * @param declareRoles the list of roles.
     */
    @Override
    public void setDeclareRoles(final List<String> declareRoles) {
        this.metadata.setProperty("metadata.ejbjar.declareRoles", declareRoles);
    }

    /**
     * @return the list of roles declared on this class.
     */
    @Override
    public List<String> getDeclareRoles() {
        return this.metadata.getProperty("metadata.ejbjar.declareRoles");
    }

    /**
     * Set the list of roles allowed on this class/method.
     * @param rolesAllowed the list of roles.
     */
    @Override
    public void setRolesAllowed(final List<String> rolesAllowed) {
        this.metadata.setProperty("metadata.ejbjar.rolesAllowed", rolesAllowed);
    }

    /**
     * @return the list of roles allowed on this class/method.
     */
    @Override
    public List<String> getRolesAllowed() {
        return this.metadata.getProperty("metadata.ejbjar.rolesAllowed");
    }



    /**
     * This class needs eager initialization.
     * @param startup the boolean value.
     */
    @Override
    public void setStartup(final boolean startup) {
        this.metadata.setProperty("javax.ejb.Startup", startup);
    }

    /**
     * @return true if this class(singleton) needs eager initialization.
     */
    @Override
    public boolean isStartup() {
        return this.metadata.getBoolean("javax.ejb.Startup");
    }



    /**
     * @return true if the interceptors have been ordered for this class.
     */
    @Override
    public boolean isOrderedInterceptors() {
        return this.metadata.getBoolean("ejb.metadata.orderedInterceptors");
    }

    /**
     * Sets flag to true if interceptors are ordered.
     * @param orderedInterceptors boolean with true/false.
     */
    @Override
    public void setOrderedInterceptors(final boolean orderedInterceptors) {
        this.metadata.setProperty("ejb.metadata.orderedInterceptors", orderedInterceptors);
    }

    /**
     * Set the value of the remote Interface property.
     * @param remoteInterface the remote Interface property.
     */
    @Override
    public void setRemoteInterface(final String remoteInterface) {
        this.metadata.setProperty("ejb.metadata.remoteInterface", remoteInterface);
    }

    /**
     * @return the name of the remote interface.
     */
    @Override
    public String getRemoteInterface() {
        return this.metadata.getProperty("ejb.metadata.remoteInterface");
    }

    /**
     * construct a new JClass.
     * @param array interface list
     */
    @Override
    public void setInterfaces(final String[] array) {
        if (metadata instanceof IClassMetadata) {
            IClassMetadata classMetadata = (IClassMetadata) metadata;
            IClass jClass = classMetadata.getJClass();
            classMetadata.setJClass(new JClass(jClass.getVersion(), jClass.getAccess(), jClass.getName(), jClass.getSignature(), jClass.getSuperName(), array));
        } else {
            throw new IllegalArgumentException("Cannot be called on other metadata than classmetadata '" + metadata + "'");
        }
    }


    /**
     * {@inheritDoc}
     */
    public Collection<? extends IEnvEntry> getEnvEntryCollection() {
        return metadata.getProperty("metadata.ejbjar.envEntryCollection");
    }

    /**
     * @param envEntryCollection a collection of env-entry
     */
    public void setEnvEntryCollection(final Collection<? extends IEnvEntry> envEntryCollection) {
        metadata.setProperty("metadata.ejbjar.envEntryCollection", envEntryCollection);
    }


    /**
     * Get a classmetadata linked from this one.
     * @param className the name of the class that is wanted
     * @return the class metadata
     */
    @Override
    public IClassMetadata getLinkedClassMetadata(final String className) {
        // Check if this class is part of a bean or not
        // If it is linked to a bean, a cloned copy will be obtained, else the scanned class metadata will be retrieved
        if (getLinkedBean() != null) {
            return getEjbJarDeployableMetadata().getBeanEjbJarClassMetadataForEjbName(getLinkedBean(), className);
        }
        return getEjbJarDeployableMetadata().getScannedClassMetadata(className);
    }

    /**
     * Gets the list of &lt;security-role-ref&gt; elements.
     * @return list of &lt;security-role-ref&gt; elements.
     */
    @Override
    public List<ISecurityRoleRef> getSecurityRoleRefList() {
        return metadata.getProperty("metadata.ejbjar.securityRoleRefList");
    }

    /**
     * Sets the list of &lt;security-role-ref&gt; element.
     * @param securityRoleRefList the list of &lt;security-role-ref&gt; element
     */
    @Override
    public void setSecurityRoleRefList(final List<ISecurityRoleRef> securityRoleRefList) {
        this.metadata.setProperty("metadata.ejbjar.securityRoleRefList", securityRoleRefList);
    }

    /**
     * @return lock type
     * @see javax.ejb.LockType
     */
    @Override
    public ILockType getLockType() {
        return metadata.get(ILockType.class);
    }

    /**
     * Set Lock Type.
     * @see javax.ejb.LockType
     * @param lockType the type of the lock.
     */
    @Override
    public void setLockType(final ILockType lockType) {
        this.metadata.set(ILockType.class, lockType);
    }

    /**
     * @return transaction management type from
     * @see javax.ejb.ConcurrencyManagementType.
     */
    @Override
    public IConcurrencyManagementType getConcurrencyManagementType() {
        return metadata.get(IConcurrencyManagementType.class);
    }

    /**
     * Sets concurrency management type.
     * @see javax.ejb.ConcurrencyManagementType
     * @param concurrencyManagementType value.
     *      BEAN,CONCURRENCY_NOT_ALLOWED, CONTAINER
     */
    @Override
    public void setConcurrencyManagementType(final IConcurrencyManagementType concurrencyManagementType) {
        this.metadata.set(IConcurrencyManagementType.class, concurrencyManagementType);
    }

    /**
     * @return true if the class is a local bean.
     */
    @Override
    public boolean isLocalBean() {
        return metadata.getBoolean("javax.ejb.LocalBean");
    }

    /**
     * Defines if the bean is a local bean or not.
     * @param isLocalBean true if the bean is a local bean (and expose a no-interface local view)
     */
    @Override
    public void setLocalBean(boolean isLocalBean) {
        metadata.setProperty("javax.ejb.LocalBean", isLocalBean);
    }


    /**
     * @return the access timeout on this class/method.
     */
    @Override
    public IJEjbAccessTimeout getJavaxEjbAccessTimeout() {
        return metadata.get(IJEjbAccessTimeout.class);
    }

    /**
     * Sets the access timeout on this class/method.
     * @param jEjbAccessTimeout information on access timeout.
     */
    @Override
    public void setJavaxEjbAccessTimeout(IJEjbAccessTimeout jEjbAccessTimeout) {
        this.metadata.set(IJEjbAccessTimeout.class, jEjbAccessTimeout);
    }

    /**
     * This bean is linked to a session synchronization and then should report TX lifecycle to the callbacks.
     */
    @Override
    public void setSessionSynchronization() {
        this.metadata.setProperty("metadata.ejbjar.hasSessionSynchronization", true);
    }

    /**
     * @return true if the bean is part of a session synchronization
     */
    @Override
    public boolean hasSessionSynchronization() {
        return this.metadata.getBoolean("metadata.ejbjar.hasSessionSynchronization");
    }

    @Override
    public IJEjbStatefulTimeout getJavaxEjbStatefulTimeout() {
        return metadata.get(IJEjbStatefulTimeout.class);
    }

    @Override
    public void setJavaxEjbStatefulTimeout(IJEjbStatefulTimeout jEjbStatefulTimeout) {
        this.metadata.set(IJEjbStatefulTimeout.class, jEjbStatefulTimeout);
    }

    /**
     * Sets Transaction Attribute Level.
     * @param transactionAttributeLevel the level of transaction.
     */
    @Override
    public void setTransactionAttributeLevel(final TransactionAttributeLevel transactionAttributeLevel) {
        this.metadata.set(TransactionAttributeLevel.class, transactionAttributeLevel);
    }

    /**
     * @return transaction Attribute Level.
     */
    @Override
    public TransactionAttributeLevel getTransactionAttributeLevel() {
        return this.metadata.get(TransactionAttributeLevel.class);
    }

    /**
     * @return true if the method is a business method.
     */
    @Override
    public boolean isBusinessMethod() {
        return this.metadata.getBoolean("metada.ejbjar.businessMethod");
    }

    /**
     * This method is a business method.
     * @param flag true/false if method is a business method.
     */
    @Override
    public void setBusinessMethod(final boolean flag) {
        this.metadata.setProperty("metada.ejbjar.businessMethod", flag);
    }

    /**
     * @return true if the method is a lifecycle method.
     */
    @Override
    public boolean isLifeCycleMethod() {
        return  isPostActivate() || commonView.isPostConstruct() || commonView.isPreDestroy() || isPrePassivate();
    }

    /**
     * @return remove attributes  &#64;{@link javax.ejb.Remove} attributes
     */
    @Override
    public IJEjbRemove getJRemove() {
        return this.metadata.get(IJEjbRemove.class);
    }

    /**
     * Sets &#64;{@link javax.ejb.Remove} attribute.
     * @param remove contains the attribute with retainIfException.
     */
    @Override
    public void setRemove(final IJEjbRemove remove) {
        this.metadata.set(IJEjbRemove.class, remove);
    }

    /**
     * @return true if method has  &#64;{@link javax.ejb.PostActivate}
     */
    @Override
    public boolean isPostActivate() {
        return this.metadata.getBoolean("javax.ejb.PostActivate");
    }

    /**
     * Sets true if method has &#64;{@link javax.ejb.PostActivate}.
     * @param postActivate true/false.
     */
    @Override
    public void setPostActivate(final boolean postActivate) {
        this.metadata.setProperty("javax.ejb.PostActivate", postActivate);
    }

    /**
     * @return true if method has &#64;{@link javax.ejb.PrePassivate}.
     */

    @Override
    public boolean isPrePassivate() {
        return this.metadata.getBoolean("javax.ejb.PrePassivate");
    }

    /**
     * Sets true if method has &#64;{@link javax.ejb.PrePassivate}.
     * @param prePassivate true/false.
     */
    @Override
    public void setPrePassivate(final boolean prePassivate) {
        this.metadata.setProperty("javax.ejb.PrePassivate", prePassivate);
    }

    /**
     * @return true if method has &#64;{@link javax.ejb.Timeout}.
     */
    @Override
    public boolean isTimeout() {
        return this.metadata.getBoolean("javax.ejb.Timeout");
    }

    /**
     * Sets true if method has &#64;{@link javax.ejb.Timeout}.
     * @param timeout true/false.
     */
    @Override
    public void setTimeout(final boolean timeout) {
        this.metadata.setProperty("javax.ejb.Timeout",  timeout);
    }

    /**
     * @return true if method has &#64;{@link javax.interceptor.AroundInvoke}.
     */
    @Override
    public boolean isAroundInvoke() {
        return this.metadata.getBoolean("javax.interceptor.AroundInvoke");

    }

    /**
     * Sets true if method has &#64;{@link javax.interceptor.AroundInvoke}.
     * @param aroundInvoke true/false
     */
    @Override
    public void setAroundInvoke(final boolean aroundInvoke) {
        this.metadata.setProperty("javax.interceptor.AroundInvoke",  aroundInvoke);
    }

    /**
     * @return true if this method won't use user interceptors.
     */
    @Override
    public boolean isExcludedClassInterceptors() {
        return this.metadata.getBoolean("metadata.ejbjar.excludeClassInterceptors");
    }

    /**
     * Flag this method as a method which exclude user interceptors.
     * @param excludeClassInterceptors true if this method is a method which exclude user interceptors.
     */
    @Override
    public void setExcludeClassInterceptors(final boolean excludeClassInterceptors) {
        this.metadata.setProperty("metadata.ejbjar.excludeClassInterceptors", excludeClassInterceptors);
    }

    /**
     * @return Map&lt;interceptor type &lt;--&gt; List of methods/class corresponding to the interceptor&gt;
     * of user interceptors that enhancer will use.
     */
    @Override
    public Map<InterceptorType, List<? extends IJClassInterceptor>> getUserEasyBeansInterceptors() {
        return this.metadata.getProperty("metadata.ejbjar.userInterceptors");
    }

    /**
     * Sets the list of user interceptors that enhancers will use.<br>
     * These interceptors are defined in bean class.
     * @param userInterceptors list of interceptors that enhancer will use.
     */
    @Override
    public void setUserInterceptors(final Map<InterceptorType, List<? extends IJClassInterceptor>> userInterceptors) {
        this.metadata.setProperty("metadata.ejbjar.userInterceptors", userInterceptors);
    }


    /**
     * This method has DenyAll annotation.
     * @param denyAll the boolean value.
     */
    @Override
    public void setDenyAll(final boolean denyAll) {
        this.metadata.setProperty("metadata.ejbjar.denyall",  denyAll);
    }

    /**
     * @return true if DenyAll annotation.
     */
    @Override
    public boolean hasDenyAll() {
        return this.metadata.getBoolean("metadata.ejbjar.denyall");
    }



    /**
     * @return IJavaxJwsWebMethod object representing javax.jws.WebMethod annotation.
     */
    @Override
    public IJavaxJwsWebMethod getJavaxJwsWebMethod() {
        return metadata.get(IJavaxJwsWebMethod.class);
    }

    /**
     * Set webMethod object.
     * @param webMethod object representing javax.jws.WebMethod annotation.
     */
    @Override
    public void setJavaxJwsWebMethod(final IJavaxJwsWebMethod webMethod) {
        this.metadata.set(IJavaxJwsWebMethod.class,  webMethod);
    }


    /**
     * @return true if method is an afterBegin method.
     */
    @Override
    public boolean isAfterBegin() {
        return metadata.getBoolean("javax.ejb.AfterBegin");
    }

    /**
     * Sets true if method is an afterBegin method.
     */
    @Override
    public void setAfterBegin() {
        metadata.setProperty("javax.ejb.AfterBegin", true);
        IMetadata parentMetadata = metadata.getParent();
        IEJBClassView classView = parentMetadata.as(IEJBClassView.class);
        classView.setSessionSynchronization();
    }

    /**
     * @return true if method is an AfterCompletion method.
     */
    @Override
    public boolean isAfterCompletion() {
        return metadata.getBoolean("javax.ejb.AfterCompletion");
    }

    /**
     * Sets true if method is an AfterCompletion method.
     */
    @Override
    public void setAfterCompletion() {
        metadata.setProperty("javax.ejb.AfterCompletion", true);
        IMetadata parentMetadata = metadata.getParent();
        IEJBClassView classView = parentMetadata.as(IEJBClassView.class);
        classView.setSessionSynchronization();
    }

    /**
     * @return true if method is an beforeCompletion method.
     */
    @Override
    public boolean isBeforeCompletion() {
        return metadata.getBoolean("javax.ejb.BeforeCompletion");

    }

    /**
     * Sets true if method is an beforeCompletion method.
     */
    @Override
    public void setBeforeCompletion() {
        metadata.setProperty("javax.ejb.BeforeCompletion", true);
        IMetadata parentMetadata = metadata.getParent();
        IEJBClassView classView = parentMetadata.as(IEJBClassView.class);
        classView.setSessionSynchronization();
    }

    /**
     * @return list representing &#64;{@link javax.ejb.Schedules} and
     * &#64;{@link javax.ejb.Schedule} annotation.
     */
    @Override
    public List<IJEjbSchedule> getJavaxEjbSchedules() {
        return metadata.getProperty("javax.ejb.Schedules");
    }

    /**
     * Set jEjbSchedules object.
     * @param jEjbSchedules list representing &#64;{@link javax.ejb.Schedules}
     * and &#64;{@link javax.ejb.Schedule} annotation.
     */
    @Override
    public void setJavaxEjbSchedules(final List<IJEjbSchedule> jEjbSchedules) {
        this.metadata.setProperty("javax.ejb.Schedules", jEjbSchedules);
    }



    /**
     * Adds a new jEjbSchedule on this method.
     * @param jEjbSchedule information on the schedules.
     */
    @Override
    public void addJavaxEjbSchedule(IJEjbSchedule jEjbSchedule) {
        if (this.getJavaxEjbSchedules() == null) {
            List<IJEjbSchedule> jeEjbSchedules = new ArrayList<IJEjbSchedule>();
            setJavaxEjbSchedules(jeEjbSchedules);
        }
        getJavaxEjbSchedules().add(jEjbSchedule);
    }


    @Override
    public IEjbJarMetadata getEjbJarDeployableMetadata() {
        IMetadata parent = metadata.getParent();
        if (parent instanceof IEjbJarMetadata) {
            return (IEjbJarMetadata) parent;
        }
        throw new IllegalArgumentException("No EJBJAR deployable as parent. Found '" + parent + "' while current metadata is '" + this.metadata + "'");
    }


}
