/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.ejbjar.impl.xml.parsing;

import static org.ow2.util.ee.metadata.common.api.xml.struct.ILifeCycleCallback.POST_ACTIVATE;
import static org.ow2.util.ee.metadata.common.api.xml.struct.ILifeCycleCallback.PRE_PASSIVATE;

import java.net.URL;
import java.util.List;
import java.util.concurrent.TimeUnit;

import org.ow2.util.ee.metadata.common.api.struct.ITransactionAttributeType;
import org.ow2.util.ee.metadata.common.api.xml.struct.IActivationConfig;
import org.ow2.util.ee.metadata.common.api.xml.struct.IEnvironment;
import org.ow2.util.ee.metadata.common.impl.xml.parsing.CommonDeploymentDescLoader;
import org.ow2.util.ee.metadata.common.impl.xml.parsing.ParsingException;
import org.ow2.util.ee.metadata.common.impl.xml.struct.ActivationConfig;
import org.ow2.util.ee.metadata.common.impl.xml.struct.AroundInvoke;
import org.ow2.util.ee.metadata.common.impl.xml.struct.MethodDD;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IAssemblyDescriptor;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IConcurrentMethod;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IContainerTransaction;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IEnterpriseBeans;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IInterceptor;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IInterceptorBinding;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IInterceptors;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.ILockType;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IMessageDestination;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IMessageDriven;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IMethodPermission;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IRemoveMethod;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.ISession;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.ITimer;
import org.ow2.util.ee.metadata.ejbjar.impl.struct.JApplicationException;
import org.ow2.util.ee.metadata.ejbjar.impl.struct.JEJBSchedule;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.AbsBean;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.AccessTimeout;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.AssemblyDescriptor;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.ConcurrentMethod;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.ContainerTransaction;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.EJB3;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.EnterpriseBeans;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.Interceptor;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.InterceptorBinding;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.Interceptors;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.MessageDestination;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.MessageDriven;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.MethodPermission;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.RemoveMethod;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.Session;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.StatefulTimeout;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.Timer;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.xml.DocumentParser;
import org.ow2.util.xml.DocumentParserException;
import org.ow2.util.xml.XMLUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * This class analyzes the EJB3 deployment descriptor and fill a struct with
 * getters.
 * @author Florent Benoit
 */
public final class EJB3DeploymentDescLoader extends CommonDeploymentDescLoader {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(EJB3DeploymentDescLoader.class);

    /**
     * EJB3 instance object that is the result of the parsing.
     */
    private EJB3 ejb3 = null;

    /**
     * Utility class, no public constructor.
     * @param url the url that is pointing to the EJB Xml file.
     */
    private EJB3DeploymentDescLoader(final URL url) {
        super(url);
        this.ejb3 = new EJB3();
    }

    /**
     * Load the ejb-jar.xml file.
     * @param url the URL of the XML file.
     * @throws ParsingException if parsing of XML file fails.
     * @return an application object.
     */
    public static EJB3 loadDeploymentDescriptor(final URL url) throws ParsingException {
        logger.debug("Analyzing url {0}", url);

        EJB3DeploymentDescLoader ejbDeploymentDescLoader = new EJB3DeploymentDescLoader(url);
        ejbDeploymentDescLoader.parse();
        return ejbDeploymentDescLoader.getEJB3();
    }

    /**
     * Analyze the URL and build an EJB3 object which contains data of the XML.
     * @throws ParsingException if the analyze of the file fails.
     */
    public void parse() throws ParsingException {
        // Get document
        Document document = null;
        try {
            document = DocumentParser.getDocument(getURL(), isValidating(), new EJB3EntityResolver());
        } catch (DocumentParserException e) {
            throw new ParsingException("Cannot parse the url", e);
        }

        // Root element = <ejb-jar>
        Element ejbJarRootElement = document.getDocumentElement();

        // Define NS
        defineXMLNS(ejbJarRootElement);

        // enterprise-beans
        NodeList enterprisebeansList = ejbJarRootElement.getElementsByTagNameNS(getXMLNS(), IEnterpriseBeans.NAME);
        if (enterprisebeansList.getLength() == 1) {
            Element enterpriseBeansElement = (Element) enterprisebeansList.item(0);

            // Create struct
            IEnterpriseBeans enterpriseBeans = new EnterpriseBeans();
            ejb3.setEnterpriseBeans(enterpriseBeans);

            // Analyze beans
            analyzeBeans(enterpriseBeansElement, enterpriseBeans);

        }

        // interceptors
        NodeList interceptorsList = ejbJarRootElement.getElementsByTagNameNS(getXMLNS(), IInterceptors.NAME);
        if (interceptorsList.getLength() == 1) {
            Element interceptorsElement = (Element) interceptorsList.item(0);

            // Create struct
            IInterceptors interceptors = new Interceptors();
            ejb3.setInterceptors(interceptors);

            // Analyze the interceptors element
            analyzeInterceptorsElement(interceptorsElement, interceptors);
        }

        // assembly-descriptor
        NodeList assemblyDescriptorList = ejbJarRootElement.getElementsByTagNameNS(getXMLNS(), IAssemblyDescriptor.NAME);
        if (assemblyDescriptorList.getLength() == 1) {
            Element assemblyDescriptorElement = (Element) assemblyDescriptorList.item(0);

            // Create struct
            IAssemblyDescriptor assemblyDescriptor = new AssemblyDescriptor();
            ejb3.setAssemblyDescriptor(assemblyDescriptor);

            // Analyze the assembly-descriptor element
            analyzeAssemblyDescriptor(assemblyDescriptorElement, assemblyDescriptor);
        }

        if ("true".equals(ejbJarRootElement.getAttribute("metadata-complete"))) {
            ejb3.setMetadataComplete(true);
        } else {
            ejb3.setMetadataComplete(false);
        }

    }

    /**
     * Analyze common stuff of beans.
     * @param enterpriseBeansElement the dom element.
     * @param enterpriseBeans the structure representing object.
     */
    private void analyzeBeans(final Element enterpriseBeansElement, final IEnterpriseBeans enterpriseBeans) {
        // Get session beans
        NodeList sessionBeansList = enterpriseBeansElement.getElementsByTagNameNS(getXMLNS(), ISession.NAME);

        // Loop on session beans
        for (int i = 0; i < sessionBeansList.getLength(); i++) {
            Element sessionElement = (Element) sessionBeansList.item(i);

            // Build instance that is created.
            Session session = new Session();
            enterpriseBeans.addSession(session);

            // Analyze session bean
            analyzeSessionBean(sessionElement, session);
        }

        // Get MDB beans
        NodeList messageDrivenBeansList = enterpriseBeansElement.getElementsByTagNameNS(getXMLNS(), IMessageDriven.NAME);

        // Loop on MDB
        for (int i = 0; i < messageDrivenBeansList.getLength(); i++) {
            Element mdbElement = (Element) messageDrivenBeansList.item(i);

            // Build instance that is created.
            MessageDriven messageDriven = new MessageDriven();
            enterpriseBeans.addMessageDriven(messageDriven);

            // Analyze MDB
            analyzeMessageDrivenBean(mdbElement, messageDriven);
        }

    }

    /**
     * Analyze common stuff of a bean.
     * @param beanElement the dom element.
     * @param bean the structure representing object.
     */
    private void analyzeBean(final Element beanElement, final AbsBean bean) {
        // Get ejb-name
        String ejbName = XMLUtils.getStringValueElement(getXMLNS(), beanElement, "ejb-name");
        bean.setEjbName(ejbName);

        // Get ejb-class
        String ejbClass = XMLUtils.getStringValueElement(getXMLNS(), beanElement, "ejb-class");
        bean.setEjbClass(ejbClass);

        // Security identity (run-as) if any
        String runAsRole = XMLUtils.getStringValueElement(getXMLNS(), beanElement, "run-as");
        if (runAsRole != null) {
            bean.setRunAsRole(runAsRole);
        }

        // Mapped Name
        String mappedName = XMLUtils.getChildStringValueForElement(beanElement, "mapped-name");
        if (mappedName != null) {
            bean.setMappedName(mappedName);
        }

        // Get transaction-type
        String transactionType = XMLUtils.getStringValueElement(getXMLNS(), beanElement, "transaction-type");
        bean.setTransactionType(transactionType);

        // Needs to find timeout-method if specified (old fashion timer)
        Element timeoutMethodElement = getChildElementForGivenName(beanElement, "timeout-method");
        if (timeoutMethodElement != null) {
            MethodDD methodDD = getMethodFromElement(timeoutMethodElement);
            bean.setTimeoutMethod(methodDD);
        }

        // Analyze timer (schedule based timer)
        analyzeTimer(beanElement, bean);

        // Analyze common things
        analyzeEnvironment(beanElement, bean);
    }

    /**
     * Analyze XML of a MDB.
     * @param beanElement the dom element.
     * @param bean the structure representing object.
     */
    private void analyzeMessageDrivenBean(final Element beanElement, final MessageDriven bean) {
        analyzeBean(beanElement, bean);

        // Message destination type
        String messageDestinationType = XMLUtils.getChildStringValueForElement(beanElement, "message-destination-type");
        bean.setMessageDestinationType(messageDestinationType);

        // Message destination link
        String messageDestinationLink = XMLUtils.getChildStringValueForElement(beanElement, "message-destination-link");
        bean.setMessageDestinationLink(messageDestinationLink);

        // Messaging type
        String messagingType = XMLUtils.getStringValueElement(getXMLNS(), beanElement, "messaging-type");
        bean.setMessagingType(messagingType);

        // Get activation-config
        NodeList activationConfigList = beanElement.getElementsByTagNameNS(getXMLNS(), IActivationConfig.NAME);
        // Loop on the activation-config
        for (int i = 0; i < activationConfigList.getLength(); i++) {
            Element activationConfigElement = (Element) activationConfigList.item(i);

            // Get activation-config-property
            NodeList activationConfigPropertyList = activationConfigElement.getElementsByTagNameNS(getXMLNS(),
                    "activation-config-property");
            for (int j = 0; j < activationConfigPropertyList.getLength(); j++) {
                Element activationConfigPropertyElement = (Element) activationConfigPropertyList.item(j);

                // Build Object
                ActivationConfig activationConfig = new ActivationConfig();
                bean.addActivationConfig(activationConfig);
                String name = XMLUtils.getStringValueElement(getXMLNS(), activationConfigPropertyElement,
                        "activation-config-property-name");
                String value = XMLUtils.getStringValueElement(getXMLNS(), activationConfigPropertyElement,
                        "activation-config-property-value");
                activationConfig.setName(name);
                activationConfig.setValue(value);
            }
        }

    }

    /**
     * Analyze XML of a session bean.
     * @param beanElement the dom element.
     * @param bean the structure representing object.
     */
    private void analyzeSessionBean(final Element beanElement, final Session bean) {
        analyzeBean(beanElement, bean);

        // Get business-remote
        List<String> businessRemoteList = XMLUtils.getStringListValueElement(getXMLNS(), beanElement, "business-remote");
        for (String businessRemote : businessRemoteList) {
            bean.addBusinessRemote(businessRemote);
        }

        // Get business-local
        List<String> businessLocalList = XMLUtils.getStringListValueElement(getXMLNS(), beanElement, "business-local");
        for (String businessLocal : businessLocalList) {
            bean.addBusinessLocal(businessLocal);
        }

                // Get stateful-timeout
        Element statefulTimeoutElement = getChildElementForGivenName(beanElement, "stateful-timeout");
        if (statefulTimeoutElement != null) {
            StatefulTimeout statefulTimeout = new StatefulTimeout();
            String timeoutValueStr = XMLUtils.getStringValueElement(getXMLNS(), statefulTimeoutElement, "timeout");
            statefulTimeout.setTimeout(Long.valueOf(timeoutValueStr));
            String unitStr = XMLUtils.getStringValueElement(getXMLNS(), statefulTimeoutElement, "unit");
            statefulTimeout.setUnit(TimeUnit.valueOf(unitStr.toUpperCase()));
            bean.setStatefulTimeout(statefulTimeout);
        }


        // Remove Methods
        NodeList removeMethodList = beanElement.getElementsByTagNameNS(getXMLNS(), IRemoveMethod.NAME);

        // Loop on the remove-method
        for (int i = 0; i < removeMethodList.getLength(); i++) {
            Element removeMethodElement = (Element) removeMethodList.item(i);

            RemoveMethod removeMethod = new RemoveMethod();
            bean.addRemoveMethod(removeMethod);

            // Analyze method
            NodeList beanMethodList = removeMethodElement.getElementsByTagNameNS(getXMLNS(), "bean-method");
            removeMethod.setMethod(getMethodFromElement((Element) beanMethodList.item(0)));

            // Retain if exception
            String retainIfExceptionStr = XMLUtils.getStringValueElement(getXMLNS(), removeMethodElement, "retain-if-exception");
            if (retainIfExceptionStr != null) {
                // convert into boolean value
                boolean retainIfException = false;
                retainIfException = Boolean.parseBoolean(retainIfExceptionStr);
                removeMethod.setRetainIfException(retainIfException);
            }
        }

        // Concurrent Methods
        NodeList concurrentMethodList = beanElement.getElementsByTagNameNS(getXMLNS(), IConcurrentMethod.NAME);

        // Loop on the concurrent-method
        for (int i = 0; i < concurrentMethodList.getLength(); i++) {
            Element concurrentMethodElement = (Element) concurrentMethodList.item(i);

            ConcurrentMethod concurrentMethod = new ConcurrentMethod();
            bean.addConcurrentMethod(concurrentMethod);

            // Analyze method
            NodeList beanMethodList = concurrentMethodElement.getElementsByTagNameNS(getXMLNS(), "method");
            concurrentMethod.setMethod(getMethodFromElement((Element) beanMethodList.item(0)));

            String lockStr = XMLUtils.getStringValueElement(getXMLNS(), concurrentMethodElement, "lock");
            if (lockStr != null) {
                // convert into enum value
                concurrentMethod.setLockType(ILockType.valueOf(lockStr.toUpperCase()));
            }

            // Access-Timeout (list with at more one element)
            NodeList accessTimeoutList = concurrentMethodElement.getElementsByTagNameNS(getXMLNS(), "access-timeout");
            for (int j = 0; j < accessTimeoutList.getLength(); j++) {
                AccessTimeout accessTimeout = new AccessTimeout();
                concurrentMethod.setAccessTimeout(accessTimeout);
                Element accessTimeoutElement = (Element) accessTimeoutList.item(j);
                String timeoutValueStr = XMLUtils.getStringValueElement(getXMLNS(), accessTimeoutElement, "timeout");
                accessTimeout.setTimeout(Long.valueOf(timeoutValueStr));
                String unitStr = XMLUtils.getStringValueElement(getXMLNS(), accessTimeoutElement, "unit");
                accessTimeout.setUnit(TimeUnit.valueOf(unitStr.toUpperCase()));
            }
        }


        // Loop on the after-begin-method (loop on all items but it can have only one)
        NodeList afterBeginMethodList = beanElement.getElementsByTagNameNS(getXMLNS(), "after-begin-method");
        for (int i = 0; i < afterBeginMethodList.getLength(); i++) {
            Element afterBeginMethodElement = (Element) afterBeginMethodList.item(i);
            bean.setAfterBeginMethod(getMethodFromElement(afterBeginMethodElement));
        }
        // Loop on the before-completion-method (loop on all items but it can have only one)
        NodeList beforeCompletionMethodList = beanElement.getElementsByTagNameNS(getXMLNS(), "before-completion-method");
        for (int i = 0; i < beforeCompletionMethodList.getLength(); i++) {
            Element beforeCompletionMethodElement = (Element) beforeCompletionMethodList.item(i);
            bean.setBeforeCompletionMethod(getMethodFromElement(beforeCompletionMethodElement));
        }
        // Loop on the after-completion-method (loop on all items but it can have only one)
        NodeList afterCompletionMethodList = beanElement.getElementsByTagNameNS(getXMLNS(), "after-completion-method");
        for (int i = 0; i < afterCompletionMethodList.getLength(); i++) {
            Element afterCompletionMethodElement = (Element) afterCompletionMethodList.item(i);
            bean.setAfterCompletionMethod(getMethodFromElement(afterCompletionMethodElement));
        }

        // Get remote home
        String remoteHome = XMLUtils.getChildStringValueForElement(beanElement, "home");
        bean.setRemoteHome(remoteHome);

        // Get Local home
        String localHome = XMLUtils.getChildStringValueForElement(beanElement, "local-home");
        bean.setLocalHome(localHome);

        // Get Local bean
        NodeList localBeanList  = beanElement.getElementsByTagNameNS(getXMLNS(), "local-bean");
        if (localBeanList != null && localBeanList.getLength() == 1) {
            bean.setLocalBean();
        }

        // Get session-type
        String sessionType = XMLUtils.getStringValueElement(getXMLNS(), beanElement, "session-type");
        bean.setSessionType(sessionType);

    }

    /**
     * Gets the methodDD from the given dom element.
     * @param methodElement the dom element to analyze
     * @return the given method object
     */
    private MethodDD getMethodFromElement(final Element methodElement) {
        // create object
        MethodDD method = new MethodDD();

        // Analyze name
        String methodName = XMLUtils.getStringValueElement(getXMLNS(), methodElement, "method-name");
        method.setName(methodName);

        // Sometime there is an ejb-name
        String ejbName = XMLUtils.getStringValueElement(getXMLNS(), methodElement, "ejb-name");
        method.setEjbName(ejbName);

        // Params
        NodeList methodParamsList = methodElement.getElementsByTagNameNS(getXMLNS(), "method-params");
        if (methodParamsList != null && methodParamsList.getLength() == 1) {
            // There are params but maybe if will defined empty parameters so init the params here
            method.initParams();
            // first item is the method-params element
            Element methodParamsElement = (Element) methodParamsList.item(0);

            List<String> paramList = XMLUtils.getStringListValueElement(getXMLNS(), methodParamsElement, "method-param");
            for (String param : paramList) {
                method.addParam(param);
            }

        }
        return method;
    }

    /**
     * Returns the child node of the specified name
     * @param base the element from where to search.
     * @param name of the element to get.
     * @return the found element or null if not found.
     */
    public static Element getChildElementForGivenName(final Element base, final String name) {
        Element foundElement = null;
        NodeList nodeList = base.getChildNodes();
        if (nodeList.getLength() > 0) {
            int length = nodeList.getLength();
            for (int i = 0; i < length; i++) {
                Node node = nodeList.item(i);

                // Get an element, create an instance of the element
                if (Node.ELEMENT_NODE == node.getNodeType()) {
                    if (name.equals(node.getNodeName())) {
                        // Get value of this child
                        foundElement = (Element) node;
                        if (foundElement != null) {
                            break;
                        }
                    }
                }
            }
        }
        return foundElement;
    }

    /**
     * Analyze the interceptors element of an EJB-JAR.
     * @param interceptorsElement the dom element.
     * @param interceptors the structure representing object.
     */
    private void analyzeInterceptorsElement(final Element interceptorsElement, final IInterceptors interceptors) {
        // Get interceptor list elements
        NodeList interceptorList = interceptorsElement.getElementsByTagNameNS(getXMLNS(), IInterceptor.NAME);

        // Loop on <interceptor> elements
        for (int i = 0; i < interceptorList.getLength(); i++) {
            Element interceptorElement = (Element) interceptorList.item(i);

            Interceptor interceptor = new Interceptor();
            interceptors.addInterceptor(interceptor);

            // Extract interceptor class
            String interceptorClass = XMLUtils.getStringValueElement(getXMLNS(), interceptorElement, "interceptor-class");
            interceptor.setInterceptorClass(interceptorClass);

            // Analyze the object
            analyzeEnvironment(interceptorElement, interceptor);
        }
    }

    /**
     * Analyze the assembly descriptor of an EJB-JAR.
     * @param assemblyDescriptorElement the dom element.
     * @param assemblyDescriptor the structure representing object.
     */
    private void analyzeAssemblyDescriptor(final Element assemblyDescriptorElement,
            final IAssemblyDescriptor assemblyDescriptor) {
        analyzeApplicationException(assemblyDescriptorElement, assemblyDescriptor);

        analyzeInterceptorBinding(assemblyDescriptorElement, assemblyDescriptor);

        analyzeContainerTransaction(assemblyDescriptorElement, assemblyDescriptor);

        analyzeSecurity(assemblyDescriptorElement, assemblyDescriptor);

        analyzeMessageDestination(assemblyDescriptorElement, assemblyDescriptor);
    }

    /**
     * Analyze the interceptor binding of the assembly descriptor of an EJB-JAR.
     * @param assemblyDescriptorElement the dom element.
     * @param assemblyDescriptor the structure representing object.
     */
    private void analyzeInterceptorBinding(final Element assemblyDescriptorElement,
            final IAssemblyDescriptor assemblyDescriptor) {
        // Get interceptor-binding list
        NodeList interceptorBindingList = assemblyDescriptorElement.getElementsByTagNameNS(getXMLNS(), IInterceptorBinding.NAME);

        // Loop on interceptor-binding
        for (int i = 0; i < interceptorBindingList.getLength(); i++) {
            Element interceptorBindingElement = (Element) interceptorBindingList.item(i);

            // Build struct and set it in the assembly descriptor
            InterceptorBinding interceptorBinding = new InterceptorBinding();
            assemblyDescriptor.addInterceptorBinding(interceptorBinding);

            // name
            String ejbName = XMLUtils.getStringValueElement(getXMLNS(), interceptorBindingElement, "ejb-name");
            interceptorBinding.setEjbName(ejbName);

            // interceptor-order
            NodeList interceptorOrderList = interceptorBindingElement.getElementsByTagNameNS(getXMLNS(), "interceptor-order");
            if (interceptorOrderList != null && interceptorOrderList.getLength() == 1) {
                // first item is the interceptor-order element
                Element interceptorOrderElement = (Element) interceptorOrderList.item(0);
                List<String> interceptorOrderClassList = XMLUtils.getStringListValueElement(getXMLNS(), interceptorOrderElement,
                        "interceptor-class");
                for (String interceptorClass : interceptorOrderClassList) {
                    interceptorBinding.addOrderedInterceptorClass(interceptorClass);
                }
            } else {
                // interceptor-class (as there is no interceptor-order element)
                List<String> interceptorClassList = XMLUtils.getStringListValueElement(getXMLNS(), interceptorBindingElement,
                        "interceptor-class");
                for (String interceptorClass : interceptorClassList) {
                    interceptorBinding.addInterceptorClass(interceptorClass);
                }
            }

            // method
            NodeList methodList = interceptorBindingElement.getElementsByTagNameNS(getXMLNS(), "method");
            if (methodList.getLength() > 0) {
                interceptorBinding.setMethod(getMethodFromElement((Element) methodList.item(0)));
            }

            // exclude-class-interceptors
            String excludeClassInterceptorsStr = XMLUtils.getStringValueElement(getXMLNS(), interceptorBindingElement,
                    "exclude-class-interceptors");
            if (excludeClassInterceptorsStr != null) {
                // convert into boolean value
                boolean excludeClassInterceptors = false;
                excludeClassInterceptors = Boolean.parseBoolean(excludeClassInterceptorsStr);
                interceptorBinding.setExcludeClassInterceptors(excludeClassInterceptors);
            }

            // exclude-default-interceptors
            String excludeDefaultInterceptorsStr = XMLUtils.getStringValueElement(getXMLNS(), interceptorBindingElement,
                    "exclude-default-interceptors");
            if (excludeDefaultInterceptorsStr != null) {
                // convert into boolean value
                boolean excludeDefaultInterceptors = false;
                excludeDefaultInterceptors = Boolean.parseBoolean(excludeDefaultInterceptorsStr);
                interceptorBinding.setExcludeDefaultInterceptors(excludeDefaultInterceptors);
            }

        }
    }

    /**
     * Analyze the application Exception of the assembly descriptor of an
     * EJB-JAR.
     * @param assemblyDescriptorElement the dom element.
     * @param assemblyDescriptor the structure representing object.
     */
    private void analyzeApplicationException(final Element assemblyDescriptorElement,
            final IAssemblyDescriptor assemblyDescriptor) {
        // Get application-exception list
        NodeList applicationExceptionList = assemblyDescriptorElement.getElementsByTagNameNS(getXMLNS(),
                JApplicationException.NAME);

        // Loop on application-exception
        for (int i = 0; i < applicationExceptionList.getLength(); i++) {
            Element applicationExceptionElement = (Element) applicationExceptionList.item(i);

            // name
            String className = XMLUtils.getStringValueElement(getXMLNS(), applicationExceptionElement, "exception-class");

            // rollback
            String rollbackStr = XMLUtils.getStringValueElement(getXMLNS(), applicationExceptionElement, "rollback");

            // inherited
            String inheritedStr = XMLUtils.getStringValueElement(getXMLNS(), applicationExceptionElement, "inherited");


            // convert into boolean value
            boolean rollback = false;
            if (rollbackStr != null) {
                rollback = Boolean.parseBoolean(rollbackStr);
            }

            // convert into boolean value
            boolean inherited = true;
            if (inheritedStr != null) {
                inherited = Boolean.parseBoolean(inheritedStr);
            }

            // Build instance that is created.
            JApplicationException applicationException = new JApplicationException(className, rollback, inherited);
            assemblyDescriptor.addApplicationException(applicationException);
        }
    }

    /**
     * Analyze the container transaction element of the assembly descriptor of
     * an EJB-JAR.
     * @param assemblyDescriptorElement the dom element.
     * @param assemblyDescriptor the structure representing object.
     */
    private void analyzeContainerTransaction(final Element assemblyDescriptorElement,
            final IAssemblyDescriptor assemblyDescriptor) {

        // Get container-transaction list
        NodeList containerTransactionList = assemblyDescriptorElement.getElementsByTagNameNS(getXMLNS(),
                IContainerTransaction.NAME);

        // Loop on container-transaction
        for (int i = 0; i < containerTransactionList.getLength(); i++) {
            Element containerTransactionElement = (Element) containerTransactionList.item(i);

            // Build struct and set it in the assembly descriptor
            ContainerTransaction containerTransaction = new ContainerTransaction();
            assemblyDescriptor.addContainerTransaction(containerTransaction);

            // Get Method element
            NodeList methodList = containerTransactionElement.getElementsByTagNameNS(getXMLNS(), "method");
            for (int j = 0; j < methodList.getLength(); j++) {
                Element methodElement = (Element) methodList.item(j);

                // Set method
                containerTransaction.addMethod(getMethodFromElement(methodElement));
            }

            // Tx attribute
            String transactionAttribute = XMLUtils.getStringValueElement(getXMLNS(), containerTransactionElement,
                    "trans-attribute");
            if (transactionAttribute != null) {
                if ("NotSupported".equals(transactionAttribute)) {
                    containerTransaction.setTransactionAttribute(ITransactionAttributeType.NOT_SUPPORTED);
                } else if ("Supports".equals(transactionAttribute)) {
                    containerTransaction.setTransactionAttribute(ITransactionAttributeType.SUPPORTS);
                } else if ("Required".equals(transactionAttribute)) {
                    containerTransaction.setTransactionAttribute(ITransactionAttributeType.REQUIRED);
                } else if ("RequiresNew".equals(transactionAttribute)) {
                    containerTransaction.setTransactionAttribute(ITransactionAttributeType.REQUIRES_NEW);
                } else if ("Mandatory".equals(transactionAttribute)) {
                    containerTransaction.setTransactionAttribute(ITransactionAttributeType.MANDATORY);
                } else if ("Never".equals(transactionAttribute)) {
                    containerTransaction.setTransactionAttribute(ITransactionAttributeType.NEVER);
                } else {
                    throw new IllegalStateException("Invalid transaction attribute '" + transactionAttribute + "'");
                }
            }
        }
    }

    /**
     * Analyze the security includind security-role, method-permission and
     * exclude-list elements of the assembly descriptor of an EJB-JAR.
     * @param assemblyDescriptorElement the dom element.
     * @param assemblyDescriptor the structure representing object.
     */
    private void analyzeSecurity(final Element assemblyDescriptorElement, final IAssemblyDescriptor assemblyDescriptor) {

        // Get security-role list
        NodeList securityRoleList = assemblyDescriptorElement.getElementsByTagNameNS(getXMLNS(), "security-role");

        // Loop on security-role
        for (int i = 0; i < securityRoleList.getLength(); i++) {
            Element securityRoleElement = (Element) securityRoleList.item(i);

            // Get the security role name
            String securityRole = XMLUtils.getStringValueElement(getXMLNS(), securityRoleElement, "role-name");
            if (securityRole != null) {
                assemblyDescriptor.addSecurityRole(securityRole);
            }
        }

        // Get method-permission
        NodeList methodPermissionList = assemblyDescriptorElement.getElementsByTagNameNS(getXMLNS(), IMethodPermission.NAME);

        // Loop on method-permission
        for (int i = 0; i < methodPermissionList.getLength(); i++) {
            Element methodPermissionElement = (Element) methodPermissionList.item(i);

            // Build object
            MethodPermission methodPermission = new MethodPermission();
            assemblyDescriptor.addMethodPermission(methodPermission);

            // Get all the roles for this method permission
            List<String> roles = XMLUtils.getStringListValueElement(getXMLNS(), methodPermissionElement, "role-name");
            for (String roleName : roles) {
                methodPermission.addRoleName(roleName);
            }

            // Unchecked ?
            NodeList uncheckedList = methodPermissionElement.getElementsByTagNameNS(getXMLNS(), "unchecked");
            if (uncheckedList.getLength() > 0) {
                methodPermission.setUnchecked(true);
            }

            // Get Method element
            NodeList methodList = methodPermissionElement.getElementsByTagNameNS(getXMLNS(), "method");
            for (int j = 0; j < methodList.getLength(); j++) {
                Element methodElement = (Element) methodList.item(j);

                // Add method
                methodPermission.addMethod(getMethodFromElement(methodElement));
            }
        }

        // Get exclude-list
        NodeList excludeList = assemblyDescriptorElement.getElementsByTagNameNS(getXMLNS(), "exclude-list");

        // Loop on exclude-list (in fact only one element may be found on this
        // list)
        for (int i = 0; i < excludeList.getLength(); i++) {
            Element excludeListElement = (Element) excludeList.item(i);

            // Get Method element
            NodeList methodList = excludeListElement.getElementsByTagNameNS(getXMLNS(), "method");
            for (int j = 0; j < methodList.getLength(); j++) {
                Element methodElement = (Element) methodList.item(j);

                // Add method
                assemblyDescriptor.addExcludeList(getMethodFromElement(methodElement));
            }
        }

    }

    /**
     * Analyze the message-destination of the assembly descriptor of an EJB-JAR.
     * @param assemblyDescriptorElement the dom element.
     * @param assemblyDescriptor the structure representing object.
     */
    private void analyzeMessageDestination(final Element assemblyDescriptorElement,
            final IAssemblyDescriptor assemblyDescriptor) {

        // Get message-destination list
        NodeList messageDestinationList = assemblyDescriptorElement.getElementsByTagNameNS(getXMLNS(), IMessageDestination.NAME);

        // Loop on message-destination
        for (int i = 0; i < messageDestinationList.getLength(); i++) {
            Element messageDestinationElement = (Element) messageDestinationList.item(i);

            // Create object
            IMessageDestination messageDestination = new MessageDestination();

            // Get the destination name
            String messageDestinationName = XMLUtils.getStringValueElement(getXMLNS(), messageDestinationElement,
                    "message-destination-name");
            if (messageDestinationName != null) {
                messageDestination.setName(messageDestinationName);
                assemblyDescriptor.addMessageDestination(messageDestination);
            }

            // Get the mappedName
            String mappedName = XMLUtils.getStringValueElement(getXMLNS(), messageDestinationElement, "mapped-name");
            if (mappedName != null) {
                messageDestination.setMappedName(mappedName);
            }


        }

    }

    /**
     * Analyze the lifecycle elements.
     * @param envElement the dom element.
     * @param env the structure representing object.
     */
    private void analyzeAroundInvoke(final Element envElement, final IEnvironment env) {
        // Get the around-invoke elements
        NodeList aroundInvokeList = envElement.getElementsByTagNameNS(getXMLNS(), "around-invoke");

        // Loop on the around-invoke
        for (int i = 0; i < aroundInvokeList.getLength(); i++) {
            Element aroundInvokeElement = (Element) aroundInvokeList.item(i);

            // Build instance that is created.
            AroundInvoke aroundInvoke = new AroundInvoke();
            env.addAroundInvoke(aroundInvoke);

            // lifecycle-callback-class
            String className = XMLUtils.getStringValueElement(getXMLNS(), aroundInvokeElement, "class");
            aroundInvoke.setClassName(className);

            // lifecycle-callback-method
            String method = XMLUtils.getStringValueElement(getXMLNS(), aroundInvokeElement, "method-name");
            aroundInvoke.setMethodName(method);

        }

    }

    /**
     * Analyze the Timer element.
     * @param beanElement the dom element.
     * @param bean the structure representing object.
     */
    protected void analyzeTimer(final Element beanElement, final AbsBean bean) {
        // Get the lifecycle elements
        NodeList timerList = beanElement.getElementsByTagNameNS(getXMLNS(), ITimer.NAME);
        for (int i = 0; i < timerList.getLength(); i++) {
            Element timerElement = (Element) timerList.item(i);

            Timer timer = new Timer();
            bean.addTimer(timer);

            // Schedule
            JEJBSchedule schedule = new JEJBSchedule();
            timer.setSchedule(schedule);

            NodeList scheduleList = timerElement.getElementsByTagNameNS(getXMLNS(), "schedule");
            Element scheduleElement = (Element) scheduleList.item(0);

            String second = XMLUtils.getStringValueElement(getXMLNS(), scheduleElement, "second");
            if (second != null) {
                schedule.setSecond(second);
            }
            String minute = XMLUtils.getStringValueElement(getXMLNS(), scheduleElement, "minute");
            if (minute != null) {
                schedule.setMinute(minute);
            }
            String hour = XMLUtils.getStringValueElement(getXMLNS(), scheduleElement, "hour");
            if (hour != null) {
                schedule.setHour(hour);
            }
            String dayOfMonth = XMLUtils.getStringValueElement(getXMLNS(), scheduleElement, "day-of-month");
            if (dayOfMonth != null) {
                schedule.setDayOfMonth(dayOfMonth);
            }
            String month = XMLUtils.getStringValueElement(getXMLNS(), scheduleElement, "month");
            if (month != null) {
                schedule.setMonth(month);
            }
            String dayOfWeek = XMLUtils.getStringValueElement(getXMLNS(), scheduleElement, "day-of-week");
            if (dayOfWeek != null) {
                schedule.setDayOfWeek(dayOfWeek);
            }
            String year = XMLUtils.getStringValueElement(getXMLNS(), scheduleElement, "year");
            if (year != null) {
                schedule.setYear(year);
            }

            String persistent = XMLUtils.getStringValueElement(getXMLNS(), timerElement, "persistent");
            if (persistent != null) {
                schedule.setPersistent(Boolean.parseBoolean(persistent));
            }

            String timezone = XMLUtils.getStringValueElement(getXMLNS(), timerElement, "timezone");
            if (timezone != null) {
                schedule.setTimezone(timezone);
            }

            String info = XMLUtils.getStringValueElement(getXMLNS(), timerElement, "info");
            if (info != null) {
                schedule.setInfo(info);
            }

            String start = XMLUtils.getStringValueElement(getXMLNS(), scheduleElement, "start");
            if (start != null) {
                schedule.setStart(start);
            }

            String end = XMLUtils.getStringValueElement(getXMLNS(), scheduleElement, "end");
            if (end != null) {
                schedule.setEnd(end);
            }

            // Analyze method
            NodeList timeoutMethodList = timerElement.getElementsByTagNameNS(getXMLNS(), "timeout-method");
            timer.setTimeoutMethod(getMethodFromElement((Element) timeoutMethodList.item(0)));
        }

    }

    /**
     * Add analyze of passivate interceptors and aroundInvoke.
     * @param envElement the xml environment element
     * @param env the environment element
     */
    @Override
    protected void analyzeEnvironment(final Element envElement, final IEnvironment env) {
        analyzeLifeCycle(envElement, env, PRE_PASSIVATE);
        analyzeLifeCycle(envElement, env, POST_ACTIVATE);
        analyzeAroundInvoke(envElement, env);
        super.analyzeEnvironment(envElement, env);
    }

    /**
     * @return the object resulting of the XML parsing.
     */
    public EJB3 getEJB3() {
        return ejb3;
    }

}
