/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.ejbjar.impl.xml.struct;

import java.util.ArrayList;
import java.util.List;

import org.ow2.util.ee.metadata.common.api.xml.struct.IMethodDD;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IApplicationException;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IAssemblyDescriptor;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IContainerTransaction;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IInterceptorBinding;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IMessageDestination;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IMethodPermission;


/**
 * This class defines the implementation of the element assembly-descriptor.
 * @author Florent Benoit
 */
public class AssemblyDescriptor implements IAssemblyDescriptor {

    /**
     * Serial version UID.
     */
    private static final long serialVersionUID = 7146961120585818684L;

    /**
     * List of &lt;application-exception&gt; elements.
     */
    private List<IApplicationException> applicationExceptionList = null;

    /**
     * List of &lt;interceptor-binding&gt; elements.
     */
    private List<IInterceptorBinding> interceptorBindingList = null;

    /**
     * List of &lt;container-transaction&gt; elements.
     */
    private List<IContainerTransaction> containerTransactionList = null;

    /**
     * List of &lt;method-permission&gt; elements.
     */
    private List<IMethodPermission> methodPermissionList = null;

    /**
     * List of &lt;exclude-list&gt; elements.
     */
    private List<IMethodDD> excludeListMethods = null;

    /**
     * List of &lt;security-role&gt; elements.
     */
    private List<String> securityRoleList = null;

    /**
     * List of &lt;message-destination&gt; elements.
     */
    private List<IMessageDestination> messageDesinationList = null;


    /**
     * Constructor.
     */
    public AssemblyDescriptor() {
        this.applicationExceptionList = new ArrayList<IApplicationException>();
        this.interceptorBindingList = new ArrayList<IInterceptorBinding>();
        this.containerTransactionList = new ArrayList<IContainerTransaction>();
        this.securityRoleList = new ArrayList<String>();
        this.excludeListMethods = new ArrayList<IMethodDD>();
        this.methodPermissionList = new ArrayList<IMethodPermission>();
        this.messageDesinationList = new ArrayList<IMessageDestination>();
    }

    /**
     * Gets the list of &lt;application-exception&gt; elements.
     * @return list of &lt;application-exception&gt; elements.
     */
    public List<IApplicationException> getApplicationExceptionList() {
        return applicationExceptionList;
    }

    /**
     * Adds a new &lt;application-exception&gt; element to assembly descriptor.
     * @param applicationException the &lt;application-exception&gt; element
     */
    public void addApplicationException(final IApplicationException applicationException) {
        applicationExceptionList.add(applicationException);
    }

    /**
     * Gets the list of &lt;container-transaction&gt; elements.
     * @return list of &lt;container-transaction&gt; elements.
     */
    public List<IContainerTransaction> getContainerTransactionList() {
        return containerTransactionList;
    }

    /**
     * Adds a new &lt;container-transaction&gt; element to assembly descriptor.
     * @param containerTransaction the &lt;application-exception&gt; element
     */
    public void addContainerTransaction(final IContainerTransaction containerTransaction) {
        containerTransactionList.add(containerTransaction);
    }

    /**
     * Gets the list of &lt;interceptor-binding&gt; elements.
     * @return list of &lt;interceptor-binding&gt; elements.
     */
    public List<IInterceptorBinding> getInterceptorBindingList() {
        return interceptorBindingList;
    }

    /**
     * Adds a new &lt;interceptor-binding&gt; element to assembly descriptor.
     * @param interceptorBinding the &lt;interceptor-binding&gt; element
     */
    public void addInterceptorBinding(final IInterceptorBinding interceptorBinding) {
        interceptorBindingList.add(interceptorBinding);
    }

    /**
     * Gets the list of &lt;method-permission&gt; elements.
     * @return list of &lt;method-permission&gt; elements.
     */
    public List<IMethodPermission> getMethodPermissionList() {
        return methodPermissionList;
    }

    /**
     * Adds a new &lt;method-permission&gt; element to assembly descriptor.
     * @param methodPermission the &lt;method-permission&gt; element
     */
    public void addMethodPermission(final IMethodPermission methodPermission) {
        methodPermissionList.add(methodPermission);
    }

    /**
     * Gets the list of &lt;exclude-list&gt; elements.
     * @return list of &lt;exclude-list&gt; elements.
     */
    public List<IMethodDD> getExcludeListMethods() {
        return excludeListMethods;
    }

    /**
     * Adds a new &lt;exclude-list&gt; element to assembly descriptor.
     * @param excludeListMethod the &lt;exclude-list&gt; element
     */
    public void addExcludeList(final IMethodDD excludeListMethod) {
        excludeListMethods.add(excludeListMethod);
    }


    /**
     * Gets the list of &lt;security-role&gt; elements.
     * @return list of &lt;security-role&gt; elements.
     */
    public List<String> getSecurityRoleList() {
        return securityRoleList;
    }

    /**
     * Adds a new &lt;security-role&gt; element to assembly descriptor.
     * @param securityRole the &lt;security-role&gt; element
     */
    public void addSecurityRole(final String securityRole) {
        securityRoleList.add(securityRole);
    }

    /**
     * Gets the list of &lt;message-destination&gt; elements.
     * @return list of &lt;message-destination&gt; elements.
     */
    public List<IMessageDestination> getMessageDestinationlist() {
        return messageDesinationList;
    }

    /**
     * Adds a new &lt;message-destination&gt; element to assembly descriptor.
     * @param messageDestination the &lt;message-destination&gt; element
     */
    public void addMessageDestination(final IMessageDestination messageDestination) {
        messageDesinationList.add(messageDestination);
    }

}
