/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.ejbjar.impl.xml.struct;

import java.util.ArrayList;
import java.util.List;

import org.ow2.util.ee.metadata.common.api.xml.struct.IMethodDD;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IInterceptorBinding;

/**
 * This class represents the &lt;interceptor-binding&gt; element.
 * @author Florent Benoit
 */
public class InterceptorBinding implements IInterceptorBinding {

    /**
     * Serial version UID.
     */
    private static final long serialVersionUID = 5397163888723207502L;

    /**
     * Name of the EJB. (if * = default interceptor: apply to all beans)
     */
    private String ejbName = null;

    /**
     * List of interceptor-class.
     */
    private List<String> interceptorClassList = null;

    /**
     * Interceptor-order element.<br>
     * (which contains a list of interceptor-classList)
     */
    private List<String> orderInterceptorClassList = null;

    /**
     * Exclude the default interceptors for this bean ?
     */
    private boolean excludeDefaultInterceptors = false;

    /**
     * Exclude class interceptors for a method on a bean.
     */
    private boolean excludeClassInterceptors = false;

    /**
     * setExcludeClassInterceptors() called.
     */
    private boolean excludeClassInterceptorsCalled = false;


    /**
     * setExcludeClassInterceptors() called.
     */
    private boolean excludeDefaultInterceptorsCalled = false;

    /**
     * Method on which apply the interceptors.
     */
    private IMethodDD method = null;

    /**
     * Default constructor.<br>
     * orderInterceptorClassList is null by default.
     */
    public InterceptorBinding() {
        this.interceptorClassList = new ArrayList<String>();
    }


    /**
     * @return the name of the EJB.
     */
    public String getEjbName() {
        return ejbName;
    }


    /**
     * Sets the name of this ejb (or wildcard).
     * @param ejbName the name of the ejb.
     */
    public void setEjbName(final String ejbName) {
        this.ejbName = ejbName;
    }

    /**
     * @return the method element
     */
    public IMethodDD getMethod() {
        return method;
    }

    /**
     * Method on which define the interceptors.
     * @param method the given method.
     */
    public void setMethod(final IMethodDD method) {
        this.method = method;
    }


    /**
     * @return true if the class interceptors are excluded for the given method
     */
    public boolean isExcludeClassInterceptors() {
        return excludeClassInterceptors;
    }


    /**
     * Exclude or not class interceptors ?
     * @param excludeClassInterceptors true/false
     */
    public void setExcludeClassInterceptors(final boolean excludeClassInterceptors) {
        this.excludeClassInterceptors = excludeClassInterceptors;
        this.excludeClassInterceptorsCalled = true;
    }

    /**
     * @return true if method setExcludeClassInterceptors has been called (with false or true)
     */
    public boolean isExcludeClassInterceptorsCalled() {
        return excludeClassInterceptorsCalled;
    }


    /**
     * @return true if the default interceptors are excluded for the given bean
     */
    public boolean isExcludeDefaultInterceptors() {
        return excludeDefaultInterceptors;
    }

    /**
     * Exclude or not default interceptors ?
     * @param excludeDefaultInterceptors true/false
     */
    public void setExcludeDefaultInterceptors(final boolean excludeDefaultInterceptors) {
        this.excludeDefaultInterceptors = excludeDefaultInterceptors;
        this.excludeDefaultInterceptorsCalled = true;
    }

    /**
     * @return true if method setExcludeDefaultInterceptors has been called (with false or true)
     */
    public boolean isExcludeDefaultInterceptorsCalled() {
        return excludeDefaultInterceptorsCalled;
    }

    /**
     * @return list of the interceptor classes.
     */
    public List<String> getInterceptorClassList() {
        return interceptorClassList;
    }

    /**
     * @return list of ordered interceptor classes.
     */
    public List<String> getOrderInterceptorClassList() {
        return orderInterceptorClassList;
    }

    /**
     * Add the given interceptor to the list.
     * @param interceptorClassName the name of the interceptor's class
     */
    public void addInterceptorClass(final String interceptorClassName) {
        interceptorClassList.add(interceptorClassName);
    }

    /**
     * Add the given interceptor to the ordered list.
     * @param interceptorClassName the name of the interceptor's class
     */
    public synchronized void addOrderedInterceptorClass(final String interceptorClassName) {
        if (orderInterceptorClassList == null) {
            orderInterceptorClassList = new ArrayList<String>();
        }
        orderInterceptorClassList.add(interceptorClassName);
    }

}
