/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.ejbjar.impl.xml.struct;

import java.util.ArrayList;
import java.util.List;

import org.ow2.util.ee.metadata.common.api.xml.struct.IMethodDD;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IConcurrentMethod;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IRemoveMethod;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.ISession;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IStatefulTimeout;

/**
 * Defines a representation for &lt;session&gt; element.
 * @author Florent Benoit
 */
public class Session extends AbsBean implements ISession {

    /**
     * Serial version UID.
     */
    private static final long serialVersionUID = 5475755346906125496L;

    /**
     * business-remote interfaces.
     */
    private List<String> businessRemoteList = null;

    /**
     * business-local interfaces.
     */
    private List<String> businessLocalList = null;

    /**
     * List of remove-method (only for stateful beans).
     */
    private List<IRemoveMethod> removeMethods = null;

    /**
     * Type of session bean (stateless/stateful).
     */
    private String sessionType = null;

    /**
     * afterBegin method (for Stateful SB).
     */
    private IMethodDD afterBeginMethod = null;

    /**
     * beforeCompletion method (for Stateful SB).
     */
    private IMethodDD beforeCompletionMethod = null;

    /**
     * afterCompletion method (for Stateful SB).
     */
    private IMethodDD afterCompletionMethod = null;
    
    /**
     * Remote Home.
     */
    private String remoteHome = null;

    /**
     * Local Home.
     */
    private String localHome = null;
    
    /**
     * Concurrent methods.
     */
    private List<IConcurrentMethod> concurrentMethods = null;
    
    /**
     * Local Bean view ?
     */
    private boolean localBean = false;

    /**
       * Stateful timeout
       */
      private IStatefulTimeout statefulTimeout = null;


    /**
     * Constructor.
     */
    public Session() {
        super();
        businessRemoteList = new ArrayList<String>();
        businessLocalList = new ArrayList<String>();
        removeMethods = new ArrayList<IRemoveMethod>();
        this.concurrentMethods = new ArrayList<IConcurrentMethod>();
    }

    /**
     * Gets the business-remote interface list.
     * @return business-remote interface list
     */
    public List<String> getBusinessRemoteList() {
        return businessRemoteList;
    }

    /**
     * Gets the business-local interface list.
     * @return business-local interface list
     */
    public List<String> getBusinessLocalList() {
        return businessLocalList;
    }

    /**
     * Add the business-remote interface.
     * @param businessRemote business-remote interface.
     */
    public void addBusinessRemote(final String businessRemote) {
        businessRemoteList.add(businessRemote);
    }

    /**
     * Add the business-local interface.
     * @param businessLocal business-remote interface.
     */
    public void addBusinessLocal(final String businessLocal) {
        businessLocalList.add(businessLocal);
    }

    /**
     * Gets the list of remove methods.
     * @return list of remove methods
     */
    public List<IRemoveMethod> getRemoveMethodList() {
        return removeMethods;
    }

    /**
     * Add the method used as a remove method.
     * @param removeMethod the given remove method
     */
    public void addRemoveMethod(final IRemoveMethod removeMethod) {
        removeMethods.add(removeMethod);
    }

    /**
     * Add the method used as a concurrent method.
     * @param concurrentMethod the given concurrent method
     */
    public void addConcurrentMethod(final IConcurrentMethod concurrentMethod) {
        concurrentMethods.add(concurrentMethod);
    }
    
    /**
     * Gets the list of concurrent methods.
     * @return list of concurrent methods
     */
    public List<IConcurrentMethod> getConcurrentMethodList() {
        return concurrentMethods;
    }
    
    /**
     * Gets the session-type.
     * @return the session-type.
     */
    public String getSessionType() {
        return sessionType;
    }

    /**
     * Set the session-type.
     * @param sessionType the type of session.
     */
    public void setSessionType(final String sessionType) {
        this.sessionType = sessionType;
    }

    /**
     * Gets the remote Home.
     * @return the remote Home
     */
    public String getRemoteHome() {
        return remoteHome;
    }

    /**
     * Set the remote Home.
     * @param remoteHome the given remote Home.
     */
    public void setRemoteHome(final String remoteHome) {
        this.remoteHome = remoteHome;
    }

    /**
     * Gets the local Home.
     * @return the local Home
     */
    public String getLocalHome() {
        return localHome;
    }

    /**
     * Set the local Home.
     * @param localHome the given local Home.
     */
    public void setLocalHome(final String localHome) {
        this.localHome = localHome;
    }
    
    /**
     * Flag this bean with a no-interface view.
     */
    public void setLocalBean() {
        this.localBean = true;
    }

    /**
     * @return true if this bean has a no-interface view.
     */
    public boolean isLocalBean() {
        return this.localBean;
    }
    
    /**
     * Sets the afterBegin method used by stateful.
     * @param afterBeginMethod the given afterBegin method
     */
    public void setAfterBeginMethod(final IMethodDD afterBeginMethod) {
       this.afterBeginMethod = afterBeginMethod;
    }
    
    /**
     * @return the given afterBegin method
     */
    public IMethodDD getAfterBeginMethod() {
       return this.afterBeginMethod;
    }
    
    /**
     * Sets the afterCompletion method used by stateful.
     * @param afterCompletionMethod the given afterCompletion method
     */
    public void setAfterCompletionMethod(final IMethodDD afterCompletionMethod) {
       this.afterCompletionMethod = afterCompletionMethod;
    }
    
    /**
     * @return the given afterCompletion method
     */
    public IMethodDD getAfterCompletionMethod() {
       return this.afterCompletionMethod;
    }
    
    /**
     * Sets the beforeCompletion method used by stateful.
     * @param beforeCompletionMethod the given afterCompletion method
     */
    public void setBeforeCompletionMethod(final IMethodDD beforeCompletionMethod) {
       this.beforeCompletionMethod = beforeCompletionMethod;
    }
    
    /**
     * @return the given beforeCompletion method
     */
    public IMethodDD getBeforeCompletionMethod() {
       return this.beforeCompletionMethod;
    }


    public void setStatefulTimeout(IStatefulTimeout statefulTimeout) {
        this.statefulTimeout = statefulTimeout;
    }

    public IStatefulTimeout getStatefulTimeout() {
        return statefulTimeout;
    }

}
