/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.ejbjar.test;

import static org.testng.Assert.assertNotNull;

import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.TimeUnit;

import org.ow2.util.archive.impl.MemoryArchive;
import org.ow2.util.asm.Opcodes;
import org.ow2.util.ee.metadata.common.api.struct.IJAnnotationSqlDataSourceDefinition;
import org.ow2.util.ee.metadata.common.api.struct.IJInterceptors;
import org.ow2.util.ee.metadata.common.api.struct.IJavaxJwsWebMethod;
import org.ow2.util.ee.metadata.common.impl.view.CommonView;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.exceptions.EJBJARMetadataException;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IConcurrencyManagementType;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJEjbAccessTimeout;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJEjbSchedule;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJEjbStatefulTimeout;
import org.ow2.util.ee.metadata.ejbjar.api.struct.ILockType;
import org.ow2.util.ee.metadata.ejbjar.api.view.IEJBClassView;
import org.ow2.util.ee.metadata.ejbjar.api.view.IEJBMethodView;
import org.ow2.util.ee.metadata.ejbjar.impl.EjbJarMetadataFactory;
import org.ow2.util.marshalling.Serialization;
import org.ow2.util.marshalling.SerializationException;
import org.ow2.util.scan.api.metadata.IClassMetadata;
import org.ow2.util.scan.api.metadata.IMethodMetadata;
import org.ow2.util.scan.impl.ASMScannerImpl;
import org.ow2.util.scan.impl.metadata.JMethod;
import org.testng.Assert;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

/**
 * @author Gael Lalire
 */
public class TestEjbJarMetadata {

    /**
     * archive.
     */
    private MemoryArchive archive = null;

    /**
     * Stateless metadata.
     */
    private IClassMetadata ejb3StatelessJarClassMetadata = null;

    /**
     * Stateless DD metadata.
     */
    private IClassMetadata ejb3StatelessDDJarClassMetadata = null;

    /**
     * Stateful timeout metadata.
     */
    private IClassMetadata ejb3StatefulTimeoutJarClassMetadata = null;

    /**
     * Stateful timeout override metadata.
     */
    private IClassMetadata ejb3StatefulTimeoutOverrideJarClassMetadata = null;

    /**
     * ManagedBean metadata.
     */
    private IClassMetadata ejb3ManagedBeanJarClassMetadata = null;

    /**
     * The metadata of memory archive.
     */
    private IEjbJarMetadata ejbJarMetadata;


    /**
     * Create directories for the test.
     * @throws EJBJARMetadataException
     */
    @BeforeClass
    public void init() throws EJBJARMetadataException  {
        archive = new MemoryArchive(MyEJB3Stateless.class.getClassLoader(), Arrays.asList(MyEJB3Stateless.class.getName(),
                MyEJB3Stateful.class.getName(), MyInterceptor.class.getName(), MyEJBSingleton.class.getName(), MyEJB3StatelessDD.class.getName(),
                MyStatefulTimeout.class.getName(), MyStatefulTimeoutOverride.class.getName(), MyManagedBean.class.getName()));
        archive.addResource("META-INF/ejb-jar.xml", TestEjbJarMetadata.class.getClassLoader().getResource("org/ow2/util/ee/metadata/ejbjar/test/ejb-jar.xml"));


        EjbJarMetadataFactory metadataFactory = new EjbJarMetadataFactory(new ASMScannerImpl());
        ejbJarMetadata = metadataFactory.createArchiveMetadata(archive);
        assertNotNull(ejbJarMetadata);


        this.ejb3StatelessJarClassMetadata = ejbJarMetadata
        .getScannedClassMetadata(MyEJB3Stateless.class.getName().replace('.', '/'));


        ejb3StatelessDDJarClassMetadata = ejbJarMetadata
        .getBeanEjbJarClassMetadataForEjbName("TestXML", MyEJB3StatelessDD.class.getName().replace('.', '/'));

        ejb3StatefulTimeoutJarClassMetadata = ejbJarMetadata
                .getScannedClassMetadata(MyStatefulTimeout.class.getName().replace('.', '/'));


        ejb3StatefulTimeoutOverrideJarClassMetadata = ejbJarMetadata
                .getBeanEjbJarClassMetadataForEjbName("overrided-stateful-timeout",
                        MyStatefulTimeoutOverride.class.getName().replace('.', '/'));

        ejb3ManagedBeanJarClassMetadata = ejbJarMetadata.getScannedClassMetadata(MyManagedBean.class.getName().replace(".", "/"));
    }

    /**
     * Remote the temp folder created at the end of the test.
     */
    @AfterClass
    public void remove() {
    }

    /**
     * Verify the validity of metadata archive.
     */
    @Test
    public void testMetadataValidity() {

        // EJB view
        IEJBClassView classView = ejb3StatelessJarClassMetadata.as(IEJBClassView.class);

        // EJB stateless
        Assert.assertTrue(classView.isStateless());
        Assert.assertFalse(classView.isStateful());
        Assert.assertFalse(classView.isSingleton());


        // EJB stateful
        IClassMetadata ejbJarClassStatefulMetadata = ejbJarMetadata
                .getBeanEjbJarClassMetadataForEjbName("VLA", MyEJB3Stateful.class.getName().replace('.', '/'));
        IEJBClassView statefulView = ejbJarClassStatefulMetadata.as(IEJBClassView.class);

        Assert.assertFalse(statefulView.isStateless());
        Assert.assertTrue(statefulView.isStateful());
        Assert.assertFalse(statefulView.isSingleton());

        // EJB singleton
        IClassMetadata ejbJarClassSingletonMetadata = ejbJarMetadata
                .getScannedClassMetadata(MyEJBSingleton.class.getName().replace('.', '/'));
        IEJBClassView singletonView = ejbJarClassSingletonMetadata.as(IEJBClassView.class);


        Assert.assertTrue(singletonView.isSingleton());
        Assert.assertFalse(singletonView.isStateless());
        Assert.assertFalse(singletonView.isStateful());
        Assert.assertTrue(singletonView.isStartup());
        Assert.assertEquals(IConcurrencyManagementType.CONTAINER, singletonView.getConcurrencyManagementType());
        Assert.assertEquals(ILockType.READ, singletonView.getLockType());
        JMethod myMethod = new JMethod(0, "myMethod", "()V", "", null);
        IMethodMetadata myMethodMetadata = ejbJarClassSingletonMetadata.getMethodMetadata(myMethod);
        IEJBMethodView myMethodView = myMethodMetadata.as(IEJBMethodView.class);
        Assert.assertEquals(ILockType.WRITE, myMethodView.getLockType());

        JMethod myOtherMethod = new JMethod(0, "myOtherMethod", "()V", "", null);

        IMethodMetadata myOtherMethodMetadata = ejbJarClassSingletonMetadata
                .getMethodMetadata(myOtherMethod);
        IEJBMethodView myOtherMethodView = myOtherMethodMetadata.as(IEJBMethodView.class);
        Assert.assertEquals(ILockType.READ, myOtherMethodView.getLockType());


        // Check the WS method
        Iterator<IMethodMetadata> itMethods = ejb3StatelessJarClassMetadata.getMethodMetadataCollection()
                .iterator();
        while (itMethods.hasNext()) {
            IMethodMetadata method = itMethods.next();
            if ("webserviceMethod".equals(method.getJMethod().getName())) {
                IEJBMethodView methodView = method.as(IEJBMethodView.class);
                IJavaxJwsWebMethod webMethod = methodView.getJavaxJwsWebMethod();
                Assert.assertNotNull(webMethod);
                Assert.assertEquals(webMethod.getExclude(), false);
                Assert.assertEquals(webMethod.getAction(), "actionNameFlorent");
                Assert.assertEquals(webMethod.getOperationName(), "opName2");
            }

        }

    }



    /**
     * Check that the classmetadata can be duplicated through cloneObject method.
     * @throws SerializationException if metadata can't be duplicated
     */
    @Test
    public void checkSerialization() throws SerializationException {

        // clone it
        IClassMetadata clonedClassMetadata = null;
        try {
            clonedClassMetadata = Serialization.cloneObject(ejb3StatelessJarClassMetadata);
        } catch (SerializationException e) {
            e.printStackTrace();
            throw e;
        }


        // Ensure that some string are here and equals after clone
        Assert.assertEquals(ejb3StatelessJarClassMetadata.getJClass().getInterfaces(), clonedClassMetadata.getJClass().getInterfaces());
        Assert.assertEquals(ejb3StatelessJarClassMetadata.getJClass(), clonedClassMetadata.getJClass());

        // Check that instance are different
        IEJBClassView originView = ejb3StatelessJarClassMetadata.as(IEJBClassView.class);
        IEJBClassView cloneView = clonedClassMetadata.as(IEJBClassView.class);


        Assert.assertFalse(originView.getEjbJarDeployableMetadata().equals(
                cloneView.getEjbJarDeployableMetadata()));


    }

    /**
     * Test that @AccessTimeout is correctly analyzed with annotations and DD
     */
    @Test
    public void checkAccessTimeout() {
        // EJB stateful
        IClassMetadata ejbJarClassStatefulMetadata = ejbJarMetadata
                .getBeanEjbJarClassMetadataForEjbName("VLA", MyEJB3Stateful.class.getName().replace('.', '/'));

        JMethod methodDD = new JMethod(Opcodes.ACC_PUBLIC, "methodAccessTimeoutInDD", "()V", null, null);

        IMethodMetadata methodMetadataDD = ejbJarClassStatefulMetadata.getMethodMetadata(methodDD);
        IEJBMethodView methodDDView = methodMetadataDD.as(IEJBMethodView.class);
        IJEjbAccessTimeout accessTimeoutDD = methodDDView.getJavaxEjbAccessTimeout();
        Assert.assertEquals(accessTimeoutDD.getValue(), 200);
        Assert.assertEquals(accessTimeoutDD.getUnit(), TimeUnit.MICROSECONDS);

        JMethod methodAnnotation = new JMethod(Opcodes.ACC_PUBLIC, "methodAccessTimeout", "()V", null, null);
        IMethodMetadata methodMetadataAnnotation = ejbJarClassStatefulMetadata.getMethodMetadata(methodAnnotation);
        IEJBMethodView methodAnnotationView = methodMetadataAnnotation.as(IEJBMethodView.class);
        IJEjbAccessTimeout accessTimeoutAnnotation = methodAnnotationView.getJavaxEjbAccessTimeout();
        Assert.assertEquals(accessTimeoutAnnotation.getValue(), 150);
        Assert.assertEquals(accessTimeoutAnnotation.getUnit(), TimeUnit.NANOSECONDS);

    }


    /**
     * Test that @Schedule is correctly analyzed with annotations
     */
    @Test
    public void checkSchedule1() {

        JMethod methodDD = new JMethod(Opcodes.ACC_PUBLIC, "dummyTimerMethod1", "()V", null, null);
        IMethodMetadata methodMetadataDD = ejb3StatelessJarClassMetadata.getMethodMetadata(methodDD);
        IEJBMethodView methodDDView = methodMetadataDD.as(IEJBMethodView.class);
        List<IJEjbSchedule> javaxEjbSchedules = methodDDView.getJavaxEjbSchedules();
        Assert.assertNotNull(javaxEjbSchedules);
        Assert.assertEquals(javaxEjbSchedules.size(), 1);

        IJEjbSchedule javaxEjbSchedule = javaxEjbSchedules.get(0);

        Assert.assertEquals(javaxEjbSchedule.getSecond(), "1");
        Assert.assertEquals(javaxEjbSchedule.getMinute(), "2");
        Assert.assertEquals(javaxEjbSchedule.getHour(), "3");
        Assert.assertEquals(javaxEjbSchedule.getDayOfMonth(), "4");
        Assert.assertEquals(javaxEjbSchedule.getMonth(), "5");
        Assert.assertEquals(javaxEjbSchedule.getDayOfWeek(), "6");
        Assert.assertEquals(javaxEjbSchedule.getYear(), "2012");
        Assert.assertEquals(javaxEjbSchedule.getInfo(), "infoTimer");
        Assert.assertFalse(javaxEjbSchedule.isPersistent());
        Assert.assertEquals(javaxEjbSchedule.getTimezone(), "CEST");

    }

    /**
     * Test that @Schedule is correctly analyzed with annotations
     */
    @Test
    public void checkSchedule2() {

        JMethod methodDD = new JMethod(Opcodes.ACC_PUBLIC, "dummyTimerMethod2", "()V", null, null);
        IMethodMetadata methodMetadataDD = ejb3StatelessJarClassMetadata.getMethodMetadata(methodDD);
        IEJBMethodView methodDDView = methodMetadataDD.as(IEJBMethodView.class);
        List<IJEjbSchedule> javaxEjbSchedules = methodDDView.getJavaxEjbSchedules();
        Assert.assertNotNull(javaxEjbSchedules);
        Assert.assertEquals(javaxEjbSchedules.size(), 1);

        IJEjbSchedule javaxEjbSchedule = javaxEjbSchedules.get(0);

        // @Schedule(second="50", persistent=true)
        Assert.assertEquals(javaxEjbSchedule.getSecond(), "50");
        Assert.assertTrue(javaxEjbSchedule.isPersistent());

        // Now, check defaults
        Assert.assertEquals(javaxEjbSchedule.getMinute(), "0");
        Assert.assertEquals(javaxEjbSchedule.getHour(), "0");
        Assert.assertEquals(javaxEjbSchedule.getDayOfMonth(), "*");
        Assert.assertEquals(javaxEjbSchedule.getMonth(), "*");
        Assert.assertEquals(javaxEjbSchedule.getDayOfWeek(), "*");
        Assert.assertEquals(javaxEjbSchedule.getYear(), "*");
        Assert.assertEquals(javaxEjbSchedule.getInfo(), "");
        Assert.assertEquals(javaxEjbSchedule.getTimezone(), "");

    }

    /**
     * Test that @Schedules is correctly analyzed with annotations
     */
    @Test
    public void checkSchedule3() {

        JMethod methodDD = new JMethod(Opcodes.ACC_PUBLIC, "dummyTimerMethod3", "()V", null, null);
        IMethodMetadata methodMetadataDD = ejb3StatelessJarClassMetadata.getMethodMetadata(methodDD);
        IEJBMethodView methodDDView = methodMetadataDD.as(IEJBMethodView.class);
        List<IJEjbSchedule> javaxEjbSchedules = methodDDView.getJavaxEjbSchedules();
        Assert.assertNotNull(javaxEjbSchedules);
        Assert.assertEquals(javaxEjbSchedules.size(), 3);


        /**
        @Schedules({
          @Schedule(second="10", minute="15", hour="25"),
          @Schedule(second="20", minute="25", hour="35"),
          @Schedule(second="30", minute="35", hour="45")
        })
        */

        IJEjbSchedule javaxEjbSchedule1 = javaxEjbSchedules.get(0);
        IJEjbSchedule javaxEjbSchedule2 = javaxEjbSchedules.get(1);
        IJEjbSchedule javaxEjbSchedule3 = javaxEjbSchedules.get(2);

        Assert.assertEquals(javaxEjbSchedule1.getSecond(), "10");
        Assert.assertEquals(javaxEjbSchedule1.getMinute(), "15");
        Assert.assertEquals(javaxEjbSchedule1.getHour(), "25");

        Assert.assertEquals(javaxEjbSchedule2.getSecond(), "20");
        Assert.assertEquals(javaxEjbSchedule2.getMinute(), "25");
        Assert.assertEquals(javaxEjbSchedule2.getHour(), "35");

        Assert.assertEquals(javaxEjbSchedule3.getSecond(), "30");
        Assert.assertEquals(javaxEjbSchedule3.getMinute(), "35");
        Assert.assertEquals(javaxEjbSchedule3.getHour(), "45");



    }

    /**
     * Test that @Schedules is correctly analyzed with annotations
     */
    @Test
    public void checkSchedule4() {

        JMethod methodDD = new JMethod(Opcodes.ACC_PUBLIC, "dummyTimerMethod4", "()V", null, null);
        IMethodMetadata methodMetadataDD = ejb3StatelessJarClassMetadata.getMethodMetadata(methodDD);
        IEJBMethodView methodDDView = methodMetadataDD.as(IEJBMethodView.class);
        List<IJEjbSchedule> javaxEjbSchedules = methodDDView.getJavaxEjbSchedules();
        Assert.assertNotNull(javaxEjbSchedules);
        Assert.assertEquals(javaxEjbSchedules.size(), 4);

        /**
        @Schedules({
          @Schedule(second="10"),
          @Schedule(second="20"),
          @Schedule(second="30")
        })
        @Schedule(second="40")
        */

        IJEjbSchedule javaxEjbSchedule1 = javaxEjbSchedules.get(0);
        IJEjbSchedule javaxEjbSchedule2 = javaxEjbSchedules.get(1);
        IJEjbSchedule javaxEjbSchedule3 = javaxEjbSchedules.get(2);
        IJEjbSchedule javaxEjbSchedule4 = javaxEjbSchedules.get(3);

        Assert.assertEquals(javaxEjbSchedule1.getSecond(), "10");
        Assert.assertEquals(javaxEjbSchedule2.getSecond(), "20");
        Assert.assertEquals(javaxEjbSchedule3.getSecond(), "30");
        Assert.assertEquals(javaxEjbSchedule4.getSecond(), "40");

    }



    /**
     * Test that old fashion timeout method is correctly analyzed
     */
    @Test
    public void checkOldFashionTimeoutMethodDD() {

        JMethod methodDD = new JMethod(Opcodes.ACC_PUBLIC, "myoldFashionTimeoutMethod", "()V", null, null);
        IMethodMetadata methodMetadataDD = ejb3StatelessDDJarClassMetadata.getMethodMetadata(methodDD);
        IEJBMethodView methodDDView = methodMetadataDD.as(IEJBMethodView.class);
        Assert.assertNotNull(methodMetadataDD);
        Assert.assertTrue(methodDDView.isTimeout());

    }

    /**
     * Test that schedule in DD is correctly analyzed
     */
    @Test
    public void checkScheduleDD1() {

        JMethod methodDD = new JMethod(Opcodes.ACC_PUBLIC, "simpleTimerMethod1", "()V", null, null);
        IMethodMetadata methodMetadataDD = ejb3StatelessDDJarClassMetadata.getMethodMetadata(methodDD);
        IEJBMethodView methodDDView = methodMetadataDD.as(IEJBMethodView.class);
        List<IJEjbSchedule> javaxEjbSchedules = methodDDView.getJavaxEjbSchedules();
        Assert.assertNotNull(javaxEjbSchedules);
        Assert.assertEquals(javaxEjbSchedules.size(), 1);

        IJEjbSchedule javaxEjbSchedule = javaxEjbSchedules.get(0);

        Assert.assertEquals(javaxEjbSchedule.getSecond(), "1");
        Assert.assertEquals(javaxEjbSchedule.getMinute(), "2");
        Assert.assertEquals(javaxEjbSchedule.getHour(), "3");
        Assert.assertEquals(javaxEjbSchedule.getDayOfMonth(), "4");
        Assert.assertEquals(javaxEjbSchedule.getMonth(), "5");
        Assert.assertEquals(javaxEjbSchedule.getDayOfWeek(), "6");
        Assert.assertEquals(javaxEjbSchedule.getYear(), "2012");
        Assert.assertEquals(javaxEjbSchedule.getInfo(), "infoTimerDD");
        Assert.assertFalse(javaxEjbSchedule.isPersistent());
        Assert.assertEquals(javaxEjbSchedule.getTimezone(), "CEST");

    }

    /**
     * Test that schedule in DD is correctly analyzed
     */
    @Test
    public void checkScheduleDD2() {

        JMethod methodDD = new JMethod(Opcodes.ACC_PUBLIC, "simpleTimerMethod2", "()V", null, null);
        IMethodMetadata methodMetadataDD = ejb3StatelessDDJarClassMetadata.getMethodMetadata(methodDD);
        IEJBMethodView methodDDView = methodMetadataDD.as(IEJBMethodView.class);
        List<IJEjbSchedule> javaxEjbSchedules = methodDDView.getJavaxEjbSchedules();
        Assert.assertNotNull(javaxEjbSchedules);
        Assert.assertEquals(javaxEjbSchedules.size(), 1);

        IJEjbSchedule javaxEjbSchedule = javaxEjbSchedules.get(0);

        Assert.assertEquals(javaxEjbSchedule.getSecond(), "50");
        Assert.assertEquals(javaxEjbSchedule.getMinute(), "0");
        Assert.assertEquals(javaxEjbSchedule.getHour(), "0");
        Assert.assertEquals(javaxEjbSchedule.getDayOfMonth(), "*");
        Assert.assertEquals(javaxEjbSchedule.getMonth(), "*");
        Assert.assertEquals(javaxEjbSchedule.getDayOfWeek(), "*");
        Assert.assertEquals(javaxEjbSchedule.getYear(), "*");
        Assert.assertEquals(javaxEjbSchedule.getInfo(), "");
        Assert.assertTrue(javaxEjbSchedule.isPersistent());
        Assert.assertEquals(javaxEjbSchedule.getTimezone(), "");

    }


    /**
     * Test that schedule in DD is correctly analyzed
     */
    @Test
    public void checkScheduleDD3() {

        JMethod methodDD = new JMethod(Opcodes.ACC_PUBLIC, "simpleTimerMethod3", "()V", null, null);
        IMethodMetadata methodMetadataDD = ejb3StatelessDDJarClassMetadata.getMethodMetadata(methodDD);
        IEJBMethodView methodDDView = methodMetadataDD.as(IEJBMethodView.class);
        List<IJEjbSchedule> javaxEjbSchedules = methodDDView.getJavaxEjbSchedules();
        Assert.assertNotNull(javaxEjbSchedules);
        // 3 schedules
        Assert.assertEquals(javaxEjbSchedules.size(), 3);

        IJEjbSchedule javaxEjbSchedule1 = javaxEjbSchedules.get(0);

        Assert.assertEquals(javaxEjbSchedule1.getSecond(), "10");
        Assert.assertEquals(javaxEjbSchedule1.getMinute(), "15");
        Assert.assertEquals(javaxEjbSchedule1.getHour(), "25");
        Assert.assertEquals(javaxEjbSchedule1.getDayOfMonth(), "*");
        Assert.assertEquals(javaxEjbSchedule1.getMonth(), "*");
        Assert.assertEquals(javaxEjbSchedule1.getDayOfWeek(), "*");
        Assert.assertEquals(javaxEjbSchedule1.getYear(), "*");
        Assert.assertEquals(javaxEjbSchedule1.getInfo(), "");
        Assert.assertTrue(javaxEjbSchedule1.isPersistent());
        Assert.assertEquals(javaxEjbSchedule1.getTimezone(), "");

        IJEjbSchedule javaxEjbSchedule2 = javaxEjbSchedules.get(1);
        Assert.assertEquals(javaxEjbSchedule2.getSecond(), "20");
        Assert.assertEquals(javaxEjbSchedule2.getMinute(), "25");
        Assert.assertEquals(javaxEjbSchedule2.getHour(), "35");

        IJEjbSchedule javaxEjbSchedule3 = javaxEjbSchedules.get(2);
        Assert.assertEquals(javaxEjbSchedule3.getSecond(), "30");
        Assert.assertEquals(javaxEjbSchedule3.getMinute(), "35");
        Assert.assertEquals(javaxEjbSchedule3.getHour(), "45");


    }

    /**
     *
     */
    @Test
    public void checkDataSource() {
        CommonView classView = ejb3StatelessDDJarClassMetadata.as(CommonView.class);

        Assert.assertNotNull(classView.getJAnnotationSqlDataSourceDefinitions());
        Assert.assertEquals(classView.getJAnnotationSqlDataSourceDefinitions().size(), 1);

        IJAnnotationSqlDataSourceDefinition sourceDefinition =
                classView.getJAnnotationSqlDataSourceDefinitions().get(0);

        //Test override by xml descriptor
        Assert.assertEquals(sourceDefinition.getClassName(), "org.foo.Bar");
        //Test merge from annotations
        Assert.assertEquals(sourceDefinition.getMaxIdleTime(), 10);
    }

    /**
     * Test that StatefulTimeout is correctly analyzed
     */
    @Test
    public void checkStatefulTimeout() {
        IEJBClassView classView = ejb3StatefulTimeoutJarClassMetadata.as(IEJBClassView.class);
        Assert.assertNotNull(classView.getJavaxEjbStatefulTimeout());
        IJEjbStatefulTimeout statefulTimeout = classView.getJavaxEjbStatefulTimeout();
        /*
        @StatefulTimeout(2)
         */
        Assert.assertEquals(statefulTimeout.getValue(), 2, "Stateful timeout value mismatch");
        Assert.assertEquals(statefulTimeout.getUnit(), TimeUnit.MINUTES, "Stateful timeout unit mismatch");

    }

    /**
     * Test that StatefulTimeout defined in ejb-jar.xml overrides annotations
     */
    @Test
    public void checkStatefulTimeoutOverride() {
        IEJBClassView classView = ejb3StatefulTimeoutOverrideJarClassMetadata.as(IEJBClassView.class);
        Assert.assertNotNull(classView.getJavaxEjbStatefulTimeout());
        IJEjbStatefulTimeout statefulTimeout = classView.getJavaxEjbStatefulTimeout();
        /*
        <stateful-timeout>
   <timeout>1</timeout>
   <unit>Hours</unit>
 </stateful-timeout>
        */
        Assert.assertEquals(statefulTimeout.getValue(), 1, "Stateful timeout override value mismatch");
        Assert.assertEquals(statefulTimeout.getUnit(), TimeUnit.HOURS, "Stateful timeout override unit mismatch");

    }

    @Test
    public void checkManagedBeanMetadata() {
        CommonView commonView = ejb3ManagedBeanJarClassMetadata.as(CommonView.class);

        Assert.assertTrue(commonView.isManagedBean());
        // Check that ManagedBean is parsed as an EJB
        Assert.assertTrue(ejbJarMetadata.getBeanNames().contains(commonView.getManagedBeanName()));

        Assert.assertNull(commonView.getAnnotationInterceptors());
        List<IMethodMetadata> aroundInvokes = ejb3ManagedBeanJarClassMetadata.searchMethodMetadata("aroundInvoke");
        Assert.assertEquals(aroundInvokes.size(), 1);
        IMethodMetadata aroundInvoke = aroundInvokes.get(0);
        IEJBMethodView aroundInvokeView = aroundInvoke.as(IEJBMethodView.class);
        Assert.assertTrue(aroundInvokeView.isAroundInvoke(), "aroundInvoke method is annotated with @AroundInvoke");

        List<IMethodMetadata> myMethods = ejb3ManagedBeanJarClassMetadata.searchMethodMetadata("myMethod");
        Assert.assertEquals(myMethods.size(), 1);
        IMethodMetadata myMethod = myMethods.get(0);
        CommonView myMethodView = myMethod.as(CommonView.class);
        IJInterceptors interceptors = myMethodView.getAnnotationInterceptors();
        Assert.assertNotNull(interceptors, "myMethod method is annotated with @Interceptors");
        Assert.assertEquals(interceptors.size(), 1);
        String interceptorClassName = interceptors.getClasses().get(0);
        Assert.assertEquals(interceptorClassName, "org/ow2/util/ee/metadata/ejbjar/test/MyInterceptor");
    }

}
