/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.ejbjar.test;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertNotNull;
import static org.testng.Assert.assertTrue;

import java.net.URL;
import java.util.List;

import org.ow2.util.ee.metadata.common.api.xml.struct.IEnvEntry;
import org.ow2.util.ee.metadata.common.impl.xml.parsing.ParsingException;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IEnterpriseBeans;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.ISession;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.parsing.EJB3DeploymentDescLoader;
import org.ow2.util.ee.metadata.ejbjar.impl.xml.struct.EJB3;
import org.ow2.util.xml.DocumentParserException;
import org.testng.annotations.Test;
import org.xml.sax.SAXException;

/**
 * Tests the parsing of an ejb-jar.xml file.
 * @author Florent Benoit
 */
public class TestLoading {

    /**
     * Gets an url for a given ejb-jar.xml filename.
     * @param fileName the name of the fiel to search.
     * @return an URL if found
     * @throws ParsingException if no URL is found.
     */
    private static URL getURL(final String fileName) throws ParsingException {
        URL url = TestLoading.class.getResource(fileName);
        if (url == null) {
            throw new ParsingException("URl for '" + fileName + "' was not found.");
        }
        return url;
    }

    /**
     * Parse a given filename and gets the resulting parsing.
     * @param fileName the name of the file to parse
     * @return a struct representing the EJB
     * @throws ParsingException if parsing has failed.
     */
    private EJB3 getEjb3(final String fileName) throws ParsingException {
        URL urlEjb01 = getURL(fileName);

        return EJB3DeploymentDescLoader.loadDeploymentDescriptor(urlEjb01);

    }

    /**
     * Checks that an xml file can be parsed by analyzing the result.
     * @throws ParsingException if xml is not parsed.
     */
    @Test
    public void analyzeParsedXml() throws ParsingException {
        EJB3 ejb3 = getEjb3("ejb-jar01.xml");

        // Ensure that there is a bean named TestBean.
        IEnterpriseBeans enterpriseBeans = ejb3.getEnterpriseBeans();
        assertNotNull(enterpriseBeans);

        // there are session beans
        List<ISession> sessionList = enterpriseBeans.getSessionList();
        assertNotNull(sessionList);
        assertTrue(sessionList.size() > 0);

        // first bean = TestBean
        ISession testBean = sessionList.get(0);
        assertNotNull(testBean);
        assertEquals("TestBean", testBean.getEjbName());

        // Env-Entry
        List<IEnvEntry> envEntries = testBean.getEnvEntryList();
        // there are envEntries
        assertNotNull(envEntries);
        assertTrue(envEntries.size() > 0);

        IEnvEntry envEntry = envEntries.get(0);
        assertEquals("testBoolean", envEntry.getEnvEntryName());
        assertEquals("java.lang.Boolean", envEntry.getEnvEntryType());
        assertEquals("false", envEntry.getEnvEntryValue());

    }


    /**
     * Checks that an xml file for stateful is correctly analyzed
     * @throws ParsingException if xml is not parsed.
     */
    @Test
    public void testStateful() throws ParsingException {
        EJB3 ejb3 = getEjb3("ejb-jar02.xml");

        // Ensure that there is a bean named TestStatefulBean.
        IEnterpriseBeans enterpriseBeans = ejb3.getEnterpriseBeans();
        assertNotNull(enterpriseBeans);

        // there are session beans
        List<ISession> sessionList = enterpriseBeans.getSessionList();
        assertNotNull(sessionList);
        assertTrue(sessionList.size() > 0);

        // first bean = TestStatefulBean
        ISession testStatefulBean = sessionList.get(0);
        assertNotNull(testStatefulBean);
        assertEquals("TestStatefulBean", testStatefulBean.getEjbName());

        assertNotNull(testStatefulBean.getAfterBeginMethod());
        assertEquals("testMethodName1", testStatefulBean.getAfterBeginMethod().getName());
        assertNotNull(testStatefulBean.getBeforeCompletionMethod());
        assertEquals("testMethodName2", testStatefulBean.getBeforeCompletionMethod().getName());
        assertNotNull(testStatefulBean.getAfterCompletionMethod());
        assertEquals("testMethodName3", testStatefulBean.getAfterCompletionMethod().getName());


    }
    /**
     * Checks that an xml file is invalid.
     */
    @Test
    public void error01() {
        // don't use here @ExpectedExceptions
        try {
            getEjb3("ejb-jar-error01.xml");
            assert false;
        } catch (ParsingException e) {
            Throwable t = e.getCause();
            assertTrue(t instanceof DocumentParserException);
            DocumentParserException dpe = (DocumentParserException) t;
            Throwable tt = dpe.getCause();
            assertTrue(tt instanceof SAXException);
            SAXException se = (SAXException) tt;
            // Check error message
            assertTrue(se.getMessage().contains("session"));
        }

    }

}
