/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.war.impl;

import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.ow2.util.archive.api.ArchiveException;
import org.ow2.util.archive.api.IArchive;
import org.ow2.util.archive.api.IArchiveManager;
import org.ow2.util.scan.api.IClassesLocator;
import org.ow2.util.scan.api.ScanException;
import org.ow2.util.scan.api.classlocator.ArchiveClassesLocator;

/**
 * This class is a temporary solution to solve war archive scan issue. Indeed an
 * instance of a class which implements ArchiveHelper should not be archive
 * dependent.
 * @author Gael Lalire
 */
public class WarClassesLocator implements IClassesLocator {

    /**
     * Class prefix.
     */
    private static final String CLASS_PREFIX = "WEB-INF/classes/";

    /**
     * Lib prefix.
     */
    private static final String LIB_PREFIX = "WEB-INF/lib/";

    /**
     * Lib extension.
     */
    private static final String LIB_SUFFIX = ".jar";

    /**
     * The archive Manager.
     */
    private final IArchiveManager archiveManager;

    /**
     * The war.
     */
    private final IArchive archive;

    /**
     * List of library in WEB-INF/lib.
     */
    private List<IClassesLocator> libClassesLocator;

    /**
     * Constructor.
     * @param archive the war
     */
    public WarClassesLocator(final IArchiveManager archiveManager, final IArchive archive) {
        this.archiveManager = archiveManager;
        this.archive = archive;
    }

    /**
     * Create the library list.
     * @throws ArchiveException if an error occurs
     */
    private void init() throws ArchiveException {
        if (libClassesLocator != null) {
            return;
        }
        synchronized (this) {
            libClassesLocator = new ArrayList<IClassesLocator>();
            Iterator<String> iterator = archive.getEntries();
            while (iterator.hasNext()) {
                String entry = iterator.next();
                if (entry.startsWith(LIB_PREFIX) && entry.endsWith(LIB_SUFFIX)) {
                    libClassesLocator.add(new ArchiveClassesLocator(archiveManager.getArchive(archive.getResource(entry))));
                }
            }
        }
    }

    /**
     * @param className a class name
     * @return url of className inside the archive
     * @throws ScanException if error occurs
     */
    public URL getClassURL(final String className) throws ScanException {
        try {
            init();
        } catch (ArchiveException e) {
            throw new ScanException("A lib of a war cannot be analyzed", e);
        }
        String classLocation = className.replace('.', '/').concat(".class");
        URL url;
        try {
            url = archive.getResource(CLASS_PREFIX + classLocation);
        } catch (ArchiveException e) {
            throw new ScanException("The war cannot be analyzed", e);
        }
        if (url == null) {
            // search in WEB-INF/lib libs
            for (IClassesLocator classesLocator : libClassesLocator) {
                url = classesLocator.getClassURL(className);
                if (url != null) {
                    break;
                }
            }
        }
        return url;
    }

}
