/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.war.impl;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.ow2.util.archive.api.ArchiveException;
import org.ow2.util.archive.api.IArchive;
import org.ow2.util.archive.api.IArchiveManager;
import org.ow2.util.ee.metadata.common.impl.xml.parsing.ParsingException;
import org.ow2.util.ee.metadata.war.api.IWarMetadata;
import org.ow2.util.ee.metadata.war.api.IWarMetadataFactory;
import org.ow2.util.ee.metadata.war.api.exceptions.WARDeploymentDescException;
import org.ow2.util.ee.metadata.war.api.exceptions.WARMetadataException;
import org.ow2.util.ee.metadata.war.impl.configurator.WarSessionConfigurator;
import org.ow2.util.ee.metadata.war.impl.xml.merge.WarMetadataMerge;
import org.ow2.util.ee.metadata.war.impl.xml.struct.Filter;
import org.ow2.util.ee.metadata.war.impl.xml.struct.Listener;
import org.ow2.util.ee.metadata.war.impl.xml.struct.Servlet;
import org.ow2.util.ee.metadata.war.impl.xml.struct.Tag;
import org.ow2.util.ee.metadata.war.impl.xml.struct.WAR;
import org.ow2.util.ee.metadata.war.impl.xml.struct.WARDeploymentDesc;
import org.ow2.util.ee.metadata.ws.api.view.IWebservicesView;
import org.ow2.util.ee.metadata.ws.api.xml.merge.IClassMetadataFinder;
import org.ow2.util.ee.metadata.ws.api.xml.struct.IWebservices;
import org.ow2.util.ee.metadata.ws.impl.xml.merge.WebservicesMetadataMerger;
import org.ow2.util.ee.metadata.ws.impl.xml.parsing.Webservices12DeploymentDesc;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.scan.api.IClassesLocator;
import org.ow2.util.scan.api.IScanner;
import org.ow2.util.scan.api.ScanException;
import org.ow2.util.scan.api.classlocator.ClassLoaderClassesLocator;
import org.ow2.util.scan.api.metadata.IClassMetadata;
import org.ow2.util.url.URLUtils;
import org.ow2.util.url.URLUtilsException;

/**
 */
public class WarMetadataFactory implements IWarMetadataFactory {

    /**
     * Logger.
     */
    private static final Log LOGGER = LogFactory.getLog(WarMetadataFactory.class);

    /**
     * Parsed classes, in a web-app are located in WEB-INF/classes.
     */
    private static final String WEBINF_CLASSES_PREFIX = "WEB-INF/classes/";

    private static final String CLASS_SUFFIX = ".class";

    private final IScanner scanner;

    private final IArchiveManager archiveManager;

    /**
     * Constructor with a scanner.
     * @param scanner the scanner which analyze class bytecode.
     */
    public WarMetadataFactory(final IScanner scanner, IArchiveManager archiveManager) {
        this.scanner = scanner;
        this.archiveManager = archiveManager;
    }

    /**
     * Create war metadata.
     * @param archive the war archive
     * @return the metadata
     * @throws WARMetadataException if error occurs
     */
    public IWarMetadata createArchiveMetadata(final IArchive archive) throws WARMetadataException {
        return createArchiveMetadata(archive, (ClassLoader) null);
    }

    /**
     * Create war metadata.
     * @param archive the war archive
     * @param classLoader a classloader to use if a class is not in archive
     * @return the metadata
     * @throws WARMetadataException if error occurs
     */
    public IWarMetadata createArchiveMetadata(final IArchive archive, final ClassLoader classLoader)
            throws WARMetadataException {
        List<? extends IClassesLocator> classesLocatorList = null;
        if (classLoader != null) {
            classesLocatorList = Arrays.asList(new ClassLoaderClassesLocator(classLoader));
        } else {
            classesLocatorList = Arrays.asList(new ClassLoaderClassesLocator(ClassLoader.getSystemClassLoader()));
        }
        return createArchiveMetadata(archive, classesLocatorList);
    }

    /**
     * Extract a class list from xml.
     * @param war a object representation of web.xml
     * @param classNameSet a set to fill with class name inside web.xml
     */
    private void completeClassList(final WAR war, final Set<String> classNameSet) {

        // Add the servlets
        for (Servlet servlet : war.getServlets()) {
            addClassToSetIfNotNull(servlet.getClassName(), classNameSet);
        }

        // Add the listeners
        for (Listener listener : war.getListeners()) {
            addClassToSetIfNotNull(listener.getListenerClassName(), classNameSet);
        }

        // Add the tags
        for (Tag tag : war.getTags()) {
            addClassToSetIfNotNull(tag.getTagClassName(), classNameSet);
        }

        // Add the filters
        for (Filter filter : war.getFilters()) {
            addClassToSetIfNotNull(filter.getFilterClassName(), classNameSet);
        }

    }

    /**
     * Create war metadata.
     * @param archive the war archive
     * @param warExternLibraries war extern libraries
     * @return the metadata
     */
    public IWarMetadata createArchiveMetadata(final IArchive archive,
            final List<? extends IClassesLocator> warExternLibraries)
            throws WARMetadataException {
        try {
            if (!URLUtils.urlToFile2(archive.getURL()).isDirectory()) {
                throw new WARMetadataException("war must be uncompressed (local directory expected)");
            }
        } catch (URLUtilsException e) {
            throw new WARMetadataException("war must be uncompressed (local directory expected)");
        } catch (ArchiveException e) {
            throw new WARMetadataException("war exception", e);
        }

        // for time debugging
        long tAnalyzeStart = System.currentTimeMillis();

        // Get WAR DD.
        final WAR war;
        try {
            war = WARDeploymentDesc.getWAR(archive);
        } catch (WARDeploymentDescException e) {
            throw new WARMetadataException(e);
        }

        // Get Webservices DD.
        IWebservices webservices = null;
        try {
            webservices = Webservices12DeploymentDesc.getWebservices12(archive, Webservices12DeploymentDesc.Type.WEBAPP);
        } catch (ParsingException e) {
            throw new WARMetadataException(e);
        }

        WarSessionConfigurator warSessionConfigurator = new WarSessionConfigurator(archive);
        Set<String> classNameSet = new HashSet<String>();


        // First step : extract classes from XML
        completeClassList(war, classNameSet);

        // if metadata complete then ignore standard annotation (like @resource)
        if (war.isMetadataComplete()) {
            warSessionConfigurator.setAnnotationParsingDesactived(true);
        } else {
            // enable annotation parsing because metadata descriptor is not
            // complete
            warSessionConfigurator.setAnnotationParsingDesactived(false);

            Iterator<String> entryIterator;
            try {
                entryIterator = archive.getEntries();
            } catch (ArchiveException e) {
                throw new WARMetadataException(e);
            }
            while (entryIterator.hasNext()) {
                String entry = entryIterator.next();
                if (entry.startsWith(WEBINF_CLASSES_PREFIX) && entry.endsWith(CLASS_SUFFIX)) {
                    classNameSet.add(entry.substring(WEBINF_CLASSES_PREFIX.length(),
                                                     entry.length() - CLASS_SUFFIX.length()));
                }
            }
        }

        // do annotation scan
        try {
            this.scanner.scan(classNameSet, new WarClassesLocator(archiveManager, archive), warExternLibraries, warSessionConfigurator);
        } catch (ScanException e) {
            throw new WARMetadataException("Unable to scan", e);
        }

        // Prepare merge
        final IWarMetadata warMetadata = warSessionConfigurator.getWarMetadata();

        // Merge with XML descriptor
        IWebservicesView wsView = warMetadata.as(IWebservicesView.class);
        wsView.setWebservices12(webservices);
        new WarMetadataMerge(warMetadata).merge(war);
        WebservicesMetadataMerger.merge(wsView, new IClassMetadataFinder() {

            /**
             *  Servlet Link is a link to the servlet name
             */
            public IClassMetadata findClassMetadata(final String link) {

                // When the matching servlet is found, return the metadata of the servlet
                List<Servlet> servlets = war.getServlets();
                for (Servlet servlet : servlets) {
                    if (link.equals(servlet.getName())) {
                        return warMetadata.getClassMetadata(servlet.getClassName().replace('.', '/'));
                    }
                }

                // not found
                return null;
            }
        });

        // time if debugging
        if (LOGGER.isDebugEnabled()) {
            long tAnalyzeStartEnd = System.currentTimeMillis();
            LOGGER.debug("Analyze of file {0} took {1} ms.", archive.getName(), (tAnalyzeStartEnd - tAnalyzeStart));
        }

        return warSessionConfigurator.getWarMetadata();
    }

    protected void addClassToSetIfNotNull(final String className, final Set<String> stringSet) {
        if (className != null) {
            stringSet.add(className.replace('.', '/'));
        }
    }

}
