/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.war.impl.xml.merge;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.ow2.util.ee.metadata.common.api.interfaces.IEnvironmentRefAccessor;
import org.ow2.util.ee.metadata.common.api.struct.IJAnnotationResource;
import org.ow2.util.ee.metadata.common.api.struct.IJAnnotationSqlDataSourceDefinition;
import org.ow2.util.ee.metadata.common.api.struct.IJavaxPersistenceUnit;
import org.ow2.util.ee.metadata.common.api.struct.IJaxwsWebServiceRef;
import org.ow2.util.ee.metadata.common.impl.view.CommonView;
import org.ow2.util.ee.metadata.common.impl.xml.merge.MetadataMerge;
import org.ow2.util.ee.metadata.war.api.IWarMetadata;
import org.ow2.util.ee.metadata.war.impl.xml.struct.WAR;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.scan.api.metadata.IClassMetadata;
import org.ow2.util.scan.api.metadata.IFieldMetadata;
import org.ow2.util.scan.api.metadata.IMethodMetadata;

/**
 * Merge the Web.xml Deployment descriptor with the annotations that have been parsed.
 * @author Florent Benoit
 */
public class WarMetadataMerge extends MetadataMerge {

    /**
     * Logger.
     */
    private static final Log LOGGER = LogFactory.getLog(WarMetadataMerge.class);

    /**
     * The metadata where merge will be done.
     */
    private final IWarMetadata warMetadata;

    /**
     * Constructor.
     * @param warDeployableMetadata the metadata where merge will be done
     */
    public WarMetadataMerge(final IWarMetadata warMetadata) {
        this.warMetadata = warMetadata;
    }

    /**
     * Merge current metadata with war.
     * @param war object representing the xml
     */
    public void merge(final WAR war) {
        IEnvironmentRefAccessor globalView = warMetadata.as(CommonView.class);

        // merge into classes
        for (IClassMetadata warClassmetadata : warMetadata.getClassMetadataCollection()) {

            IEnvironmentRefAccessor localClassView = warClassmetadata.as(CommonView.class);

            // Merge persistence units
            List<IJavaxPersistenceUnit> persistenceUnitsDeployable = globalView.getJavaxPersistencePersistenceUnits();
            if (persistenceUnitsDeployable != null && persistenceUnitsDeployable.size() > 0) {
                List<IJavaxPersistenceUnit> persistenceUnitsClass = localClassView.getJavaxPersistencePersistenceUnits();
                if (persistenceUnitsClass == null) {
                    localClassView.setJavaxPersistencePersistenceUnits(persistenceUnitsDeployable);
                } else {
                    persistenceUnitsClass.addAll(persistenceUnitsDeployable);
                }
            }

            // Merge @Resources
            List<IJAnnotationResource> annotationResourcesDeployable = globalView.getJAnnotationResources();
            if (annotationResourcesDeployable != null && annotationResourcesDeployable.size() > 0) {
                List<IJAnnotationResource> annotationResourcesClass = localClassView.getJAnnotationResources();
                if (annotationResourcesClass == null) {
                    localClassView.setJAnnotationResources(annotationResourcesDeployable);
                } else {
                    annotationResourcesClass.addAll(annotationResourcesDeployable);
                }
            }

            if (globalView.getJAnnotationSqlDataSourceDefinitions() == null) {
                globalView.setJAnnotationSqlDataSourceDefinitions(new ArrayList<IJAnnotationSqlDataSourceDefinition>());
            }

            List<IJAnnotationSqlDataSourceDefinition> annotationSqlDataSourceDefinitions =
                    localClassView.getJAnnotationSqlDataSourceDefinitions();
            if (annotationSqlDataSourceDefinitions != null) {
                for (IJAnnotationSqlDataSourceDefinition annotationSqlDataSourceDefinition :
                    annotationSqlDataSourceDefinitions) {
                    if (globalView.getJAnnotationSqlDataSourceDefinitions()
                                             .contains(annotationSqlDataSourceDefinition)) {
                        LOGGER.warn("Multiple definitions of DataSource {0}, this will lead to unpredicted results",
                                    annotationSqlDataSourceDefinition.getName());
                        //Leave the first one and continue
                        continue;
        }
                    //Move up the annotation to the warDeployableMetadata
                    globalView.getJAnnotationSqlDataSourceDefinitions()
                                         .add(annotationSqlDataSourceDefinition);
                }
            }

        }
        applyJndiEnvironmentRefsGroup(war, globalView);
    }

    /**
     * Needs to search the WebServiceRef in all fields/methods of the WAR.
     * @param warMetadata the war metadata
     * @param name the xml name of the service ref
     * @return a list of matching web service ref
     */
    @Override
    protected List<IJaxwsWebServiceRef> findJaxwsWebServiceRefMetadata(final IEnvironmentRefAccessor envRefAccessor,
            final String name) {


        List<IJaxwsWebServiceRef> elements = new ArrayList<IJaxwsWebServiceRef>();

        Collection<IClassMetadata> clientClassesMetadata = warMetadata.getClassMetadataCollection();
        if (clientClassesMetadata != null) {
            for (IClassMetadata classMetadata : clientClassesMetadata) {
                Collection<IFieldMetadata> fieldMetadatas = classMetadata.getFieldMetadataCollection();
                if (fieldMetadatas != null) {
                    for (IFieldMetadata fieldMetadata : fieldMetadatas) {
                        addJaxwsWebServiceRefIfFound(fieldMetadata, name, elements);
                    }
                }
                Collection<IMethodMetadata> methodMetadatas = classMetadata.getMethodMetadataCollection();
                if (methodMetadatas != null) {
                    for (IMethodMetadata methodMetadata : methodMetadatas) {
                        addJaxwsWebServiceRefIfFound(methodMetadata, name, elements);
                    }
                }
            }
        }
        return elements;
    }

    /**
     * @param className the class name
     * @return class metadata of class name
     */
    @Override
    public IClassMetadata getClassMetadata(final String className) {
        return warMetadata.getClassMetadata(className);
    }

    /**
     * No Main-Class for War archive.
     * @param env the car environment
     * @return null
     */
    @Override
    public String findDefaultClassForEnvironment(final IEnvironmentRefAccessor env) {
        return null;
    }

    @Override
    protected Collection<IClassMetadata> getAllClassMetadatas() {
        return warMetadata.getClassMetadataCollection();
    }

    @Override
    protected Collection<IMethodMetadata> getAllMethodMetadatas() {
        Collection<IMethodMetadata> methodMetadatas = new ArrayList<IMethodMetadata>();

        for (IClassMetadata classMetadata : getAllClassMetadatas()) {
            methodMetadatas.addAll(classMetadata.getMethodMetadataCollection());
        }
        return methodMetadatas;
    }

    @Override
    protected Collection<IFieldMetadata> getAllFieldMetadatas() {
        Collection<IFieldMetadata> fielddMetadatas = new ArrayList<IFieldMetadata>();

        for (IClassMetadata classMetadata : getAllClassMetadatas()) {
            fielddMetadatas.addAll(classMetadata.getFieldMetadataCollection());
        }
        return fielddMetadatas;
    }


}
