/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.war.impl.xml.parsing;

import java.net.URL;

import org.ow2.util.ee.metadata.common.impl.xml.parsing.CommonDeploymentDescLoader;
import org.ow2.util.ee.metadata.war.impl.xml.struct.Filter;
import org.ow2.util.ee.metadata.war.impl.xml.struct.Listener;
import org.ow2.util.ee.metadata.war.impl.xml.struct.Servlet;
import org.ow2.util.ee.metadata.war.impl.xml.struct.WAR;
import org.ow2.util.xml.XMLUtils;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Common stuff for parsing Web xml files (web.xml and .tld files).
 * @author Florent BENOIT
 */
public class AbsWebLoader extends CommonDeploymentDescLoader {

    /**
     * WAR instance object that is the result of the parsing.
     */
    private WAR war = null;

    /**
     * Name of element jsp-config
     */
     public static final String JSPCONFIGNAME = "jsp-config";

     /**
     * Name of element login-config
     */
     public static final String LOGINCONFIGNAME = "login-config";

    /**
     * Name of element session-config
     */
     public static final String SESSIONCONFIGNAME = "session-config";

       /**
     * Name of element url-pattern
     */
     public static final String URLPATTERNNAME = "url-pattern";

    /**
     * Constructor that creates an empty WAR object.
     * @param url the url that is pointing to the web xml file.
     */
    protected AbsWebLoader(final URL url) {
        this(url, new WAR());
    }

    /**
     * Constructor that use an existing WAR object.
     * @param url the url that is pointing to the web xml file.
     * @param war the given war object if any.
     */
    protected AbsWebLoader(final URL url, final WAR war) {
        super(url);
        this.war = war;
    }

    /**
     * Analyze the servlet.
     * @param servletElement the dom element.
     * @param servlet the structure representing object.
     */
    protected void analyzeServlet(final Element servletElement, final Servlet servlet) {
        // Get name
        String name = XMLUtils.getStringValueElement(servletElement, "servlet-name");
        servlet.setName(name);

        // Get class
        String className = XMLUtils.getStringValueElement(servletElement, "servlet-class");
        servlet.setClassName(className);
    }

    /**
     * Find and analyze the listeners in the given DOM element.
     * @param rootElement the element to analyze
     */
    protected void analyzeListeners(final Element rootElement) {
        // Listeners
        NodeList listenerList = rootElement.getElementsByTagName(Listener.NAME);
        for (int i = 0; i < listenerList.getLength(); i++) {
            Element listenerElement = (Element) listenerList.item(i);

            // Build instance of a Listener struct object
            Listener listener = new Listener();
            war.addListener(listener);

            // Analyze listener
            analyzeListener(listenerElement, listener);
        }
    }

    /**
     * Find and analyze the filters in the given DOM element.
     * @param rootElement the element to analyze
     */
    protected void analyzeFilters(final Element rootElement) {
        // Listeners
        NodeList filterList = rootElement.getElementsByTagName(Filter.NAME);
        for (int i = 0; i < filterList.getLength(); i++) {
            Element filterElement = (Element) filterList.item(i);

            // Build instance of a Filter struct object
            Filter filter = new Filter();
            war.addFilter(filter);

            // Analyze filter
            analyzeFilter(filterElement, filter);
        }
    }

    /**
     * Find and analyze the servlets in the given DOM element.
     * @param rootElement the element to analyze
     */
    protected void analyzeServlets(final Element rootElement) {
        // Servlets
        NodeList servletList = rootElement.getElementsByTagName(Servlet.NAME);
        for (int i = 0; i < servletList.getLength(); i++) {
            Element servletElement = (Element) servletList.item(i);

            // Build instance of Servlet XML struct object
            Servlet servlet = new Servlet();
            getWAR().addServlet(servlet);

            // Analyze servlet
            analyzeServlet(servletElement, servlet);
        }
    }

    /**
     * Analyze the given listener.
     * @param listenerElement the dom element.
     * @param listener the structure representing object.
     */
    protected void analyzeListener(final Element listenerElement, final Listener listener) {
        // Get class
        String listenerClassName = XMLUtils.getStringValueElement(listenerElement, "listener-class");
        listener.setListenerClassName(listenerClassName);
    }

    /**
     * Analyze the given filter.
     * @param filterElement the dom element.
     * @param filter the structure representing object.
     */
    protected void analyzeFilter(final Element filterElement, final Filter filter) {
        // Get class
        String filterClassName = XMLUtils.getStringValueElement(filterElement, "filter-class");
        filter.setFilterClassName(filterClassName);
    }

    /**
     * Verify there is no  multiple elements of jsp-config in the given DOM element.
     * @param webAppRootElement the element to analyze
     * @return  true if it is the case
     */
    protected boolean analyzeJspConfig(Element webAppRootElement) {
        NodeList servletList = webAppRootElement.getElementsByTagName(JSPCONFIGNAME);
        int nbelem = 0;
        for (int i = 0; i < servletList.getLength(); i++) {
            nbelem++;
            if ( nbelem > 1)
                    return false;
        }
        return true;
    }

     /**
     * Verify there is no  multiple elements of session-config in the given DOM element.
     * @param webAppRootElement the element to analyze
     * @return  true if it is the case
     */
    protected boolean analyzeSessionConfig(Element webAppRootElement) {
        NodeList elementList = webAppRootElement.getElementsByTagName(SESSIONCONFIGNAME);
        int nbelem = 0;
        for (int i = 0; i < elementList.getLength(); i++) {
            nbelem++;
            if ( nbelem > 1)
                    return false;
        }
        return true;
    }
     /**
     * Verify there is no  multiple elements of login-config in the given DOM element.
     * @param webAppRootElement the element to analyze
     * @return  true if it is the case
     */
    protected boolean analyzeLoginConfig(Element webAppRootElement) {
         NodeList elementList = webAppRootElement.getElementsByTagName(LOGINCONFIGNAME);
        int nbelem = 0;
        for (int i = 0; i < elementList.getLength(); i++) {
            nbelem++;
            if ( nbelem > 1)
                    return false;
        }
        return true;
    }

      /**
     * Verify there is no  new line carachter into an of url-pattern  element.
     * @param webAppRootElement the element to analyze
     * @return  true if it is the case
     */
    protected boolean analyzeUrlPattern(Element webAppRootElement) {
         NodeList elementList = webAppRootElement.getElementsByTagName(URLPATTERNNAME);
         for (int i = 0; i < elementList.getLength(); i++) {
             Element urlpatternElement = (Element) elementList.item(i);
             String pattern = urlpatternElement.getFirstChild().getNodeValue();
             if (pattern != null && pattern.indexOf('\n') != -1) {
              return false;
          }
         }

        return true;
    }

    /**
     * @return the object resulting of the XML parsing.
     */
    public WAR getWAR() {
        return war;
    }

}
