/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.war.impl.xml.parsing;

import java.net.URL;

import org.ow2.util.ee.metadata.common.impl.xml.parsing.ParsingException;
import org.ow2.util.ee.metadata.war.impl.xml.struct.WAR;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.xml.DocumentParser;
import org.ow2.util.xml.DocumentParserException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * This class analyzes the WAR deployment descriptor and fill a struct with
 * getters.
 *
 * @author Florent Benoit
 */
public class WARDeploymentDescLoader extends AbsWebLoader {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(WARDeploymentDescLoader.class);


    /**
     * Constructor.
     *
     * @param url the url that is pointing to the web xml file.
     */
    protected WARDeploymentDescLoader(final URL url) {
        super(url);
    }

    /**
     * Load the web.xml file.
     *
     * @param url the URL of the XML file.
     * @param validation true for forcong validation
     * @return an application object.
     * @throws ParsingException if parsing of XML file fails.
     */
    public static WAR loadDeploymentDescriptor(final URL url, boolean validation) throws ParsingException {
        logger.debug("Analyzing url {0}", url);

        WARDeploymentDescLoader warDeploymentDescLoader = new WARDeploymentDescLoader(url);
        warDeploymentDescLoader.setValidating(validation);
        warDeploymentDescLoader.parse();
        return warDeploymentDescLoader.getWAR();
    }

    /**
     * Analyze the URL and build an EJB3 object which contains data of the XML.
     *
     * @throws ParsingException if the analyze of the file fails.
     */
    public void parse() throws ParsingException {

        // No parsing without WEB-INF/web.xml file
        if (getURL() == null) {
            return;
        }
        
        // Get document
        Document document = null;
        try {
            document = DocumentParser.getDocument(getURL(), isValidating(), new WAREntityResolver());
        } catch (DocumentParserException e) {
            throw new ParsingException("Cannot parse the url", e);
        }

        // Root element = <web-app>
        Element webAppRootElement = document.getDocumentElement();

        if (webAppRootElement.getNamespaceURI() != null) {
            // Define NS
            defineXMLNS(webAppRootElement);
        }

        analyzeEnvironment(webAppRootElement, getWAR());

        // Find and analyze the servlets
        analyzeServlets(webAppRootElement);

        // and listeners
        analyzeListeners(webAppRootElement);

        // and filters
        analyzeFilters(webAppRootElement);


        String metadataComplete = webAppRootElement.getAttribute("metadata-complete");
        if (metadataComplete.equals("true")) {
            getWAR().setMetadataComplete(true);
        }
        if (isValidating()) {
            // vericonformity of some elements
            if (!analyzeJspConfig(webAppRootElement)) {
                throw new ParsingException(" web.xml must not contain multiple elements of jsp-config");
            }
            if (!analyzeSessionConfig(webAppRootElement)) {
                throw new ParsingException(" web.xml must not contain multiple elements of session-config");
            }
            if (!analyzeLoginConfig(webAppRootElement)) {
                throw new ParsingException(" web.xml must not contain multiple elements of login-config");
            }
            if (!analyzeUrlPattern(webAppRootElement)) {
                throw new ParsingException(" a '\\n' character inside the url pattern is forbidden");
            }
        }
    }


}
