/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.war.impl.xml.parsing;

import java.net.URL;

import org.ow2.util.ee.metadata.common.impl.xml.parsing.ParsingException;
import org.ow2.util.ee.metadata.war.impl.xml.struct.Tag;
import org.ow2.util.ee.metadata.war.impl.xml.struct.WAR;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.xml.DocumentParser;
import org.ow2.util.xml.DocumentParserException;
import org.ow2.util.xml.XMLUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * This class analyzes the TLD files and fill a given struct.
 * @author Florent Benoit
 */
public final class WARTLDLoader extends AbsWebLoader {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(WARTLDLoader.class);

    /**
     * Utility class, no public constructor.
     * @param url the url that is pointing to the tld file.
     * @param war the given war file that needs to be completed.
     */
    private WARTLDLoader(final URL url, final WAR war) {
        super(url, war);
    }

    /**
     * Load the tld file.
     * @param url the URL of the TLD file.
     * @param war the given war file that needs to be completed.
     * @throws ParsingException if parsing of XML file fails.
     * @return an application object.
     */
    public static WAR scanTLD(final URL url, final WAR war) throws ParsingException {
        logger.debug("Analyzing url {0}", url);
        WARTLDLoader warTldLoader = new WARTLDLoader(url, war);
        warTldLoader.parse();
        return warTldLoader.getWAR();
    }

    /**
     * Analyze the URL and build an EJB3 object which contains data of the XML.
     * @throws ParsingException if the analyze of the file fails.
     */
    public void parse() throws ParsingException {
        // Get document
        Document document = null;
        try {
            document = DocumentParser.getDocument(getURL(), false, null);
        } catch (DocumentParserException e) {
            throw new ParsingException("Cannot parse the url", e);
        }

        // Root element = <taglib>
        Element tagLibRootElement = document.getDocumentElement();

        // Analyze the taglib element
        analyzeListeners(tagLibRootElement);
        analyzeTags(tagLibRootElement);

    }

    /**
     * Find and analyze the tags in the given DOM element.
     * @param rootElement the element to analyze
     */
    protected void analyzeTags(final Element rootElement) {
        // Listeners
        NodeList tagList = rootElement.getElementsByTagName(Tag.NAME);
        for (int i = 0; i < tagList.getLength(); i++) {
            Element tagElement = (Element) tagList.item(i);

            // Build instance of a Tag struct object
            Tag tag = new Tag();
            getWAR().addTag(tag);

            // Analyze tag
            analyzeTag(tagElement, tag);
        }
    }


    /**
     * Analyze the given tag.
     * @param tagElement the dom element.
     * @param tag the structure representing object.
     */
    protected void analyzeTag(final Element tagElement, final Tag tag) {
        // Get class
        String tagClassName = XMLUtils.getStringValueElement(tagElement, "tag-class");
        tag.setTagClassName(tagClassName);
    }

}
