/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.war.impl.xml.struct;

import java.net.URL;
import java.util.Iterator;

import org.ow2.util.archive.api.ArchiveException;
import org.ow2.util.archive.api.IArchive;
import org.ow2.util.ee.metadata.common.impl.xml.parsing.ParsingException;
import org.ow2.util.ee.metadata.war.api.exceptions.WARDeploymentDescException;
import org.ow2.util.ee.metadata.war.impl.xml.parsing.WARDeploymentDescLoader;
import org.ow2.util.ee.metadata.war.impl.xml.parsing.WARTLDLoader;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * Utility class to get a WAR deployment descriptor.
 * @author Florent Benoit
 */
public final class WARDeploymentDesc {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(WARDeploymentDesc.class);

    /**
     * Utility class, no public constructor.
     */
    private WARDeploymentDesc() {

    }

    /**
     * Gets WAR for a given archive.
     * @param archive file/directory.
     * @throws WARDeploymentDescException if no xml is present.
     * @return WAR instance.
     */
    public static WAR getWAR(final IArchive archive) throws WARDeploymentDescException {

        // Get XML entry
        URL webXmlURL = null;
        try {
            webXmlURL = archive.getResource("WEB-INF/web.xml");
        } catch (ArchiveException e) {
            throw new WARDeploymentDescException("Cannot get resource 'WEB-INF/web.xml' on the archive '"
                    + archive.getName() + "'.");
        }

        WAR war = null;
        // Get WAR object (if url valid)
        if (webXmlURL != null) {
             try {
                 war = WARDeploymentDescLoader.loadDeploymentDescriptor(webXmlURL, false);
            } catch (ParsingException e) {
                throw new WARDeploymentDescException("Cannot parse URL '" + webXmlURL + "'.", e);
            }
        } else {
            war = new WAR();
        }

        // Check if there are .tld files
        Iterator<URL> itUrlResources = null;
        try {
            itUrlResources = archive.getResources();
        } catch (ArchiveException e) {
            throw new WARDeploymentDescException("Cannot get resources on the archive '"
                    + archive.getName() + "'.");
        }

        // Scan all URLs to search .tld
        while (itUrlResources.hasNext()) {
            URL url = itUrlResources.next();
            if (url.getPath().endsWith(".tld")) {
                // Complete the structure of the war object.
                try {
                    WARTLDLoader.scanTLD(url, war);
                } catch (ParsingException e) {
                    logger.error("Cannot analyze the TLD file ''{0}''", url, e);
                }
            }
        }



        return war;
    }
}
